"""
Dipolar ineraction using ewald summation
"Computer Simuation of Liquids, M.P.Allen and D.J.Tildesley", p. 160
"""

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


from collections import defaultdict
import numpy as np
import scipy
import scipy.special
from Lattice import Lattice

class EwaldDipolar(Lattice):
    def __init__(self,lt,musites,atpos):
        Lattice.__init__(self,lt)
        Ml2c=np.linalg.inv(self.lat2cart())
        musites_cart=[]
        for mu in musites:
            musites_cart.append(scipy.dot(Ml2c,mu))
        atpos_cart=[]
        for at in atpos:
            atpos_cart.append(scipy.dot(Ml2c,at))
        #distances between muon(s) and atoms in unit cell
        self.mu_atoms_bonds_dict=self.AtomicBonds(musites_cart,atpos_cart)

    def Ewald_BR(self,alfa,r):
        """
        B(r)=(erfc(alfa*r)+(2*alpha*r)/sqrt(pi)*exp(-alpha^2r^2))/r^3
        """
        erfc=scipy.special.erfc
        pi=scipy.pi
        sqrt=scipy.sqrt
        exp=scipy.exp
        Br1=erfc(alfa*r)
        Br2=(2*alfa*r/sqrt(pi))*exp(-alfa*alfa*r*r)
        return (Br1+Br2)/r/r/r

    def Ewald_CR(self,alfa,r):
        """
        C(r)=(3*erfc(alfa*r)+(2*alpha*r)/sqrt(pi)(3+2*alfa^2*r^2)*exp(-alpha^2r^2))/r^5
        """
        erfc=scipy.special.erfc
        pi=scipy.pi
        sqrt=scipy.sqrt
        exp=scipy.exp
        Br1=3.*erfc(alfa*r)
        Br2=(2*alfa*r/sqrt(pi))*(2.+2.*alfa*alfa*r*r)*exp(-alfa*alfa*r*r)
        return (Br1+Br2)/r/r/r/r/r

    def AtomicBonds(self,mulist,atlist):
        """
        bond vectors between muon sites and atomic positions 
        """
        bondslist=dict()
        for mu in mulist:
            tmp=[]
            for at in atlist:
                tmp.append(mu-at)
            bondslist[str(mu)]=tmp
        return bondslist

    def Ewald_UdipSelf(self,alfa,atoms_moment):
        """
        Dipolar interaction self part
        """
        mu_moment=np.array([0,0,1])
        kvec_cart=[]
        Mrec2cart=np.linalg.inv(self.recilat2cart())
        krange=scipy.arange(-3,3)
        for h in krange:
            for k in krange:
                for l in krange:
                    if h==0 and k==0 and l==0:
                        pass
                    else:
                        kvec=scipy.array([h,k,l])
                        kvec_cart.append(scipy.dot(Mrec2cart,kvec))
        kvec_cart=scipy.matrix(kvec_cart)
        kvec_cart_norm=np.linalg.norm(kvec_cart,axis=1)
        expart=scipy.exp(-kvec_cart_norm*kvec_cart_norm/4.0/alfa/alfa)
       
        print atoms_moment
        for musite, coord in self.mu_atoms_bonds_dict.iteritems():
            cospart=scipy.zeros((scipy.shape(kvec_cart_norm)[0],len(coord)))
            cospart=scipy.cos(scipy.dot(kvec_cart,scipy.matrix(coord).T))
        return
 

if __name__ == "__main__":
    lt=scipy.zeros(6)   
    lt[0]= 6.
    lt[1]= 6.
    lt[2]= 6.
    lt[3]= 90.
    lt[4]= 90.
    lt[5]= 90.

    at1=scipy.array([0.138,0.138,0.138])
    at2=scipy.array([0.362,0.862,0.638])
    at3=scipy.array([0.638,0.362,0.862])
    at4=scipy.array([0.862,0.638,0.362])
    atpos=[at1,at2,at3,at4]

    mom1=scipy.array([1.,0.,0.])
    mom2=scipy.array([0.,1.,0.])
    mom3=scipy.array([-1.,0.,0.])
    mom4=scipy.array([0.,-1.,0.])
    atmoment=[mom1,mom2,mom3,mom4]

    mu1=scipy.array([0.532,0.532,0.532])
    musites=[mu1]
    alfa=0.2
    ew=EwaldDipolar(lt,musites,atpos)
    ew.Ewald_UdipSelf(alfa,atmoment)

    r=scipy.array([0.1,0.2,0.4,0.6,0.8,1.0])
    print ew.Ewald_BR(alfa,r)
    print ew.Ewald_CR(alfa,r)


