#!/usr/bin/env python
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"



calculates metric tensor, d-spacing, transformation matrices to cartesian coordinate, etc
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

import numpy
import math

class Lattice():
    """ 
    calculates metric tensor, d-spacing, transformation matrices to cartesian coordinates
    input: a,b,c, alpha, beta, gamma as a list
    """
    
    def __init__(self, lt):
        """ 
        constructor 
        @type arg: numpy.array
        @param arg: lt[0]-[2]:a,b,c; lt[3]-[5]:alpha,beta,gamma  or 'filename' to read input from file 
        """
        
        self.lattice = lt
        self.cov_gij = numpy.array([[0,0,0],[0,0,0],[0,0,0]])
        self.cont_gij = numpy.array([[0,0,0],[0,0,0],[0,0,0]])

    def LatticeInfo(self):
        """
        return lattice parameters
        
        @return: text buffer containing lattice
        @rtype: instance of StringIO
        """

        output=StringIO.StringIO()
        output.write('lattice:\n')
        output.write('a='+str(self.lattice[0])+' b='+str(self.lattice[1])+' c='+str(self.lattice[2])+'\n')
        output.write('alpha='+str(self.lattice[3])+' beta='+str(self.lattice[4])+' gamma='+str(self.lattice[5])+'\n')
        return output
    
    def SetLattice(self,lt):
        """ 
        set lattice constants and angles
        @type lt: numpy.ndarray
        @param lt: a,b,c,alpha,beta,gamma
            e.g.
            lt[0]= 9.594644
            lt[1]= 9.594644
            lt[2]= 7.603696
            lt[3]= 90.
            lt[4]= 90.
            lt[5]= 120.
        @return: None
        """
        self.lattice=lt
        return

    def metrictensor(self):
        """ 
        calculate the metric tensor
        @return: None
        """
        a=self.lattice[0]
        b=self.lattice[1]
        c=self.lattice[2]
        alpha=self.lattice[3]/180.0*math.pi
        beta=self.lattice[4]/180.0*math.pi
        gamma=self.lattice[5]/180.0*math.pi
        self.cov_gij=numpy.array([[a**2, a*b*math.cos(gamma), a*c*math.cos(beta)],
                 [a*b*math.cos(gamma), b**2, b*c*math.cos(alpha)],
                 [a*c*math.cos(beta), b*c*math.cos(alpha),  c**2]])
        return

    def getGij(self):
        """ 
        @return: metric tensor 
        @rtype: numpy.ndarray
        """
        return self.cov_gij

    def contmetrictensor(self):
        """ 
        calculate the contravariant metric tensor 
        @return: None
        """
        self.cont_gij=numpy.linalg.inv(self.cov_gij)
        return

    def getIGij(self):
        """ 
        @return: contravariant metric tensor
        @rtype: numpy.ndarray 
        """
        return self.cont_gij

    def bond_distance(self,at1,at2):
        """ 
        calculate bond length between 2 atoms
        input: (relative ) atomic coordinates of at1 and at2 as arrays
        
        @type at1: float
        @param at1: atomic position
        @type at2: float
        @param at2: atomic postion
        @return: bond length
        @rtype: float 
        """
        dx=at1[0]-at2[0]
        dy=at1[1]-at2[1]
        dz=at1[2]-at2[2]
        alpha=self.lattice[3]/180.0*math.pi
        beta=self.lattice[4]/180.0*math.pi
        gamma=self.lattice[5]/180.0*math.pi
        d2=self.lattice[0]*self.lattice[0]*dx*dx+\
           self.lattice[1]*self.lattice[1]*dy*dy+\
           self.lattice[2]*self.lattice[2]*dz*dz+\
           2.*self.lattice[1]*self.lattice[2]*math.cos(alpha)*dy*dz+\
           2.*self.lattice[2]*self.lattice[0]*math.cos(beta)*dz*dx+\
           2.*self.lattice[0]*self.lattice[1]*math.cos(gamma)*dx*dy
        return math.sqrt(d2)

    def dspacing(self,v1):
        """ 
        @type v1: numpy.ndarray
        @param v1: HKL (AA)
        @return: d-spacing
        @rtype: float
        """
        try:
            dhkl = 1/math.sqrt(numpy.dot(v1.transpose(),numpy.dot(self.cont_gij,v1)))
        except ZeroDivisionError:
            dhkl=numpy.NINF
        return dhkl

    def unitcellvol(self):
        """ 
        @return: unit cell volume (AA^3)
        @rtype: 
        """
        self.metrictensor()
        try:
            V=math.sqrt(numpy.linalg.det(self.cov_gij))
        except ValueError:
            sys.exit('Check Lattice, Cell Volume undefined')
        return V
        
    def reciprocallattice(self):
        """ 
        calculate reciprocal lattice lengths and angles
        @return: return reciprocal moduli of lattice vectors and angles
        @rtype: dict
        """
        rldict = dict()
        a=self.lattice[0]
        b=self.lattice[1]
        c=self.lattice[2]
        alpha=self.lattice[3]/180.0*math.pi
        beta=self.lattice[4]/180.0*math.pi
        gamma=self.lattice[5]/180.0*math.pi
        V=self.unitcellvol()
        astar=b*c*math.sin(alpha)/V
        bstar=a*c*math.sin(beta)/V
        cstar=a*b*math.sin(gamma)/V
        csastar = (math.cos(beta)*math.cos(gamma) - math.cos(alpha))/(math.sin(beta)*math.sin(gamma))
        csbstar = (math.cos(alpha)*math.cos(gamma) - math.cos(beta))/(math.sin(alpha)*math.sin(gamma))
        cscstar = (math.cos(alpha)*math.cos(beta) - math.cos(gamma))/(math.sin(alpha)*math.sin(beta))
        rldict["astar"]=astar
        rldict["bstar"]=bstar
        rldict["cstar"]=cstar
        rldict["cosastar"]=csastar
        rldict["cosbstar"]=csbstar
        rldict["coscstar"]=cscstar
        return rldict

    def lat2cart(self):
        """ 
        transformation matrix to cartesian coordinates
        (real space); first axis is a
        @return: transformation matrix to cartesian coordinates
        @rtype: numpy.ndarray
        """
        a=self.lattice[0]
        b=self.lattice[1]
        c=self.lattice[2]
        calpha=math.cos(self.lattice[3]/180.0*math.pi)
        cbeta=math.cos(self.lattice[4]/180.0*math.pi)
        cgamma=math.cos(self.lattice[5]/180.0*math.pi)
        sgamma=math.sin(self.lattice[5]/180.0*math.pi)
        V=self.unitcellvol()
        M=numpy.array([[a, b*cgamma, c*cbeta],
                       [0, b*sgamma, c*(calpha-cbeta*cgamma)/sgamma],
                       [0, 0,        V/(a*b*sgamma)]])
        return M

    def recilat2cart(self):
        """ 
        @return: transformation matrix to cartesian coordinates for the reciprocal lattice
        @rtype: numpy.ndarray
        """
        M=self.lat2cart()
        acart=M[:,0]
        bcart=M[:,1]
        ccart=M[:,2]
        V=self.unitcellvol()
        astar=numpy.cross(bcart,ccart)/V
        bstar=numpy.cross(ccart,acart)/V
        cstar=numpy.cross(acart,bcart)/V
        return (numpy.array([astar,bstar,cstar])).transpose()

    def direct2dualbasis(self):
        """ 
        @return: reciprocal lattice vectors (AA^-1) 
        @rtype:  numpy.ndarray
        """
        self.metrictensor()
        self.contmetrictensor()
        ginv=self.cont_gij
        a=self.lattice[0]
        b=self.lattice[1]
        c=self.lattice[2]
        ginv.transpose()
        astar=2*math.pi*numpy.dot(numpy.array([a,0,0]),ginv)
        bstar=2*math.pi*numpy.dot(numpy.array([0,b,0]),ginv)
        cstar=2*math.pi*numpy.dot(numpy.array([0,0,c]),ginv)
        dual=numpy.array([astar, bstar, cstar])
        return dual.transpose()

    def realdotrec(self, q, v):
        """ 
        dot product between vectors in real (q) and reciprocal space (v)
        @param q: coordinates of q 
        @param v: coordinates of v 
        @return: dot product of q with v
        @rtype: float
        """
        return 2*math.pi*numpy.dot(q,v.transpose())

if __name__=="__main__":

    """ lattice parameters and angles """

    """Input can be either a lsit or a filename"""
    lt=numpy.zeros(6)
    lt[0]= 7.6
    lt[1]= 7.6
    lt[2]= 3.5
    lt[3]= 90.
    lt[4]= 90.
    lt[5]= 90.
    t=Lattice(lt)
    M=t.lat2cart()
    print 'transformation matrix to cartesian coordinates'
    print M
    rl=t.reciprocallattice()
    MR=t.recilat2cart()
    print 'transformation matrix to cartesian coordiantes, reciprocal space'
    print MR

