"""
calculate radial part of neutron form factor using 
PJ Brown approximations for <j0>,...,<j6>
"""

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


from collections import defaultdict
import numpy
import pylab
import sys
import os

class RadialFormFactor():
    def __init__(self,ion=''):
        """
        constructor
        
        @type ion: string
        @param ion: ion e.g fe2, cu1, cu2, ...
        """
        #dictionary to store the coefficients of <j_n (K)>
        """ Import radial part coefficients of <j_n(K)> from pickle file """
        import pickle
        full_path=os.path.realpath(__file__)
        path,filename=os.path.split(full_path)
        filename=path+'/FormFactor_DB.pkl'
        try:
            pkl_file = open(filename)
        except IOError:
            sys.exit("Requires FormFactor_DB.pkl")
        self.radialcoefficients = pickle.load(pkl_file)
        pkl_file.close()
        if len(ion)!=0:
            self.GetRadialCoefficients(ion)

    def GetRadialCoefficients(self,ion):
        """
        get formfactor (radial) coefficients of ion

        @type ion: string
        @param ion: ion label
        """
        self.ion = ion
        if len(self.radialcoefficients[self.ion]) == 0:
            del self.radialcoefficients[self.ion]
            for key in sorted(self.radialcoefficients.iterkeys()):
                print key
            print "Form Factor for ion " + self.ion + " not found"
            sys.exit(2)
        self.FFcoef = self.radialcoefficients[self.ion]
        
    def J0q(self,QQ):
        """ 
        calculate radial form of form factor <JQ0> as aproximated by P.J. Brown
        
        @type QQ: float
        @param QQ:
        
        @return: radial form factor <j0>
        @rtype: float
        """
        """
        # Neutron Magnetic Form Factor coefficients - thanks to J Brown
        #   d = 2*pi/Q      
        #   s = 1/2/d = Q/4/pi   
        #   sin(theta) = lambda * s
        #   QQ= s*s = Q*Q/16/pi/pi
        """

        A=self.FFcoef[0][0]['j0A']
        a=self.FFcoef[0][0]['j0a']
        B=self.FFcoef[0][0]['j0B']
        b=self.FFcoef[0][0]['j0b']
        C=self.FFcoef[0][0]['j0C']
        c=self.FFcoef[0][0]['j0c']
        D=self.FFcoef[0][0]['j0D']
        j0q=A*numpy.exp(-a*QQ)+B*numpy.exp(-b*QQ)+C*numpy.exp(-c*QQ)+D
        return j0q

    def Jnq(self,QQ, n):
        """ 
        calculate radial form of form factor <JQn> (n=2,4,6) as aproximated by P.J. Brown

        @type QQ: float
        @param QQ: 
        @type n: integer
        @param n: 2,4 or 6
        
        @return: radial form factor <jn> for n=2,4 or 6
        @rtype: float
        """
        if n not in (2,4,6):
            sys.exit('order of <j_n(k)> function should be 2,4 or 6')
        key = 'j'+str(n)+'A'
        l=n/2
        try:
            A=self.FFcoef[l][0][key]
        except IndexError:
            print 'Coefficient ', key, 'not found in table for ion:', self.ion
            sys.exit()
        key = 'j'+str(n)+'a'
        a=self.FFcoef[l][0][key]
        key = 'j'+str(n)+'B'
        B=self.FFcoef[l][0][key]
        key = 'j'+str(n)+'b'
        b=self.FFcoef[l][0][key]
        key = 'j'+str(n)+'C'
        C=self.FFcoef[l][0][key]
        key = 'j'+str(n)+'c'
        c=self.FFcoef[l][0][key]
        key = 'j'+str(n)+'D'
        D=self.FFcoef[l][0][key]
        jnq=A*QQ*numpy.exp(-a*QQ)+B*QQ*numpy.exp(-b*QQ)+C*QQ*numpy.exp(-c*QQ)+D*QQ
        #print n, A,a,B,b,C,c,D
        return jnq

if __name__ == "__main__":
    """ Calculate radial part of formfactor <j_n(k)> using
    PJ Brown coefficients """

    ion='Er3'
    fq=RadialFormFactor()
    fq.GetRadialCoefficients(ion)
    import numpy as np
    q=np.arange(0,10,0.01)
    QQ=q*q
    QQ=QQ/numpy.power(4.*numpy.pi,2)
    F0=[]
    F2=[]
    F4=[]
    F6=[]
    for Q in QQ:
        F0.append(fq.J0q(Q))
        F2.append(fq.Jnq(Q,2))
        F4.append(fq.Jnq(Q,4))
    q/=numpy.pi*4.0
    pylab.plot(q,F0,label=r'$<j_0(Q)>$',color='red')
    pylab.plot(q,F2,label=r'$<j_2(Q)>$',color='blue')
    pylab.plot(q,F4,label=r'$<j_4(Q)>$',color='green')
    g0=np.array(F0)+5./4*np.array(F4)
    pylab.plot(q,g0,label=r'$<g_0(Q)>$',color='yellow')
    pylab.legend(loc='upper right')
    pylab.title(ion+' Magnetic Formfactor')
    pylab.xlabel('$\sin(\Theta/\lambda) (\AA^{-1})$',fontsize=14)
    pylab.show()

