#!/usr/bin/env python
"""
Calculate basis vectors of magnetic representation using projection method
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

from VectorRepresentation import VectorRep
try:
    from prettytable import *
except ImportError:
    sys.exit('Module Pretty Table could not be imported')
        
        
import copy
import numpy as np
from collections import OrderedDict

NVec=dict()

class Found(Exception): pass

class BasisVectors(VectorRep):
    def __init__(self,spacegrouplabel,kvector,atomcoord,axialvector):
        np.set_printoptions(linewidth=400)
        VectorRep.__init__(self, spacegrouplabel,kvector,atomcoord,axialvector)
        #projection operator
        self.Pnkl=dict()
        #basis vectors
        self.Vectors=dict()

    def ProjectionOperator(self,magneticrepresentation):
        """
        Projection operator

        @type magneticrepresentation:   dictionary
        @param magneticrepresentation:  coefficient of the decomposition of the magnetic representation
        """
        self.Pnkl.clear()
        dim=np.shape(magneticrepresentation['{1|(0.0,0.0,0.0)}'])
        hing=float(len(magneticrepresentation.keys()))
        for irrep, val in self.gkirreduciblerepresentations.iteritems():
            ln=np.shape(self.gkirreduciblerepresentations[irrep]['{1|(0.0,0.0,0.0)}'])[0]  #dimension of irrep
            #factor=float(ln)/float(hing)
            factor=1.
            pkl=dict()
            for k in range(0,ln):
                for l in range(0,ln):
                    tmp=np.array(np.zeros(dim),dtype='complex')
                    for op, G in val.iteritems():
                        tmp=tmp+np.conjugate(G[k,l])*magneticrepresentation[op]
                    pkl[(k,l)]=np.dot(factor,tmp)
            self.Pnkl[irrep]=pkl
        return

    def CalcBasis(self,coefmagrep):
        """
        Use projection operator (P_kk) to find basis function and
        shif operator (P_kl) for the orthogonal functions
        if P_kk f != 0 for some k, then P_tk f form a basis (t = 1...dim)
        (Bradely and Cracknell, theorem 2.2.1)

        @type coefmagrep:   dictionary
        @param coefmagnrep: coefficient of the decomposition of the magnetic representation
        """
        #find function so that P_{kk}phi != 0
        BV=dict()
        for G, pkl in self.Pnkl.iteritems():
            kl = pkl.keys()                 #keys are tuples with (k,l)-indices
            dimmagrep=np.shape(pkl[kl[0]])  #dimension of the irreducible representtion G
            phisave=dict()
            try:
                for ind in kl:              #P_{kk}
                    if ind[0] == ind[1]:
                        for m in range(0,dimmagrep[0]):
                            phi=np.zeros((dimmagrep[0],1),dtype='complex')
                            phi[m]=1.0
                            bv=np.dot(self.Pnkl[G][ind],phi)
                            if np.allclose(bv,0) == False:
                                phisave[m]=ind
                                #raise Found
                            if len(phisave) == coefmagrep[G]:
                                raise Found   #first find non-zeros basis functions equal to number of times irrep appears in the decomposition
                        else:
                            pass
            except:
                pass
            BV[G]=phisave
        #use shiftor to complete basis functions for each G
        #P_{kl}phi = phi_k, k= 1..d
        dimmagrep=np.shape(pkl[kl[0]])  #dimension of the magnetic representation
        for G, bvs in BV.iteritems():
            basisvectors=dict()
            for m, ind in BV[G].iteritems():
                tmpvec=[]
                phi=np.zeros((dimmagrep[0],1),dtype='complex')
                phi[m]=1.0
                for kl in self.Pnkl[G].keys():
                    if kl[1] != ind[1]:
                        pass
                    else:
                        bv=np.dot(self.Pnkl[G][kl],phi)
                        tmpvec.append(bv)
                basisvectors[m]=tmpvec
            self.Vectors[G]=basisvectors
        return


    def CalcBasisFunctionsWithDiagonalisation(self,coefmagrep):
        """ 
        diagonalize projection matrix to find basis vectors
        (S. Flodmar, PRB, 132, 1343 (1963))
        (M. El-Batanouny, F. Wooten, Chapter 6)
        Does not work for co-representations as 
        P matrix is not hermitian
        @type coefmagrep:   dictionary
        @param coefmagnrep: coefficient of the decomposition of the magnetic representation
        """
        #find function so that P_{kk}phi != 0
        BV=dict()
        from scipy import linalg  as sl
        for G, pkl in self.Pnkl.iteritems():
            kl = pkl.keys()
            basisvectors=dict()
            for ind in kl:
                #P_{kk}
                if ind[0] == ind[1]:
                    M=self.Pnkl[G][ind]
                    W,v = np.linalg.eig(M)
                    eigvals=list(np.where(np.rint(W)==0.)[0])
                    v=np.delete(v,eigvals,1)                         #keep eigenvectors with eigenvalue 1
                    if np.shape(v)[1] != 0:
                        Q, R = np.linalg.qr(v)                       #make vectors orthogonal
                        idx = np.abs(Q).argmax(axis=0)
                        maxn = Q[idx,np.arange(Q.shape[1])]
                        EV=np.round(Q/(maxn),2)                      #normalise
                        tmpvec = []
                        for column in EV.T:
                            tmpvec.append(column)
                        basisvectors[ind]=tmpvec
                    else:
                        pass
            self.Vectors[G]=basisvectors 
        return


    def PrintBasisVectors(self):
        """
        print basis vectors
        """
        txt = '\nBasis vectors (S^1_x, S^1_y, S^1_z, ....,S^n_x,S^n_y,S^n_z) + (cc):'
        print txt
        print '-'*len(txt)

        irreps=sorted(self.Vectors.keys())
        for G in irreps:
            bvdict=self.Vectors[G]
            if len(bvdict) ==0:
                pass
            else:
                print
                print 'Irreducible representation:', G
                print
                k=1
                for val, orderparam in bvdict.iteritems():
                    print 'Basis vector: ',k
                    for eta in orderparam:
                        eta=np.transpose(eta)
                        eta=np.around(eta,2)+0.0
                        print('\n'.join([' '.join(['{:14}'.format(np.around(item,2)) for item in row])
                            for row in eta]))
                    k=k+1
                    print
                print
        return self.Vectors

    def GetModes(self):
        return self.Vectors
        
    import itertools
    def basisvectors2matrix(self,orbit):
        """
        save basisvectors to a file as a matrix for testing purposes ('test_Basisvectors.py')
        """
        irreps=sorted(self.Vectors.keys())
        tmplist=[]
        for G in irreps:
            bvdict=self.Vectors[G]
            if len(bvdict) ==0:
                pass
            else:
                for val, orderparam in bvdict.iteritems():
                    for eta in orderparam:
                        eta=np.around(eta,2)+0.0
                        tmplist.append(eta)
        x=np.zeros((len(tmplist),len(tmplist[0])),dtype='complex')
        for k in range(0,len(tmplist)):
            x[k,:]=np.transpose(np.array(tmplist[k]))
        y=np.around(x,2)
        filename = 'savebv' + str(orbit)
        #with open(filename,'w') as f:
        #  print y >> f
        #np.savetxt(filename,np.column_stack([y]).reshape(-1,len(x[1])),fmt='%.2f',delimiter=',')
        #np.savetxt(filename,y.view(float).reshape(-1, len(x[1])),fmt='%.2f',delimiter=',')
        np.savetxt(filename,y.view(complex),fmt='%.2f',delimiter=',')
        return


if __name__=="__main__":

    #title = '-P 4, k = [0.,0.,0.15]'
    #spacegroup='-P 4'
    #atomicposition=[0.,0.,0.]
    #kvector=[0,0,0.15]
    
    #title = 'CeAl2'
    #spacegroup = 'F d -3 m:2'
    #kvector=[0.45,0.55,0.50]
    #atomicposition=[1./8.,1./8.,1./8.]
    #kvector=[0.45,0.55,0.50]

    #title='test'
    #spacegroup = 'Pbca'
    #atomicposition=[0.001,0.1451,0.2248]
    #kvector = [0.5,0,0]
    
    #title='CePdAl'
    #spacegroup = 'P -6 2 m'
    #atomicposition=[0.58,0,0.]
    #kvector = [0.5,0.,0.35]

    #title='ErI3'
    #spacegroup = 'R -3:H'
    #atomicposition=[0,0,0.15]
    #kvector = [0.33333,0.33333,0.]

    title = 'MnSi, k = [0.1,0.1,0.]'
    spacegroup='P213'
    #atomicposition=[0.886,0.886,0.886]
    #atomicposition=[0.145,0.127,0.137]
    atomicposition=[0.138,0.138,0.138]
    kvector=[0.1,0.1,0.]

    #title = 'test, k = [1.,0.,0.]'
    #spacegroup='I4/m'
    #atomicposition=[0.886,0.886,0.886]
    #atomicposition=[0.145,0.127,0.137]
    #atomicposition=[0.,0.,0.5]
    #kvector=[1.0,0.0,0]

    #title = 'Jarosite, k = [0,0.,1.5]'
    #spacegroup='R-3m:H'
    #Batomicposition=[0.5,0.,0.5]
    #kvector=[0.0,0.0,1.5]

    #title = 'PrPtAl, k = [0,0.,kz]'
    #spacegroup='Pnma'
    #atomicposition=[0.1,0.25,0.2]
    #kvector=[0.0,0.0,0.1]

    #title = 'Test, k = [0,0.,0]'
    #spacegroup='Pnma'
    #atomicposition=[0.1,0.25,0.2]
    #kvector=[0.,0,0.1]

    #P-4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    #title = 'P -4, k = [0,0.,0.15]'
    #spacegroup = 'P -4'
    #kvector = [0,0,0.15]
    #atomicposition=[0,0,0]

    #Pbcn, k = (0.5,0.5,0.5)
    #title = 'Pbcn, k = [0,5.,0.5,0.5]'
    #spacegroup = 'Pbcn'
    #kvector = [0.5,0.5,0.5]
    #atomicposition=[0,0,0]

    #Pn a 21, k = (0.,0.,0.)
    #title = 'P n a 21, k = [0.,0,0]'
    #spacegroup = 'Pn a 21'
    #kvector = [0,0,0]
    #atomicposition=[0.00,0.00,0.25]

    #F d -3 m:2, k = (0.,0.,0.)
    #title = 'F d -3 m:2, k = [0.,0,0]'
    #spacegroup = 'F d -3 m:2'
    #kvector = [0,0,0]
    #atomicposition=[0.5,0.5,0.5]

    #P/4 m m, k=[0.5,0.5,0]
    #title = 'YBACuO'
    #spacegroup = 'P 4/m m m'
    #kvector=[0.5,0.5,0]
    #atomicposition=[0,0,0.162]
    
    #TEST PHONONS
    #title = 'NaCl, phonons'
    #spacegroup='P m -3 m'
    #spacegroup='P 4 m m'
    #spacegroup='F m -3 m'
    #kvector=np.array([0.1,0.,0.])
    #atomicposition=[0.,0.,0.]

    #title = 'P21/b, k=(0.15,0,0), position = [0,0,0]'
    #spacegroup = 'P 21/b 1 1'
    #kvector = [0.15,0,0]
    #atomicposition=[0.1,0.1,0.25]

    #title = 'F d -3 m:2, k=(0,0,0), position = [0.5,0.5,0.5]'
    #spacegroup = 'F d -3 m:2'
    #kvector = [0.,0,0]
    #atomicposition=[0.5,0.5,0.5]



    #axial (magnetic)/polar (phonon) vector (modes)                   
    axialvector = True

    #DO NOT MODIFY THE FOLLOWING CODE
    #--------------------------------   
    title = title + ' and atomic position ' + str(atomicposition)
    print title 
    print '-'*len(title)
    
    BV=BasisVectors(spacegroup,kvector,np.array(atomicposition),axialvector)
    orbits=BV.SplitOrbits()
    #return coefficients of the decomposition of the magnetic/mechanical representation
    if axialvector == False:
        coefrep=BV.CalcMechanicalRep(orbits)
    else:
        coefrep=BV.CalcMagneticRep(orbits)
    orbitnumber=0
    #loop over crystallographic orbits
    for orbit, positions in orbits.iteritems():
        print '\nOrbit:',orbitnumber+1
        print 'Atomic positions:\n', ('\n'.join([' '.join(['{:10}'.format(item) for item in row])
                              for row in positions]))
        #projection operator
        BV.ProjectionOperator(BV.allrepresentations[orbitnumber])
        #basis functions
        BV.CalcBasis(coefrep[orbit])
        BV.PrintBasisVectors()
        orbitnumber+=1
        #uncomment to save to file as a matrix
        #BV.basisvectors2matrix(orbitnumber)
        BV.Vectors={}

  