#!/usr/bin/env python
"""
Decode Hall symbols and return list of symmetry operations
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

import cmath
from collections import defaultdict
import numpy as np
import sys

class BravaisLattices(object):
    def __init__(self,cell):
        """
        Bravais lattices in cartesian coordinates
        """
        a=cell[0]
        b=cell[1]
        c=cell[2]
        alfa=cell[3]
        beta=cell[4]
        gama=cell[5]
        self.bravaistype=cell[6]
        self.bravais=defaultdict(dict)
        pi=np.pi
        sin=np.sin
        cos=np.cos
        alfa=alfa/180.*pi
        beta=beta/180.*pi
        gama=gama/180.*pi
        #primitive cell
        #from Bradley and Cracknell p.82 and 83
        self.bravais['aP']={'Type':'Triclinic primitive',
                       'basis vectors':None}

        self.bravais['mP']={'Type':'Monoclinic primitive',
                       'basis vectors':np.array([[0,-b,0],[a*sin(gama),-a*cos(gama),0],[0,0,c]])}
        self.bravais['mB']={'Type':'Monoclinic base-centred',
                       'basis vectors':np.array([[0,-b,0],[a/2.*sin(gama),-a/2.*cos(gama),-c/2.],[a/2.*sin(gama),-a/2.*cos(gama),c/2.]])}
        self.bravais['mC']={'Type':'Monoclinic base-centred',
                       'basis vectors':np.array([[0,0,-c],[a/2.*sin(beta),-a/2.*cos(beta),-b/2.],[a/2.*sin(beta),-a/2.*sin(beta),b/2.]])}
        self.bravais['oP']={'Type:':'Orthorhombic primitive',
                       'basis vectors':np.array([[0,-b,0],[a,0,0],[0,0,c]])}
        self.bravais['oA']={'Type':'Orthorhombic base-centred',
                       'basis vectors':np.array([[0.,-b/2.,c/2.],[a,0,0],[0,b/2.,c/2.]])}
        self.bravais['oB']={'Type':'Orthorhombic base-centred',
                       'basis vectors':np.array([[-a/2.,0,c/2.],[0,b,0],[a/2.,0,c/2.]])}
        self.bravais['oC']={'Type':'Orthorhombic base-centred',
                       'basis vectors':np.array([[a/2.,-b/2.,0],[a/2.,b/2.,0],[0,0,c]])}
        self.bravais['oI']={'Type':'Orthorhombic body-centred',
                       'basis vectors':np.array([[a/2.,b/2.,c/2.],[-a/2.,-b/2.,c/2.],[a/2.,-b/2.,-c/2.]])}
        self.bravais['oF']={'Type':'Orthorhombic face-centred',
                       'basis vectors':np.array([[a/2.,0,c/2.],[0,-b/2.,c/2.],[a/2.,-b/2.,0]])}
        self.bravais['tP']={'Type':'Tetragonal primitive',
                       'basis vectors':np.array([[a,0,0],[0,a,0],[0,0,c]])}
        self.bravais['tI']={'Type':'Tetragonal body-centred',
                       'basis vectors':np.array([[-a/2.,a/2.,c/2.],[a/2.,-a/2.,c/2.],[a/2.,a/2.,-c/2.]])}
        self.bravais['hR']={'Type':'Trigonal primitive (rhombohedral)',
                       'basis vectors':np.array([[0,-a,c],[a*cmath.sqrt(3.)/2.,a/2.,c],[-a*cmath.sqrt(3)/2.,a/2.,c]])}
        #Cracknell
        self.bravais['hP']={'Type':'Hexagonal primitive',
                       'basis vectors':np.array([[0,-a,0],[cmath.sqrt(3)*a/2.,a/2.,0],[0,0,c]])}
        #Kovalev
        #self.bravais['hP']={'Type':'Hexagonal primtive',
        #               'basis vectors':np.array([[2./np.sqrt(3)*a,1./np.sqrt(3)*a,0],[0,a,0],[0,0,c]])}
        self.bravais['cP']={'Type':'Cubic primitive',
                       'basis vectors':np.array([[a,0,0],[0,a,0],[0,0,a]])}
        self.bravais['cF']={'Type':'Cubic face-centred',
                       'basis vectors':np.array([[0,a/2.,a/2.],[a/2.,0,a/2.],[a/2.,a/2.,0]])}
        self.bravais['cI']={'Type':'Cubic body-centred',
                       'basis vectors':np.array([[-a/2.,a/2.,a/2.],[a/2.,-a/2.,a/2.],[a/2.,a/2.,-a/2.]])}

    def Lattice(self):
        return self.bravais[self.bravaistype]

    def Lat2Cart(self):
        """
        matrix transformation of lattice vectors to cartesian basis (as defined in Bradley and Cracknell)
        """
        try:
            lat2cart=np.transpose(self.bravais[self.bravaistype]['basis vectors'])
        except:
            sys.exit('unknown lattice type')
        return lat2cart


    def ReciprocalLattice(self):
        """
        @return: transformation matrix from reciprocal lattice to cartesian system
        @rtype: numpy array
        """
        t1=self.bravais[self.bravaistype]['basis vectors'][0][:]
        t2=self.bravais[self.bravaistype]['basis vectors'][1][:]
        t3=self.bravais[self.bravaistype]['basis vectors'][2][:]
        pi=cmath.pi
        V=np.dot(t1,np.cross(t2,t3))
        g1=2.*pi*np.cross(t2,t3)/V
        g2=2.*pi*np.cross(t3,t1)/V
        g3=2.*pi*np.cross(t1,t2)/V
        return np.array([g1,g2,g3])

if __name__ == "__main__":
    cell=[6.28,6.28,6.28,90.,90.,120.,'hP']

    print 'conventional cell', cell
    bl=BravaisLattices(cell)
    print 'primitve lattice vectors in cartesian units (Cracknell convention)'
    print 't1=', bl.bravais[cell[6]]['basis vectors'][0]
    print 't2=', bl.bravais[cell[6]]['basis vectors'][1]
    print 't3=', bl.bravais[cell[6]]['basis vectors'][2]
    print 'reciprocal lattice (row vectors) in cartesian coordinates:'
    g=bl.ReciprocalLattice()
    print 'moduli of reciprocal lattice vectors and angles'
    for gi in g:
        print np.sqrt(np.dot(gi,gi))
    print np.arccos(np.dot(g[0],g[1])/np.linalg.norm(g[0])/np.linalg.norm(g[1]))/np.pi*180.
    print np.arccos(np.dot(g[0],g[2])/np.linalg.norm(g[0])/np.linalg.norm(g[1]))/np.pi*180.
    print np.arccos(np.dot(g[1],g[2])/np.linalg.norm(g[0])/np.linalg.norm(g[1]))/np.pi*180.

    print 'Matrix from lattice to cartesian system'
    print bl.Lat2Cart()
    print 'Matrix from reciprocal lattice to cartesian system'
    print bl.ReciprocalLattice()
    k=np.dot(np.array([-1./3.,2./3.,0]),bl.ReciprocalLattice())
    print np.linalg.norm(k)/2./np.pi
