#!/usr/bin/env python
"""
Decode Hall symbols and return list of symmetry operations
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


from collections import OrderedDict
from fractions import Fraction
import copy
import numpy as np
import pydoc
import os
import sys
from CrystalSystem import CrystalSystem
from OperationHelper import OperationHelper
from PointGroupMatrices import PointGroupMatrices

try:
    from prettytable import *
except ImportError:
    sys.exit('Module Pretty Table could not be imported')
        
from os import path
        
class HallSymbols(CrystalSystem,OperationHelper,PointGroupMatrices):
    """
    decode Hall symbol(s) and compute symmetry operations from generators
    """
    def __init__(self):
        CrystalSystem.__init__(self)
        OperationHelper.__init__(self)
        PointGroupMatrices.__init__(self)
        #fpath=os.path.dirname(os.path.abspath(__file__))
        #symfile=fpath+'/generators/Hall.dat'
        script_dir = os.path.dirname(__file__)
        rel_path = "generators/Hall.dat"
        symfile = os.path.join(script_dir, rel_path)
        #print symfile

        try:
            with open(symfile) as f:
                data=f.read()
                lines=data.split("\n")
                lines=filter(None,lines)
        except IOError:
            sys.exit('-->file Hall.dat not found in path')
        self.spacegroups=OrderedDict()
        #read in Hall symbols
        for line in lines:
            tmp=line.split('  ')
            tmp=filter(None,tmp)
            tmpdict=dict()
            tmpdict['HM']=tmp[1]
            tmpdict['Hall']=tmp[2]
            self.spacegroups[tmp[0].replace(" ","")]=tmpdict

        self.spacegroupgenerators=[]
        self.matrixspacegroupoperations=[]
        self.centrosymmetric=False
        self._InversionSymbol()
        self._LatticeSymbols()
        self._TranslationSymbol()
        self._RotationSymbol()

    def _formtranslation(self,trans):
        """
        form translation vector from Hall keyword a,b,c,d,n
        @param trans: Hall keyword for translation: a,b,c,d,n
        @type  trans: string
        @rtype:       numpy.ndarray
        @return:      translation vector
        """
        vector=np.array([0,0,0])
        for t in trans:
            vector=vector+self.translationsymbol[t]
        return vector

    def JonesSymbol(self, m):
        """
        return Jones faithfull symbol from symmetry element
        @param m: symmetry operation as a 4x4 matrix
        @type m:  numpy.ndarray
        @rtype: string
        @return: Jones symbol
        """
        X=''
        if m[0,0] == 1: X+='+x'
        if m[0,1] == 1: X+='+y'
        if m[0,2] == 1: X+='+z'
        if m[0,0] == -1: X+='-x'
        if m[0,1] == -1: X+='-y'
        if m[0,2] == -1: X+='-z'
        if m[0,3] != 0:
            fr=Fraction(m[0,3]).limit_denominator(6)
            X+='+'+str(fr.numerator)+'/'+str(fr.denominator)
        Y=''
        if m[1,0] == 1: Y+='+x'
        if m[1,1] == 1: Y+='+y'
        if m[1,2] == 1: Y+='+z'
        if m[1,0] == -1: Y+='-x'
        if m[1,1] == -1: Y+='-y'
        if m[1,2] == -1: Y+='-z'
        if (m[1,3] != 0):
            fr=Fraction(m[1,3]).limit_denominator(6)
            Y+='+'+str(fr.numerator)+'/'+str(fr.denominator)
        Z=''
        if m[2,0] == 1: Z+='+x'
        if m[2,1] == 1: Z+='+y'
        if m[2,2] == 1: Z+='+z'
        if m[2,0] == -1: Z+='-x'
        if m[2,1] == -1: Z+='-y'
        if m[2,2] == -1: Z+='-z'
        if m[2,3] != 0:
            fr=Fraction(m[2,3]).limit_denominator(6)
            Z+='+'+str(fr.numerator)+'/'+str(fr.denominator)

        jones=X+','+Y+','+Z
        return jones

    def _LatticeSymbols(self):
        """
        define (Hall) generators of the translations
        @return: None
        """
        self.latticesymbol=OrderedDict()
        
        tmp=dict()
        tmp['generators']=1
        tmp['translations']=[np.array([0.,0.,0.])]
        tmp['c2p']=np.array([[1.,0.,0.],[0.,1.,0.],[0.,0.,1.]])
        self.latticesymbol['P']=tmp

        tmp=dict()
        tmp['generators']=2
        tmp['translations']=[np.array([0.,0.,0.])]
        tmp['c2p']=np.array([[1.,0.,0.],[0.,1.,0.],[0.,0.,1.]])
        self.latticesymbol['-P']=tmp

        tmp=dict()
        tmp['generators']=2
        tmp['translations']=[np.array([0.,0.,0.]),np.array([0.,1./2,1./2])]
        tmp['c2p']=0.5*np.array([[0.,0.,2.],[1.,1.,0.],[-1.,1.,0.]])
        self.latticesymbol['A']=tmp

        tmp=dict()
        tmp['generators']=4
        tmp['translations']=[np.array([0.,0.,0.]),np.array([0,1./2,1./2])]
        tmp['c2p']=0.5*np.array([[0.,0.,2.],[1.,1.,0.],[-1.,1.,0.]])
        self.latticesymbol['-A']=tmp

        tmp=dict()
        tmp['generators']=2
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./2.,0.,1./2.])]
        tmp['c2p']=0.5*np.array([[1.,1.,0.],[0.,0.,2.],[-1.,1.,0.]])
        self.latticesymbol['B']=tmp

        tmp=dict()
        tmp['generators']=4
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./2.,0.,1./2.])]
        tmp['c2p']=0.5*np.array([[1.,1.,0.],[0.,0.,2.],[-1.,1.,0.]])
        self.latticesymbol['-B']=tmp

        tmp=dict()
        tmp['generators']=2
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./2.,1./2.,0.])]
        tmp['c2p']=0.5*np.array([[1.,1.,0.],[-1.,1.,0.],[0.,0.,2.]])
        self.latticesymbol['C']=tmp

        tmp=dict()
        tmp['generators']=4
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./2.,1./2.,0.])]
        tmp['c2p']=0.5*np.array([[1.,1.,0.],[-1.,1.,0.],[0.,0.,2.]])
        self.latticesymbol['-C']=tmp

        tmp=dict()
        tmp['generators']=2
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./2,1./2,1./2.])]
        tmp['c2p']=0.5*np.array([[-1.,1.,1.],[1.,-1.,1.],[1.,1.,-1.]])
        self.latticesymbol['I']=tmp

        tmp=dict()
        tmp['generators']=4
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./2.,1./2.,1./2.])]
        tmp['c2p']=0.5*np.array([[-1.,1.,1.],[1.,-1.,1.],[1.,1.,-1.]])
        self.latticesymbol['-I']=tmp

        tmp=dict()
        tmp['generators']=3
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./3,2./3,2./3]),np.array([2./3,1./3,1./3])]
        tmp['c2p']=1./3.*np.array([[-1.,2.,-1.],[-2.,1.,1.],[1.,1.,1.]])
        self.latticesymbol['R']=tmp

        tmp=dict()
        tmp['generators']=6
        tmp['translations']=[np.array([0.,0.,0.]),np.array([1./3,2./3,2./3]),np.array([2./3,1./3,1./3])]
        tmp['c2p']=1./3.*np.array([[-1.,2.,-1.],[-2.,1.,1.],[1.,1.,1.]])
        self.latticesymbol['-R']=tmp

        tmp=dict()
        tmp['generators']=4
        tmp['translations']=[np.array([0.,0.,0.]),np.array([0,1./2,1./2]),np.array([1./2,0,1./2]),np.array([1./2,1./2,0])]
        tmp['c2p']=0.5*np.array([[0.,1.,1.],[1.,0.,1.],[1.,1.,0.]])
        self.latticesymbol['F']=tmp

        tmp=dict()
        tmp['generators']=8
        tmp['translations']=[np.array([0.,0.,0.]),np.array([0,1./2,1./2]),np.array([1./2,0,1./2]),np.array([1./2,1./2,0])]
        tmp['c2p']=0.5*np.array([[0.,1.,1.],[1.,0.,1.],[1.,1.,0.]])
        self.latticesymbol['-F']=tmp

    def GetLatticeTranslations(self,lat='P'):
        return self.latticesymbol[lat]
        
    def GetBravais(self,lat='P'):
        return self.bravaislattice
        
    def _InversionSymbol(self):
        """
        define inversion operation (4x4 matrix)
        @return: None
        """
        self.inversion=np.array([[-1.,0,0,0],[0,-1.,0,0],[0,0,-1.,0],[0,0,0,1.]])

    def OriginShift(self,P,v):
        """
        transform operation P origin due to origin shift by vector v
        @param P: symmetry operation as a 4x4 matrix
        @type P:  numpy.array
        @param v: (shift) vector to new origin
        @type v:  numpy.array
        @return:  symmetry operation in new (shifted) coordinate system
        @rtype:   numpy.array
        """
        v=v/12.  #shifts are given in twelthts (ITA 1.4)
        told=P[0:3,3]
        tnew=v-np.dot(P[0:3,0:3],v)+told
        P[0:3,3]=tnew
        return P

    def PrintGenerators(self):
        """
        print (Hall) generators
        @return: None
        """
        print 'Space group generators:'
        for k in self.spacegroupgenerators:
            print self.JonesSymbol(k)
        return

    def PrintIT(self):
        """
        print symmetry operations for all space groups
        @return: None
        """
        for k, val in self.spacegroups.iteritems():
            print '**********************'
            try:
                sgnr=k[0:k.index(':')]
                print 'Space group number:',sgnr
            except ValueError:
                print 'Space group number:', k
            self.HMSymbol=val['HM']
            hallsymbol=val['Hall']
            print 'Hermann-Maugin symbol:', self.HMSymbol
            print 'Hall symbol:', hallsymbol
            sginfo=k.split(':')
            try:
                self.CrystalSystem(int(sginfo[0]),sginfo[1])
            except:
                self.CrystalSystem(int(sginfo[0]))
            self.spacegroupgenerators[:]=[]
            self.SpaceGroupGenerators(hallsymbol)
            self.PrintGenerators()
            self.SpaceGroupOperations(True)
            self.PrintSpaceGroupOperations()
            print
        return

    def SpaceGroupIsSymmorphic(self, symopset):
        """
        return true if space group is symmorphic
        @param symopset: symmetry operations
        @type symopset:  set
        @rtype: boolean
        @return: True if space grop symmorphic
        """
        symoplist=list(symopset)
        issymmorphic=True
        for indx, symop in enumerate(symoplist):
            op=symop.split(',')
            for k in range(0,3):
                if ('0.5' in op[k] or
                    '0.25' in op[k] or
                    '0.75' in op[k] or
                    '0.333' in op[k] or
                    '0.633' in op[k]):
                    issymmorphic=False
                    break
        return issymmorphic

    def SymmetryOperations(self):
        """
        form Seitz symbol form symmetry operations in matrix form
        @rtype:  set
        @return: return Seitz symbols of space and point group operations
        """
        pgop=set() #Seitz symbol of point group operations
        sgop=set() #Seitz symbol of space group operations
        for k in self.matrixspacegroupoperations:
            rotpart,seitzsymbol=self._SeitzSymbol(k,self.crystalsystem[0],self.crystalsystem[1])
            pgop.add(rotpart)
            sgop.add(seitzsymbol)
        return pgop,sgop

    from prettytable import MSWORD_FRIENDLY
    def PrintSpaceGroupOperations(self):
        """
        print spacegroup operations
        @return: None
        """
        #print
        symop=PrettyTable(['Seitz','Jones'])
        #symop.set_style(MSWORD_FRIENDLY)
        for k in self.matrixspacegroupoperations:
            rotpart,seitzsymbol=self._SeitzSymbol(k,self.crystalsystem[0],self.crystalsystem[1])
            symop.add_row([seitzsymbol, self.JonesSymbol(k)])
        print symop
        return

    def CrystalSystem(self,sg_number,setting=None,verbose=True):
        """
        determine crystal system from space group number
        @param sg_number: space group number
        @type sg_number:  string
        @param setting:    space group setting
        @type setting: string
        @return: None
        """
        sg_number=int(sg_number)
        self.crystalsystem=[]
        for k, val in self.crystalsystemdict.iteritems():
            if (sg_number >= val['sg#'][0]) and (sg_number <= val['sg#'][1]):
                self.crystalsystem.append(k)
                break
        self.crystalsystem.append(setting)
        if verbose == False:
            return
        print 'Crystal system:', self.crystalsystem[0]
        if self.crystalsystem[1]!= None:
            print 'Setting:',self.crystalsystem[1]
        print 'Pearson symbol:', self.PearsonSymbol()
        return

    def PearsonSymbol(self):
        """
        determine crystal system from space group number
        @param sg_number: space group number
        @type sg_number:  string
        @param setting:   space group setting
        @type setting:    string
        @return:          None
        """
        try:
            crystalsystem = self.crystalsystem[0]
        except NameError, err:
            print 'variable not defined', err
            sys.exit()
        if crystalsystem == 'Triclinic':
            Pearson='a'
        elif crystalsystem == 'Monoclinic':
            Pearson='m'
        elif crystalsystem == 'Orthorhombic':
            Pearson='o'
        elif crystalsystem == 'Tetragonal':
            Pearson='t'
        elif crystalsystem == 'Trigonal':
            Pearson = 'h'
        elif crystalsystem == 'Hexagonal':
            Pearson='h'
        elif crystalsystem == 'Cubic':
            Pearson='c'
        else:
            pass
        try:
            Pearson+=(self.HMSymbol.lstrip())[0]
        except NameError:
            sys.exit('Pearson symbol not found')
        return Pearson

    def SpaceGroup(self,spacegroup, verbose=True):
        """
        find hall symbol from input string and list
        all known symbols if not found
        @param spacegroup: space group symbol
        @type spacegroup : string
        @param verbose   : print information about space group
        @type verbose    : boolean
        @return          : None
        """
        try:
            self.HMSymbol=self.spacegroups[spacegroup]['HM']
            self.hallsymbol=self.spacegroups[spacegroup]['Hall']
            if verbose:
                print 'Hermann-Mauguin symbol:', self.HMSymbol
                print 'Hall symbol:', self.hallsymbol
                print 'Spacegroup number:', spacegroup
            sginfo=spacegroup.split(':')
            sgnumber=int(sginfo[0])
            try:
                setting=sginfo[1]
            except:
                setting=None
        except KeyError:
            for k,val in self.spacegroups.iteritems():
                if val['Hall'].replace(" ","")==spacegroup.replace(" ","") or \
                   val['HM'].replace(" ","")==spacegroup.replace(" ",""):
                    number=k.split(':')
                    sgnumber=number[0]
                    try:
                        setting=number[1]
                    except IndexError:
                        setting=None
                        pass
                    self.hallsymbol=self.spacegroups[k]['Hall']
                    self.HMSymbol=self.spacegroups[k]['HM']
                    self.HMSymbol=self.HMSymbol.lstrip()
                    if verbose:
                        print 'spacegroup number:',number[0]
                        print 'Hermann-Mauguin symbol:',self.HMSymbol
                        print 'Hall symbol:', self.hallsymbol
                    break
        try:
            self.SpaceGroupGenerators(self.hallsymbol)
        except AttributeError:
            print 'Space group entry not recognised'
            text=''
            for k,val in self.spacegroups.iteritems():
                text+=k + ',' + val['Hall'] + ',' + val['HM']+'\n'
            pydoc.pager(text)
            sys.exit(0)
        self.CrystalSystem(sgnumber,setting,verbose)
        return

    def SpaceGroupGenerators(self,hallsymbol):
        """
        decode Hall symbol and determine (Hall) generators
        @param hallsymbol: Hall space group symbol
        @type hallsymbol : string
        @return: None
        """
        self.spacegroupgenerators=[]
        self.nsymmetryelements=1
        self.spacegroupgenerators.append(np.eye(4))
        halloperators=hallsymbol.split(" ")
        if halloperators[0]=="":
            del halloperators[0]
        self.bravaislattice=halloperators[0]
        if '-' in halloperators[0]:
            self.centrosymmetric=True
        else:
            self.centrosymmetric=False

        #origin shift?
        if '(' in halloperators[-1]:
            t=(halloperators[-1][1:-1]).split(',')
            oshift=np.array([float(t[0]),float(t[1]),float(t[2])])
            del halloperators[-1]
        else:
            oshift=np.array([0,0,0])
        #lattice
        for k in halloperators:
            if '-1' in k:
                self.nsymmetryelements*=2
        nop=[filter(str.isdigit,k) for k in halloperators[1:]]
        for k in nop:
            self.nsymmetryelements*=int(k[0])

        for k in range(1,len(halloperators)):
            rotmat=np.eye(4)
            unitmat=np.eye(4)
            symop=halloperators[k]
            #inversion with translation
            if '-1' in symop:
                trans=symop[2:]
                tau=self._formtranslation(trans)
                rotmat[0:3,0:3]=self.inversion[0:3,0:3]
                rotmat[0:3,3]=tau
            else:
                # combined with inversion
                if symop[0] == '-':
                    unitmat=np.dot(unitmat,self.inversion)
                    symop=symop[1:]
                if ('x' in symop) or ('y' in symop):
                    try:
                        rotmat[0:3,0:3]=self.rotationcomponent[symop]
                    except KeyError:
                        rotmat[0:3,0:3]=self.rotationcomponent[symop[0:2]]
                        trans=symop[2:]
                        tau=self._formtranslation(trans)
                        rotmat[0:3,3]=tau
                else: #implicit direction for symmetry axis
                    # first symbol refers to c-direction
                    if k==1 and ('*' not in symop):
                        rot=symop[0]+'z'
                    else:
                        if '2' in halloperators[k] and '"' not in halloperators[k]:
                            if ('2' in halloperators[k-1]) or ('4' in halloperators[k-1]): rot=symop[0]+'x'
                            if ('3' in halloperators[k-1]) or ('6' in halloperators[k-1]): rot="2\'(a-b)"
                        if '3' in halloperators[k]: rot="3*"
                        if '"' in halloperators[k]:
                            symop=symop.replace('"','')
                            rot='2\"(a+b)'
                    #skip P1 and -P1
                    if rot != '1z': rotmat[0:3,0:3]=self.rotationcomponent[rot]
                    if (symop=='31') or (symop=='32') or \
                       (symop=='61') or (symop=='62') or (symop=='64') or (symop=='65'):
                        tau=self.translationsymbol[symop]
                    elif symop == '3*':
                        tau=np.array([0,0,0])
                    else:
                        tau=self._formtranslation(symop[1:])
                    rotmat[0:3,3]=tau
            rotmat[0:3,0:3]=np.dot(unitmat[0:3,0:3],rotmat[0:3,0:3])
            #origin shift
            rotmat=self.OriginShift(rotmat,oshift)
            #Jones symbol
            if np.allclose(rotmat,np.eye(4)):
                #'x,y,z' already in list
                pass
            else:
                self.spacegroupgenerators.append(rotmat)
        return

    def SpaceGroupOperations(self,bravaistranslations):
        """
        calculate symmetry operations from generators
        (see Hall's original paper)

        @param bravaistranslations: add Bravais translations to space group operators for non-primitive lattices if True
        @type bravaistranslations: boolean
        @return: None
        """
        self.matrixspacegroupoperations[:]=[]
        for k in self.spacegroupgenerators:
            self.matrixspacegroupoperations.append(k)
        nsym=self.nsymmetryelements
        m=len(self.spacegroupgenerators)
        mm=range(1,m)
        for i in mm:
            for j in mm:
                tmp=np.dot(self.matrixspacegroupoperations[i],self.matrixspacegroupoperations[j])
                addel=True
                for indx, matrix in enumerate(self.matrixspacegroupoperations):
                    R1=matrix[0:3,0:3]
                    R2=tmp[0:3,0:3]
                    if np.allclose(R1,R2)==True:
                        addel=False
                        break
                    #remove inversion if centrosymmetric
                    if self.centrosymmetric:
                        R1=-1.0*R1
                        if np.allclose(R1,R2)==True:
                            addel=False
                            break
                if addel == False:
                    pass
                else:
                    self.matrixspacegroupoperations.append(tmp)
                    mm.append(m)
                    m=m+1
                if len(self.matrixspacegroupoperations)==nsym: break
            if len(self.matrixspacegroupoperations)==nsym: break
        #add inversion
        cpmat=copy.copy(self.matrixspacegroupoperations)
        if self.centrosymmetric==True:
            nsym*=2
            for k in cpmat:
                mat=copy.copy(k)
                i=self.inversion
                mat[0:3,0:3]=np.dot(i[0:3,0:3],mat[0:3,0:3])
                self.matrixspacegroupoperations.append(mat)
        #check that number symmetry operations is right
        if len(self.matrixspacegroupoperations) != nsym:
            print 'number of created symmetry operations:',len(self.matrixspacegroupoperations)
            print 'while expected from Hall symbol:',nsym
            self.PrintSpaceGroupOperations()
            sys.exit()
        #add bravais translations to the generators if required
        cpmat=copy.copy(self.matrixspacegroupoperations)
        if bravaistranslations==True:
            for T in self.latticesymbol[self.bravaislattice]['translations'][1:]:
                for k in cpmat:
                    mat=copy.copy(k)
                    mat[0:3,3]=mat[0:3,3]+T
                    self.matrixspacegroupoperations.append(mat)
        return

    def _RotationSymbol(self):
        """
        Hall rotations operators
        """
        self.rotationcomponent=OrderedDict()
        self.rotationcomponent['2x']=np.array([[1.,0,0],[0,-1.,0],[0,0,-1.]])
        self.rotationcomponent['3x']=np.array([[1.,0,0],[0,0,-1.],[0,1.,-1.]])
        self.rotationcomponent['4x']=np.array([[1.,0,0],[0,0,-1.],[0,1.,0]])
        self.rotationcomponent['6x']=np.array([[1.,0,0],[0,1.,-1.],[0,1.,0]])

        self.rotationcomponent['2y']=np.array([[-1.,0,0],[0,1.,0],[0,0,-1.]])
        self.rotationcomponent['3y']=np.array([[-1.,0,1.],[0,1.,0],[-1.,0,0]])
        self.rotationcomponent['4y']=np.array([[0,0,1.],[0,1.,0],[-1.,0,0]])
        self.rotationcomponent['6y']=np.array([[0,0,1.],[0,1.,0],[-1.,0,1.]])

        self.rotationcomponent['2z']=np.array([[-1.,0,0],[0,-1.,0],[0,0,1.]])
        self.rotationcomponent['3z']=np.array([[0,-1.,0],[1.,-1.,0],[0,0,1.]])
        self.rotationcomponent['4z']=np.array([[0,-1.,0],[1.,0,0],[0,0,1.]])
        self.rotationcomponent['6z']=np.array([[1.,-1.,0],[1.,0,0],[0,0,1.]])

        self.rotationcomponent["2\'(b-c)"]=np.array([[-1.,0,0],[0,0,-1.],[0,-1.,0]])
        self.rotationcomponent["2\"(b+c)"]=np.array([[-1.,0,0],[0,0,1.],[0,1.,0]])
        self.rotationcomponent["2\'(a-c)"]=np.array([[0,0,-1.],[0,-1.,0],[-1.,0,0]])
        self.rotationcomponent["2\"(a+c)"]=np.array([[0,0,1.],[0,-1.,0],[1.,0,0]])
        self.rotationcomponent["2\'(a-b)"]=np.array([[0,-1.,0],[-1.,0,0],[0,0,-1.]])
        self.rotationcomponent["2\"(a+b)"]=np.array([[0,1.,0],[1.,0,0],[0,0,-1.]])
        self.rotationcomponent["3*"]=np.array([[0,0,1.],[1.,0,0],[0,1.,0]])

    def _TranslationSymbol(self):
        """
        Hall translations
        """
        self.translationsymbol=OrderedDict()
        self.translationsymbol['a']=np.array([1./2.,0,0])
        self.translationsymbol['b']=np.array([0,1./2.,0])
        self.translationsymbol['c']=np.array([0,0,1./2.])
        self.translationsymbol['n']=np.array([1./2.,1./2.,1./2.])
        self.translationsymbol['u']=np.array([1./4.,0,0])
        self.translationsymbol['v']=np.array([0,1./4.,0])
        self.translationsymbol['w']=np.array([0,0,1./4.])
        self.translationsymbol['d']=np.array([1./4.,1./4.,1./4.])

        self.translationsymbol['31']=np.array([0,0,1./3.])
        self.translationsymbol['32']=np.array([0,0,2./3.])
        self.translationsymbol['61']=np.array([0,0,1./6.])
        self.translationsymbol['62']=np.array([0,0,1./3.])
        self.translationsymbol['64']=np.array([0,0,2./3.])
        self.translationsymbol['65']=np.array([0,0,5./6.])



if __name__ == "__main__":
    hall=HallSymbols()
    #hall.SpaceGroup('F 4d 2 3 -1cd')
    #hall.SpaceGroup('P 4abw 2nw')
    #hall.SpaceGroup('P 2yb')
    #hall.SpaceGroup('P 4')
    #hall.SpaceGroup('54')
    #hall.SpaceGroup('P 3 1 2"')
    #hall.SpaceGroup('P 65')
    #hall.SpaceGroup('I 4bw 2bw -1bw')
    #hall.SpaceGroup('140')
    #hall.SpaceGroup('151')
    #hall.SpaceGroup('-F 2uv 2vw')
    #hall.SpaceGroup('P 1')
    #hall.SpaceGroup('P 3* -2n')
    #hall.SpaceGroup('-P 2yb')
    #hall.SpaceGroup('-I 4bd 2c 3')
    #hall.SpaceGroup('160:H')
    #hall.SpaceGroup('P 4 b m')
    #hall.SpaceGroup('R -3 m:H')
    #hall.SpaceGroup('P 1 21 1')
    #hall.SpaceGroup('F m -3 m')
    #hall.SpaceGroup('I 1 a 1')
    #hall.SpaceGroup('C 2/c 1 1')
    #hall.SpaceGroup('R -3 m:H')
    #hall.SpaceGroup('F d -3 m:1')
    #hall.SpaceGroup('I 4/m')
    #hall.SpaceGroup('P 4/m')
    #hall.SpaceGroup('P -3')
    #hall.SpaceGroup('P 4/n:1')
    #hall.SpaceGroup('P 2 2 -1n')
    #hall.SpaceGroup('P 4ab -1ab')
    #hall.SpaceGroup('R -3 m:H')
    #hall.SpaceGroup('-P 3')
    #hall.SpaceGroup('P 4/m')
    #hall.SpaceGroup('P b a 2')
    hall.SpaceGroup('F m -3 m')
    #hall.SpaceGroup('P1')
    hall.PrintGenerators()
    hall.SpaceGroupOperations(True)
    hall.PrintSpaceGroupOperations()
    hall.PearsonSymbol()
    hall.SymmetryOperations()
    #hall.PrintIT()
