#!/usr/bin/env python
"""
Determine magnetic little group type
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

import copy
import cmath
import sys
import numpy as np

from Hall import HallSymbols

class KCoefficients():
    """
    conversion from conventional to primitive lattice ITC A 5.1.3.1
    rule:
    1.-real space:
    (p1,p2,p3)^T=(a1,a2,a3)^T*P where ()^T is a row vector, p_i's is a primitive
    basis and a_i's the conventional basis
    2.-reciprocal space:
    (p1^*,p2^*,p3^*)=P^-1(a1^*,a2^*,a3^*) where () is a column vector, p^*_i's is a primitive
    basis and a^*_i's the conventional basis defined so that a_ia^*_j=\delta_{i,j}
    """
    def  __init__(self):
        self.conventional2primitive=dict()
        self.conventional2primitive['P']=np.array([[1.,0,0],[0,1.,0],[0,0,1.]])
        self.conventional2primitive['aP']=np.array([[1.,0,0],[0,1.,0],[0,0,1.]])
        self.conventional2primitive['mP']=np.array([[1.,0,0],[0,1.,0],[0,0,1.]])
        self.conventional2primitive['oP']=np.array([[1.,0,0],[0,1.,0],[0,0,1.]])
        self.conventional2primitive['tP']=np.array([[1.,0,0],[0,1.,0],[0,0,1.]])
        self.conventional2primitive['cP']=np.array([[1.,0,0],[0,1.,0],[0,0,1.]])
        self.conventional2primitive['hP']=np.array([[1.,0,0.],[0.,1,0.],[0.,0.,1.]])
        self.conventional2primitive['hR']=np.array([[1.,0,0.],[0.,1,0.],[0.,0.,1.]])

        self.conventional2primitive['mA']=np.array([[1.,0,0],[0.,1.,-1.],[0,1.,1.]])
        self.conventional2primitive['oA']=np.array([[1.,0,0],[0.,1.,-1.],[0,1.,1.]])

        self.conventional2primitive['mC']=np.array([[1., 1.,0],[-1.,1.,0.],[0,0.,1.]])
        self.conventional2primitive['oC']=np.array([[1., 1.,0],[-1.,1.,0.],[0,0.,1.]])

        self.conventional2primitive['oF']=np.array([[-1.,1.,1.],[1.,-1.,1.],[1.,1.,-1.]])
        self.conventional2primitive['cF']=np.array([[-1.,1.,1.],[1.,-1.,1.],[1.,1.,-1.]])

        self.conventional2primitive['oI']=np.array([[ 0.,1.,1.],[1., 0.,1.],[1.,1., 0.]])
        self.conventional2primitive['tI']=np.array([[ 0.,1.,1.],[1., 0.,1.],[1.,1., 0.]])
        self.conventional2primitive['cI']=np.array([[ 0.,1.,1.],[1., 0.,1.],[1.,1., 0.]])

    def Conventional2Primitive_real(self,lat,a):
        """
        @type a: array
        @param a: vector
        @type lat: string
        @param lat: lattice
        """
        P=self.conventional2primitive[lat]
        return np.dot(a,P)


    def Conventional2Primitive_reciprocal(self,lat,astar):
        """
        @type astar: array
        @param astar: vector
        @type lat: string
        @param lat: lattice
        """
        P=self.conventional2primitive[lat]
        return np.dot(np.linalg.inv(P),astar)

    def IsReciprocalVector(self,lat,K):
        """
        @type K: array
        @param K: coefficients of k-vector in conventional reciprocal lattice
        @type lat: string
        @param lat: lattice

        K is a reciprocal lattice vector only and
        only if the coefficients
        (k_p1,k_p2,k_p3)^T = (K1,K2,K3)^T*P^-1
        are integers
        """
        P=self.conventional2primitive[lat]
        kp=np.dot(K,np.linalg.inv(P))
        diff = kp - np.round(kp)
        if np.allclose(diff,0):
            return True
        else:
            return False

class Kvector(HallSymbols):
    """
    k-vector information
    """
    def __init__(self,sglabel,k0=np.array([0,0,0])):
        """
        @type cell: numpy array
        @param cell:cell constants and angles
        @type sglabel:list
        @param sglabel:Hermann-Maugin sapce group symbol
        @type k0: numpy.array
        @param k0: propagation vector
        """
        HallSymbols.__init__(self)

        """
        calculate (translational) coset representatives of the space group
        i.e. G=R_1T+R_2T+...+R_NT
        {T}: Lattice translations
        {R_i|v}: Rotation and translational parts of symmetry operation
        """
        self.SpaceGroup(sglabel,verbose=False)
        #operations in factor group G/T, with T the Bravais translations
        #--> False
        self.SpaceGroupOperations(False)
        print '\nCoset representatives of translational sub-group:'
        self.PrintSpaceGroupOperations()
        self.pointgroupsymmetryoperations,self.spacegroupsymmetryoperations=list(self.SymmetryOperations())
        #print 'point group operations:', len(self.pointgroupsymmetryoperations)
        self.k0=np.array(k0)
        self.cogroupsymmetryoperations=[]
        #coset of magnetic group
        self.Mcoset=set()
        ps=self.PearsonSymbol()
        self.reversingoperators=list() #list of operators that reverse k  to form the magnetic group
        self.magneticlittlegroup=[]

    def KVectorOnBZB(self):
        """
        check if \vec k is a special point in the BZ, i.e.
        if there is a symmetry operation of the co-group which transforms k into an
        equivalent vector and (\alpha^* \vec k -\vec k)\cdot \vec \tau ? 0
        """
        if np.allclose(self.k0,0.):
            return False
        else:
            issymmorphic = True
            for k in self.cogroupsymmetryoperations:
                for op in list(self.spacegroupsymmetryoperations):
                    s=op.index('{')+1
                    e=op.index('|')
                    if k == op[s:e]:
                        opmat=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
                        rotpart=opmat[0:3,0:3]
                        trans=opmat[0:3,3]
                        if not np.allclose(trans,0):
                            issymmorphic = False
                            break

            onBZ = False
            for k in self.cogroupsymmetryoperations:
                for op in list(self.spacegroupsymmetryoperations):
                    s=op.index('{')+1
                    e=op.index('|')
                    if k == op[s:e]:
                        opmat=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
                        rotpart=opmat[0:3,0:3]
                        trans=opmat[0:3,3]
                        break
                krotated=self.Transformkvector(rotpart,self.k0)
                K = self.k0-krotated
                if np.allclose(K,0):
                    pass
                else:
                    if self.ReciprocalLatticeVector(K) == True:
                        onBZ = True
                    else:
                        onBz = False
            if (onBZ == True) and (issymmorphic == False):
                return True
            else:
                return False

    def CosetOfSpaceGroup(self,starofk0):
        """
        return cosetrepresentatives of G relative to little group

        @type starofk0: dictionary
        @param starofk0: star of propagation vector
        @rtype cosetrep: list
        @return cosetrep: coset representatives
        """
        cosetrep=[]
        for k,val in starofk0.iteritems():
            try:
                symop=val[1]
            except IndexError:
                cosetrep.append('{1|(0.0,0.0,0.0)}')
            else:
                for op in list(self.spacegroupsymmetryoperations):
                    s=op.index('{')+1
                    e=op.index('|')
                    if symop == op[s:e]:
                        cosetrep.append(op)
        return cosetrep

    def LittleCoGroup(self):
        """
        determine
        1.- the set of the (rotation part of the) elements in the space group that
        transforms the propagation vector to an equivalent one
        2.- the coset representatives of the magnetic little group Mk=1*Gk+T*R*Gk with self.Mcoset={1,T*R}
        """
        for pgsymbol in self.pointgroupsymmetryoperations:
            rotpart=self.PointGroupSymbol2Matrix(pgsymbol,self.crystalsystem[0],self.crystalsystem[1])
            krotated=self.Transformkvector(rotpart,self.k0)
            K=self.k0-krotated
            IsEquivalent=self.ReciprocalLatticeVector(K)
            if IsEquivalent==True:
                self.cogroupsymmetryoperations.append(pgsymbol)
            #Rk=-k?
            if np.allclose(self.k0+krotated,0):
                self.reversingoperators.append(pgsymbol)
        #print '\nPropagation vector information:'
        self.isFedorovGroup = False
        self.isGreyGroup = False
        self.isColorGroup = False
        print '\nk-vector information:'
        print '---------------------'
        print 'The k-vector is: k = ', self.k0
        if len(self.reversingoperators) == 0: # or np.allclose(self.k0,0.) :
            print 'There is no operator that reverses the propagation vector'
            print 'The magnetic little group is the Fedorov group'
            self.isFedorovGroup = True
        else:
            #print '\t-k belongs to the star of k and'
            K=np.dot(-2.,self.k0)
            if self.ReciprocalLatticeVector(K) == True:
                print 'k is equivalent to -k'
                print '--> The magnetic little group is the grey group'
                self.isGreyGroup = True
            else:
                print 'k is not equivalent to -k'
                print '-->The magnetic little group is the black and white group'
                self.isColorGroup = True
                
        ################
        #to skip time-reversal and co-representations set
        
        #self.isFedorovGroup = True
        #self.isGreyGroup = False
        #self.isColorGroup = False
        
        ###############

        if self.isFedorovGroup:
            self.hasreversingoperator=False
        elif np.allclose(self.k0,0):
            self.hasreversingoperator=True
            self.Mcoset.add('{1|(0.0,0.0,0.0)}*T')
        elif self.isGreyGroup:
            self.hasreversingoperator=True
            self.Mcoset.add('{1|(0.0,0.0,0.0)}*T')
        elif self.isColorGroup:
            self.hasreversingoperator=True
            #choose inversion if in list
            try:
                indx = self.reversingoperators.index('-1')
                op = self.reversingoperators[indx]
            except ValueError:
                op=self.reversingoperators[0]
            for el in self.spacegroupsymmetryoperations:
                s=el.index('{')+1
                e=el.index('|')
                if op == el[s:e]:
                    self.Mcoset.add(el+'*T')
                    break
        self.Mcoset.add('{1|(0.0,0.0,0.0)}')
        return

    def MagneticLittleGroup(self,littlegroup,Mcoset):
        """
        form the magnetic little group as a list
        """
        #magnetic little group
        for coset in Mcoset:
            s=coset.index('{')
            e=coset.index('}')
            #if found coset.find('*T')
            cosetmat=self._Seitz2Matrix(coset[s:e],self.crystalsystem[0],self.crystalsystem[1])
            for lk in littlegroup:
                s=lk.index('{')
                e=lk.index('}')
                lkmat=self._Seitz2Matrix(lk[s:e],self.crystalsystem[0],self.crystalsystem[1])
                seitz=self._SeitzSymbol(np.dot(cosetmat,lkmat),self.crystalsystem[0],self.crystalsystem[1])[1]
                #which symmetry operation in little group is it?
                for op in self.spacegroupsymmetryoperations:
                    s=op.index('{')+1
                    e=op.index('|')
                    if seitz[s:e] == op[s:e]:
                        symop=op
                        break
                if coset.find('*T') != -1:
                    symop=symop+'*T'
                self.magneticlittlegroup.append(symop)
        return

    def ReciprocalLatticeVector(self,K):
        """
        return True if K is a reciprocal lattice vectors
        uses following conditions  (Table 2.2.13.1 ITC A):
        cond={'P': None,
         'A': ['k+l=2n'],
         'B': ['h+l=2n'],
         'C': ['h+k=2n'],
         'I': ['h+k+l=2n'],
         'F': ['h+k=2n','h+l=2n','k+l=2n'],
         'R': ['-h+k+l=3n']
        }

        @type K: numpy array
        @param K: reciprocal vector in rlu
        @rtype IsVec: boolean
        @return IsVec: True if vector is a reciprocal lattice vector
        """

        bl=self.HMSymbol[0]
        if bl == 'R':
            setting = self.HMSymbol[-1:] #hexagonal or rhombohedral setting
        IsVec = None
        K=np.abs(K)
        if K[0]-int(K[0]) != 0 or K[1]-int(K[1]) != 0 or K[2]-int(K[2]) != 0:
            return False

        if bl == 'P' or (bl == 'R' and setting == 'R'):
            #return true if it is an integer
            diff = K-np.round(K)
            if np.allclose(diff,0):
                IsVec=True
            else:
                IsVec=False
        if bl == 'A':
            if (K[1]+K[2]) % 2 == 0:
                IsVec=True
            else:
                IsVec=False
        if bl == 'B':
            if (K[0]+K[2]) %2 == 0:
                IsVec=True
            else:
                IsVec=False
        if bl == 'C':
            if (K[0]+K[1]) %2 == 0:
                IsVec=True
            else:
                IsVec=False
        if bl == 'I':
            if (K[0]+K[1]+K[2]) % 2 == 0:
                IsVec=True
            else:
                IsVec=False
        if bl == 'F':
            #hkl all odd or all even
            if ((K[0]+K[1]) %2 == 0) and ((K[0]+K[2]) %2 == 0) and ((K[1]+K[2]) %2 == 0):
                IsVec=True
            else:
                IsVec=False
        if bl == 'R' and setting == 'H':
            if (-K[0]+K[1]+K[2] %3 == 0):
                IsVec=True
            else:
                IsVec=False
        if IsVec == None:
            sys.exit('Unknown lattice centering')
        return IsVec

    def LittleGroupOfk(self):
        """
        @rtype littlegroupoperations: set
        @return littlegroupoperations: (W,t) such as W is in co-group of the propagation vector
        """
        littlegroupoperations=set()
        for k in self.cogroupsymmetryoperations:
            for sop in self.spacegroupsymmetryoperations:
                s=sop.index('{')+1
                e=sop.index('|')
                if k == sop[s:e]:
                    littlegroupoperations.add(sop)
        return littlegroupoperations

    def Transformkvector(self,R,k0):
        """
        return (transpose(R))^-1)*k
        """
        return np.dot(np.linalg.inv(np.transpose(R)),k0)

    def StarOfKvector(self):
        """
        orbit of Gamma^k (irrep of the translation group) contains
        the set {kW_m+K} with {W_m} the coset representatives of the little cogroup relative to G
        which is the star of k -> Orbit == star of k
        """
        cosets=self.pointgroupsymmetryoperations-set(self.cogroupsymmetryoperations)
        star=[]
        star.append(self.k0)
        for Wm in cosets:
            R=self.PointGroupSymbol2Matrix(Wm,self.crystalsystem[0],self.crystalsystem[1])
            ki=self.Transformkvector(R,self.k0)
            toappend=True
            for k in star:
                if np.allclose(k,ki):
                    toappend=False
                    break
            if toappend==True:
                star.append(ki)
        i=1
        starofk0=dict()
        for k in star:
            starofk0[i]=[k]
            i=i+1
        for Wm in cosets:
            R=self.PointGroupSymbol2Matrix(Wm,self.crystalsystem[0],self.crystalsystem[1])
            ki=self.Transformkvector(R,self.k0)
            for k, val in starofk0.iteritems():
                if np.allclose(ki,val[0]):
                    val.append(Wm)
                    starofk0[k]=val
        #remove equivalent k0-vectors
        tmp=copy.copy(starofk0)
        for i, val in tmp.iteritems():
            for j, k in tmp.iteritems():
                if i==j:
                    pass
                elif i<j:
                    pass
                else:
                    K=val[0]-k[0]
                    if self.ReciprocalLatticeVector(K)==True:
                        try:
                            del(starofk0[j])
                        except KeyError:
                            pass
        return starofk0

    def PrintLittleCoGroup(self):
        print 'little co-group of k_0'
        for key in sorted(self.CoGroup.iterkeys()):
            print key, self.CoGroup[key]

if __name__ == "__main__":
    #tested structures
    #----------------------

    #SpaceGroupLabel='P 4/m'
    #kvector=[0.,0.,0.]

    #SpaceGroupLabel='C 2/c 1 1'
    #kvector=[0.333333,0.333333,0.]
    #kvector=[0.5,0.,0.5]

    #SpaceGroupLabel='P -3'
    #kvector=[-1./3.,2./3.,0.]

    #SpaceGroupLabel='C 1 2/c 1'
    #kvector=[0.5, 0, -0.5]

    #SpaceGroupLabel='-P 2c'
    #kvector=[0.5,0,0.5]

    #SpaceGroupLabel='P 21 1 '
    #kvector=[0,0,0.25]

    #SpaceGroupLabel='-P 2c'
    #kvector=np.array([0.5,0,0.5])

    #SpaceGroupLabel='C 2/c 1 1'
    #kvector=np.array([0.5,0.,-0.5])

    #MnP
    #SpaceGroupLabel='P 21/b 1 1'
    #kvector=np.array([0.14,0.0,0.])

    #SpaceGroupLabel = 'P 3'
    #kvector=np.array([1./3,1./3,0])

    #CeAl2
    #SpaceGroupLabel = 'F d -3 m:2'
    #kvector= [0.45,0.55,0.5]

    #SpaceGroupLabel = 'P b a 2'
    #kvector= [0.5,0.,0.25]

    #NdFeBO
    #SpaceGroupLabel = 'R 3 2:H'
    #kvector = [0,0,1.5]

    #SpaceGroupLabel = 'P m -3 m'
    #kvector = [0.5,0.5,0.5]

    #MnSi
    #SpaceGroupLabel = 'P 21 3'
    #kvector = [0.05,0.05,0.05]

    #R3m:R, k = [0.5,0.5,0.5]
    #SpaceGroupLabel = 'R 3 m:R'
    #kvector = [0.5,0.5,0.5]

    #Pm-3m, k = [0.5,0.5,0.5]
    #SpaceGroupLabel = 'P m -3 m'
    #kvector = [0.5,0.5,0.5]

    #P4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    #SpaceGroupLabel = 'P 4'
    #kvector = [0,0,0.15]

    #P-4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    #SpaceGroupLabel = '-P 4'
    #kvector = [0,0,0.15]

    #R2T2X
    #SpaceGroupLabel='P 4/m b m'
    #kvector=[0.,0.,0.5]

    #Mn2GeO4
    #SpaceGroupLabel='P n m a'
    #kvector=[0.15,0.15,0]

    #LiHoF4
    #SpaceGroupLabel='I 41/a:2'
    #kvector=[0.,0.,0.]

    #REMnO3
    #SpaceGroupLabel = 'P b a m'
    #kvector= [0.5,0.,0.25]

    #SpaceGroupLabel = 'I m m m'
    #kvector=[0.5,0.0,0.5]

    SpaceGroupLabel = 'I 4/m c m'
    kvector=[0.5,0.0,0]

    #SpaceGroupLabel = 'I 4/m'
    #kvector=[1.,0.0,0]

    #SpaceGroupLabel ='P 63 mc'
    #kvector = [0.5,0,0]

    #SpaceGroupLabel ='R -3 c:R'
    #kvector = [0.5,0.5,0]

    #....................................................................................................
    gt=Kvector(SpaceGroupLabel,kvector)

    gt.LittleCoGroup()
    print 'Little co-group of k0=(',','.join([str(k) for k in kvector]),'):', gt.cogroupsymmetryoperations

    littlegroupoperations=gt.LittleGroupOfk()
    print 'Little group Gk has',len(littlegroupoperations),'symmetry operations:\n',
    for l in littlegroupoperations:
        print l

    star=gt.StarOfKvector()
    print 'Arms of star:'
    for k,v in star.iteritems():
        print k,':',np.around(v[0],2)
    print 'k-vector is a special point in BZ:', gt.KVectorOnBZB()

    print 'Coset representatives of', SpaceGroupLabel, 'relative to Little Group:'
    cs=gt.CosetOfSpaceGroup(star)
    for c in cs:
        print c

    Mkgroup=''
    for op in gt.Mcoset:
        Mkgroup+='('+op+'*Gk)+'
    Mkgroup=Mkgroup[:-1]
    print 'Magnetic little group (Gk= little group, T=time reversal):',Mkgroup

    gt.MagneticLittleGroup(littlegroupoperations,gt.Mcoset)
    print gt.magneticlittlegroup
