#!/usr/bin/env python
"""
Build permutation representation
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

"""
Representation of effect of symmetry operation on
atomic positions by permutation matrix
"""
from collections import OrderedDict
from Hall import HallSymbols
import copy
import numpy as np
import sys
import warnings

class Found(Exception): pass

class WyckoffPositions(HallSymbols):
    """
    atomic sites in cell
    """
    def __init__(self,spacegrouplabel='P1',atomicposition=None,centering=False):
        """
        @type spacegrouplabel: string
        @param spacegrouplabel: label of space group
        @type atomicposition: array
        @param atomicposition: atomic coordinates (rlu)
        """
        self.groupoperations=[]
        HallSymbols.__init__(self)
        #find Hall symbol
        self.SpaceGroup(spacegrouplabel)
        #space group operations
        self.SpaceGroupOperations(centering)

        for k in self.matrixspacegroupoperations:
            rotpart,seitz=self._SeitzSymbol(k,self.crystalsystem[0],self.crystalsystem[1])
            tmp=dict()
            tmp['seitzsymbol']=seitz
            tmp['matrix']=k
            self.groupoperations.append(tmp)
        if atomicposition is None:
            sys.exit('no atomic position given as input: exit')
        else:
            self.atomicposition=atomicposition

        self.atomsinconventionalcell=[]

    def SetAtomicPositions(self,atomicpositions):
        self.atomicposition=atomicpositions
        self.atomsinconventionalcell=[]

    def SetGroupOperations(self,groupoperations):
        """
        input is a list of dictionnaries with 'seitzsymbol' and 
        'matrix' as keys (as returned by method _SeitzSymbol)
        """
        self.groupoperations=groupoperations

    def AtomicPositions(self):
        """
        wyckoff position for space group operations
        and site coordinates
        """
        self.sitecoordinates=dict()
        i=1
        coord=dict()
        eqpos=[]
        vec=np.append(self.atomicposition,1)
        for g in self.groupoperations:
            tmp=dict()
            tmp['seitzsymbol']=g['seitzsymbol']
            tmp['matrix']=g['matrix']
            pos=np.dot(g['matrix'],vec)
            tmp['coordinates']=pos[0:3]
            coord[i]=tmp
            self.sitecoordinates[i]=tmp
            i+=1
        return

    def GetAtomicPositions(self):
        return self.atomsinconventionalcell
        
    def InequivalentPositions(self):
        """
        return wickoff positions
        """
        ineqpos=[]
        ineqpos.append(np.array(self.atomicposition))
        for i, val in self.sitecoordinates.iteritems():
            atpequ=val['coordinates'][0:3]
            if atpequ[0] <= 0.: atpequ[0]+=1.
            if atpequ[1] <= 0.: atpequ[1]+=1.
            if atpequ[2] <= 0.: atpequ[2]+=1.
            if atpequ[0] >= 1.: atpequ[0]-=1.
            if atpequ[1] >= 1.: atpequ[1]-=1.
            if atpequ[2] >= 1.: atpequ[2]-=1.
            addtolist=True
            for pos in ineqpos:
                if np.allclose(atpequ,pos)==True:
                    addtolist=False
                    break
            if addtolist==True:
                ineqpos.append(atpequ)
        self.atomsinconventionalcell=ineqpos
        return

    def SplitOrbits(self):
        #positive indices
        atomsinconventionalcell=[]
        for k, val in self.sitecoordinates.iteritems():
            atp=val['coordinates']
            if atp[0] <= 0.: atp[0]+=1.
            if atp[1] <= 0.: atp[1]+=1.
            if atp[2] <= 0.: atp[2]+=1.
            if atp[0] >= 1.: atp[0]-=1.
            if atp[1] >= 1.: atp[1]-=1.
            if atp[2] >= 1.: atp[2]-=1.
            atomsinconventionalcell.append(atp)
        #atoms in zeroth-cell
        L=copy.copy(self.latticesymbol[self.bravaislattice]['translations'])
        L.append([1.,0.,0.])
        L.append([0.,1.,0.])
        L.append([0.,0.,1.])
        del L[0]

        atomsinprimitivecell=[] #remove atoms in conventional cell related by lattice translation
        atomsinprimitivecell.append(self.atomicposition)
        atomsinprimitivecellnew=atomsinprimitivecell
        warnings.simplefilter("ignore", RuntimeWarning)
        for k in range(0,len(atomsinconventionalcell)):
            atomsinprimitivecell=atomsinprimitivecellnew
            try:
                ismultipleoflattice=False
                for i in range(0,len(atomsinprimitivecell)):
                    diff = np.abs(atomsinconventionalcell[k]-atomsinprimitivecell[i])
                    if np.allclose(diff,0.):
                        ismultipleoflattice=True
                        raise Found
                    else:
                        for l in L:
                            if (all(diff % l) == 0) and (np.allclose(np.dot(diff,l)/np.sqrt(np.dot(diff,diff))/np.sqrt(np.dot(l,l)), 1.0) == True):
                                ismultipleoflattice=True
                                raise Found
            except Found:
                pass
            if ismultipleoflattice == False:
                addtolist = True
                for atpos in atomsinprimitivecell:
                    if np.allclose(atpos-atomsinconventionalcell[k],0):
                        addtolist = False
                        break
                if addtolist == True:
                    atomsinprimitivecellnew.append(atomsinconventionalcell[k])
        #distribute atomic positions into orbits
        symopgk=dict()
        for k, val in self.sitecoordinates.iteritems():
            symopgk[val['seitzsymbol']]=val['matrix']
        orbits=OrderedDict()
        for k in range(0,len(atomsinprimitivecell)):
            tmp=[]
            at=atomsinprimitivecell[k]
            tmp.append(at)
            for op in self.magneticgroupoperations:
                at2=np.append(at,np.array([1.]))
                newpos=np.dot(symopgk[op],at2)
                if newpos[0] <= 0: newpos[0]+=1.
                if newpos[1] <= 0: newpos[1]+=1.
                if newpos[2] <= 0: newpos[2]+=1.
                if newpos[0] >= 1: newpos[0]-=1.
                if newpos[1] >= 1: newpos[1]-=1.
                if newpos[2] >= 1: newpos[2]-=1.
                try:
                    for atpos in tmp:
                        diff = np.abs(atpos-newpos[0:3])
                        ismultipleoflattice = False
                        for l in L:
                            if np.allclose(diff,0.):
                                ismultipleoflattice=True
                                raise Found
                            else:
                                if (all(diff % l) == 0.)  and (np.allclose(np.dot(diff,l)/np.sqrt(np.dot(diff,diff))/np.sqrt(np.dot(l,l)),1.0)):
                                    ismultipleoflattice=True
                                    raise Found
                except Found:
                    pass
                if ismultipleoflattice == False:
                    for atpos in tmp:
                        if np.allclose(atpos-newpos[0:3],0):
                            addtolist = False
                            break
                    if addtolist == True:
                        tmp.append(newpos[0:3])
            orbits[str(at)]=tmp
        #remove equvalent positions/orbits
        orbitscopy=copy.copy(orbits)
        keys=orbits.keys()
        l=0
        L.append(np.array([0.,0.,0.]))
        for orbit1 in orbits.keys():
            tmpdict=OrderedDict()
            tmpkeys=orbits.keys()[l+1:]
            for k in tmpkeys:
                tmpdict[k]=orbits[k]
            for orbit2, values2 in tmpdict.iteritems():
                for pos in values2:
                    #remove orbits related by a lattice translation
                    for lt in L:
                        pos2=pos-lt
                        lst=orbit1[1:-1].split()
                        floats = [float(x) for x in lst]
                        if np.allclose(floats,pos2):
                            try:
                                del orbitscopy[orbit2]
                            except KeyError:
                                pass
            l=l+1
        if len(orbitscopy) == 0:
            orbitscopy[orbits.keys()[0]]=orbits[orbits.keys()[0]]
            return orbitscopy
        else:
            return orbitscopy


from BravaisLattices import BravaisLattices
class PermutationRep():
    """
    calculate the permutation representation
    (effect of symmetry on atomic positions)
    """
    def __init__(self,latticetranslations,atomicpositions,littlegroup,kvector):
        """
        @type kvector: array
        @param kvector: propagation vector
        @type cell: array
        @param cell: lattice parameters, angles and type
        """
        self.latticetranslations=latticetranslations
        self.atomsinprimitivecell=atomicpositions
        self.littlegroupsymmetryoperations=littlegroup
        self.permutationrepresentation=dict()
        self.characterpermutation=dict()
        
    def CharacterPermutation(self):
        """
        Character of permutation representations
        """
        for k, val in self.permutationrepresentation.iteritems():
            character=np.around(np.trace(val),3)
            self.characterpermutation[k]=character
        return

    def PrintCharacterPermutation(self):
        print 'Character of permutation representation:'
        for k, val in self.characterpermutation.iteritems():
            print k, val

    def PermutationMatrix(self):
        """
        the little group of k, which is the factor group G^k/T, with T the
        group of Bravais translations.

        Representation matrix for each Wyckoff site
        When a symmetry operation results in an atomic position that is outside the zeroth
        cell, a phase factor must be included that relates the generated position to that in the zeroth
        cell, exp(-i*2*pi*k*T), T=lattice vector
        """
        dim=len(self.atomsinprimitivecell)
        for g in self.littlegroupsymmetryoperations:
            mat=g['matrix']
            #NOT CORRECT, TIME REVERSAL DOES NOT AFFECT TRANSLATION
            #DO NOT UNCOMMENT
            #T(exp(ikt))=exp(-ikt)
            #if 'T*' in g['seitzsymbol']:
            #    kvector=-self.k0
            #else:
            #    kvector=self.k0
            reprmat=np.zeros((dim,dim),dtype='complex')
            for k in range(0,dim):
                xyz=self.atomsinprimitivecell[k]
                xyz=np.append(xyz,[1])
                permutedatom=np.dot(mat,xyz)
                try:
                    LT=self.latticesymbol[self.bravaislattice]['translations']
                    for l in range(0,dim):
                        T=np.around(permutedatom[0:3]-self.atomsinprimitivecell[l],3)
                        Tl=np.abs(T)
                        Tl = Tl % np.array([1.,1.,1.])
                        Tl = np.abs(Tl)
                        for lt in LT:
                            if np.allclose(Tl,0) or np.allclose(Tl,np.abs(lt)):
                                reprmat[l][k]=np.around(np.exp(-2*np.pi*1j*np.dot(self.k0,T)),3)
                                raise Found
                except Found:
                    pass

            self.permutationrepresentation[g['seitzsymbol']]=reprmat
        return

    def PrintPermutationMatrix(self):
        """
        print permutation representation
        """
        print
        print 'Permutation representations:'
        print '----------------------------'
        for k,val in self.permutationrepresentation.iteritems():
            print '*:',k
            print('\n'.join([' '.join(['{:14}'.format(np.around(item,2)) for item in row])
                for row in val]))

            print


if __name__=="__main__":
    wf = WyckoffPositions('A2',(0,0,0))
    wf.AtomicPositions()
    wf.InequivalentPositions()
    print wf.atomsinconventionalcell
    pass
