#!/usr/bin/env python
"""
Calculate irreducible representations and character table of
a crystallographic point group using the induction method
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


from OperationHelper import OperationHelper
from collections import OrderedDict
import cmath
import copy
import numpy as np
import os,sys
try:
    from prettytable import *
except ImportError:
    sys.exit('Module Pretty Table could not be imported')
        
        

class Found(Exception): pass

class _PointGroupIrreps(OperationHelper):
    def __init__(self,crystalsystem):
        OperationHelper.__init__(self)
        self.conjugacyclasses=dict()
        self.crystalsystem=crystalsystem
        pass

    def BetaMatrix(self,irrep,cosetmat,k0,brillouinzb):
        """
        compute unitary beta-matrix that performs transformation
        between equivalent representations
        see miller and love

        @param irrep: known irreducible representation
        @type irrep: dictionary
        @param cosetmat: coset representative
        @type cosetmat: 4x4 matrix
        @param k0: k-vector
        @type k0: array
        @param brillouinzb: true of k-vecotr is on Brilluoin zone boundary
        @type brillouinzb: boolean
        @rtype: matrix
        @return: betamat
        """
        dim_irrepG=np.shape(irrep['{1|(0.0,0.0,0.0)}'])
        hop=irrep.keys()
        try:
            for i in range(0,dim_irrepG[0]):
                for k in range(0,dim_irrepG[1]):
                    CX=0.0
                    for h in hop:
                        r=self._Seitz2Matrix(h,self.crystalsystem[0],self.crystalsystem[1])
                        #conjugation
                        q=cosetmat
                        mat=np.dot(np.linalg.inv(q),np.dot(r,q))
                        O=self._Mat2Symbol([mat[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                        op=''
                        try:
                            for sym in hop:
                                s=sym.index('{')+1
                                e=sym.index('|')
                                if O[0] ==  sym[s:e]:
                                    op=sym
                                    raise Found
                        except Found:
                            pass
                        if brillouinzb == True:
                            T=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                            trans=mat[0:3,3]-T
                            factor=np.exp(-2*np.pi*1j*np.dot(k0,trans))
                            #round to 2 decimals
                            realpart="{0:.2f}".format(factor.real)
                            imagpart="{0:.2f}".format(factor.imag)
                            factor=float(realpart)+1j*float(imagpart)
                        else:
                            factor=1.0
                        Dconj=factor*irrep[op]
                        #r^-1
                        D=np.conjugate(irrep[h])
                        CX=CX+np.dot(D[i,i],Dconj[k,k])
                    if CX != 0.0:
                        beta_ik = CX
                        indxsaved=(i,k)
                        raise Found
        except Found:
            pass
        #beta matrix
        betamat=np.zeros((dim_irrepG[0],dim_irrepG[1]),dtype=complex)
        nu=dim_irrepG[0]
        g=float(np.size(hop))
        nu=float(nu)
        beta_ik=nu*beta_ik/g

        for j in range(0,dim_irrepG[0]):
            for l in range(0,dim_irrepG[1]):
                for h in hop:
                    r=self._Seitz2Matrix(h,self.crystalsystem[0],self.crystalsystem[1])
                    #conjugation
                    q=cosetmat
                    mat=np.dot(np.linalg.inv(q),np.dot(r,q))
                    O=self._Mat2Symbol([mat[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                    op=''
                    try:
                        for sym in hop:
                            s=sym.index('{')+1
                            e=sym.index('|')
                            if O[0] ==  sym[s:e]:
                                op=sym
                                raise Found
                    except Found:
                        pass
                    if brillouinzb == True:
                        T=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                        trans=mat[0:3,3]-T
                        factor=np.exp(-2*np.pi*1j*np.dot(k0,trans))
                        #round to 2 decimals
                        realpart="{0:.2f}".format(factor.real)
                        imagpart="{0:.2f}".format(factor.imag)
                        factor=float(realpart)+1j*float(imagpart)
                    else:
                        factor=1.0
                    Dconj=factor*irrep[op]
                    #r^-1
                    D=np.conjugate(irrep[h])
                    betamat[j,l]=betamat[j,l]+np.dot(D[indxsaved[0],j],Dconj[indxsaved[1],l])
                betamat[j,l]/=np.conjugate(beta_ik)
        return betamat

    def CharacterTable(self, Irreps):
        """
        set-up character table such as it can be printed using the
        Pretty Table module

        @param Irreps: irreducible represenations
        @type Irreps: dictionary
        @rtype: pretty table object
        @return: character table
        """
        #labels=Irreps['G0'].keys()
        labels=Irreps[Irreps.keys()[0]].keys()
        gorder=len(labels)
        labels.insert(0,'')
        charactertable=PrettyTable(labels)
        charT=dict() #to check column orthogonality theorem
        Gs=sorted(Irreps.keys())
        for G in Gs:
            rowtable=[]
            rowtable.append(G)
            tmp=dict()
            value=Irreps[G]
            for key, irrep in value.iteritems():
                if np.shape(irrep)[0] >= 2:
                    character=np.around(np.trace(irrep),3)
                else:
                    character=np.around(irrep[0][0],3)
                tmp[key]=character
            charT[G]=tmp
            for k in labels[1:]:
                rowtable.append(tmp[k])
            charactertable.add_row(rowtable)
        return charactertable,charT

    def CosetRepresentatives(self,G,H):
        """
        return the coset representatives of group G relative to normal subgroup H

        @param G: group symmetry elements
        @type G: set
        @param H: subgroup of H
        @type H: set
        @rtype: list
        @return: coset representatives
        """
        cosetrepresentatives=set()
        multiplicationtable=self.PointGroupMultiplicationTable(G)
        GH=set(G)-set(H)
        cosets=[]
        for g in list(GH):
            tmp=[]
            for h in H:
                tmp.append(multiplicationtable[(g,h)])
            cosets.append(tmp)
        #remove equal cosets
        unique_cosets=[]
        for k in cosets:
            k=sorted(k)
            if k not in unique_cosets:
                unique_cosets.append(k)
        #choose a coset representative
        cosetrepresentatives.add('1')
        for k in unique_cosets:
            cosetrepresentatives.add(k[0])
        #check that the coset representatives are complete
        #the number of cosets = G/N
        I=len(G)/len(H)
        if len(cosetrepresentatives) != I:
            sys.exit('number of coset representatives not as expected')
        return cosetrepresentatives

    def InducedRepresentations(self,Irrep,Gop,Nop,cosetrepresentatives,k0=None):
        """
        calculate an induced representation with matrix elements

        D^{ind,j}(g)_{mt,ns}=D^j(g^-1_m*h*g_n)_{t,s} if g^-1_m*g*g_n=h with h in H, 0 otherwise

        @param Irrep: irreducible representation of subgroup
        @type Irrep:  dictionary
        @param Gop: list of operation in group G
        @type Gop: list
        @param Nop: list of operation in subgroup
        @type Nop: list
        @param cosetrepresentatives: coset representatives of G relative to subgroup
        @type cosetrepresentatives: set
        @param k0: k-vector
        @type k0: numpy array
        @rtype :dictionary
        @return: induced representations of group
        """
        #point group symmetry operations
        pgop=[]
        for h in Nop:
            s=h.find('{')
            e=h.find('|')
            pgop.append(h[s+1:e])
        #list for induced representations
        IndRep=[]

        m=len(cosetrepresentatives)
        n=m
        (t,s)=np.shape(Irrep['{1|(0.0,0.0,0.0)}'])
        dim=(m*t,n*s)
        Dind=dict()
        cosets=list(cosetrepresentatives)
        for g in Gop:
            g_mat=self._Seitz2Matrix(g,self.crystalsystem[0],self.crystalsystem[1])
            m=1
            dinduced=np.zeros(dim,dtype='complex')
            for gm in cosets:
                gm_mat=self._Seitz2Matrix(gm,self.crystalsystem[0],self.crystalsystem[1])
                n=1
                for gn in cosets:
                    gn_mat=self._Seitz2Matrix(gn,self.crystalsystem[0],self.crystalsystem[1])
                    gm_inv=np.linalg.inv(gm_mat)
                    O=np.dot(gm_inv,np.dot(g_mat,gn_mat))
                    Osymbol=self._Mat2Symbol([O[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                    for sym in Nop:
                        s=sym.index('{')+1
                        e=sym.index('|')
                        if Osymbol[0]==sym[s:e]:
                            T=self._Seitz2Matrix(sym,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                            trans=O[0:3,3]-T
                            factor=np.exp(-2*np.pi*1j*np.dot(k0,trans))
                            D=factor*Irrep[sym]
                            for indx in range(1,t+1):
                                for indy in range(1,s+1):
                                    dinduced[m*indx-1][n*indy-1]=D[indx-1][indy-1]
                    n=n+1
                m=m+1
            Dind[g]=dinduced
        IndRep.append(Dind)
        return IndRep

    def Irreps(self,Ds,Orbit,Gop,Nop,cosetrepresentatives,k0,brillouinzb):
        """
        calculate irreducible represenations of a group
        1.- if length of orbit is 1 the induced representation is reduced with the method
        of Gay and Klauder;
        otherwise
        2.- the induced representation from the allowed irreducible representation of the normal subgroup are irreducuble

        @param Ds: irreducible representations of the normal subgroup
        @type Ds: dictionary
        @param Orbit: orbits of irreducible represenations of subgroup
        @type Orbit: dictionary
        @param Gop: symmetry operation of group
        @type Gop:  list
        @param Nop: symmetry operation of the normal subgroup
        @type Nop: list
        @param cosetrepresentatives: coset representatives of G relative to subgroup
        @type cosetrepresentatives: set
        @param k0: k-vector
        @type k0: numpy array
        @param brillouinzb: True if k-vector on Brillouin zone surface
        @type brillouinzb: boolean
        @rtype :dictionary
        @return: irreducbile representations of group
        """
        indxrep=0
        Dirr=dict()
        indexofsubgroup=len(Gop)/len(Nop)
        coset=list(cosetrepresentatives)
        for key,values in Orbit.iteritems():
            L=len(values)
            if L == 1:
                """
                if the length of the orbit is 1, the irreps are self-conjugate.
                the induction procedure from the subgroup will give a reducible representation.
                The method to get irreps is taken from Miller and Love
                """
                #choose coset
                for c in coset:
                    if c == '{1|(0.0,0.0,0.0)}':
                        pass
                    else:
                        cosetmat=self._Seitz2Matrix(c,self.crystalsystem[0],self.crystalsystem[1])
                        break
                irrep = Ds[list(values)[0]]
                irrep=Ds[list(values)[0]]
                #auxiliary matrix
                betamat=self.BetaMatrix(irrep,cosetmat,k0,brillouinzb)

                hop=irrep.keys()
                if indexofsubgroup == 2:
                    gamma1=dict()
                    gamma2=dict()
                    for key in irrep.iterkeys():
                        gamma1[key]=irrep[key]
                        gamma2[key]=irrep[key]
                    #normalise betamat such as betamat^2=D(q^2)
                    q=coset[1]
                    qmat=self._Seitz2Matrix(q,self.crystalsystem[0],self.crystalsystem[1])
                    q2mat=np.dot(qmat,qmat)
                    tpart=q2mat[0:3,3]
                    q2mat=q2mat[0:3,0:3]
                    q2symbol=self._Mat2Symbol([q2mat],self.crystalsystem[0],self.crystalsystem[1])
                    for sym in irrep.keys():
                        s=sym.index('{')+1
                        e=sym.index('|')
                        if q2symbol[0] == sym[s:e]:
                            op=sym
                            if brillouinzb == True:
                                T=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                                trans=tpart-T
                                factor=np.exp(-2*np.pi*1j*np.dot(k0,trans))
                                #round to 2 decimals
                                realpart="{0:.2f}".format(factor.real)
                                imagpart="{0:.2f}".format(factor.imag)
                                factor=float(realpart)+1j*float(imagpart)
                            else:
                                factor=1.0
                            D_r=factor*irrep[sym]
                    betamat2=np.dot(betamat,betamat)
                    (i,j)=np.nonzero(betamat2)
                    i=i[0]
                    j=j[0]
                    l=cmath.sqrt(D_r[i][j]/betamat2[i][j])
                    gamma1[q]=l*betamat
                    gamma2[q]=-l*betamat
                    #calculate irreps of group
                    for h in hop:
                        if '{1|(0.0,0.0,0.0)}' in h:
                            pass
                        else:
                            hmat=self._Seitz2Matrix(h,self.crystalsystem[0],self.crystalsystem[1])
                            op=np.dot(qmat,hmat)
                            sop=self._Mat2Symbol([op[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                            for sym in Gop:
                                s=sym.index('{')
                                e=sym.index('|')
                                if sop[0] == sym[s+1:e]:
                                    gamma1[sym]=np.dot(gamma1[q],gamma1[h])
                                    gamma2[sym]=np.dot(gamma2[q],gamma2[h])
                    Dirr['G'+str(indxrep)]=gamma1
                    indxrep=indxrep+1
                    Dirr['G'+str(indxrep)]=gamma2

                if indexofsubgroup == 3:
                    gamma1=dict()
                    gamma2=dict()
                    gamma3=dict()
                    for key in irrep.iterkeys():
                        gamma1[key]=irrep[key]
                        gamma2[key]=irrep[key]
                        gamma3[key]=irrep[key]
                    q=coset[1]
                    #G=H+q+q^-1*H
                    qmat=self._Seitz2Matrix(q,self.crystalsystem[0],self.crystalsystem[1])
                    q3mat=np.dot(np.dot(qmat,qmat),qmat)
                    tpart=q3mat[0:3,3]
                    q3mat=np.dot(qmat,np.dot(qmat,qmat))
                    q3mat=q3mat[0:3,0:3]
                    q3symbol=self._Mat2Symbol([q3mat],self.crystalsystem,self.crystalsystem[1])
                    for sym in irrep.keys():
                        s=sym.index('{')+1
                        e=sym.index('|')
                        if q3symbol[0] == sym[s:e]:
                            op=sym
                            if brillouinzb == True:
                                T=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                                trans=tpart-T
                                factor=np.exp(-2*np.pi*1j*np.dot(k0,trans))
                                #round to 2 decimals
                                realpart="{0:.2f}".format(factor.real)
                                imagpart="{0:.2f}".format(factor.imag)
                                factor=float(realpart)+1j*float(imagpart)
                            else:
                                factor=1.0
                            D_r=factor*irrep[sym]
                    betamat3=np.dot(betamat,np.dot(betamat,betamat))
                    (i,j)=np.nonzero(betamat3)
                    l=np.power(D_r[i][j]/betamat3[i][j],1./3.)
                    gamma1[q]=betamat*l
                    gamma2[q]=np.exp(1j*2.*np.pi/3.)*betamat*l
                    gamma3[q]=np.exp(-1j*2.*np.pi/3.)*betamat*l
                    #calculate irreps of group
                    #1.- use q as coset
                    for h in hop:
                        if '{1|(0.0,0.0,0.0)}' in h:
                            pass
                        else:
                            hmat=self._Seitz2Matrix(h,self.crystalsystem[0],self.crystalsystem[1])
                            op=np.dot(qmat,hmat)
                            sop=self._Mat2Symbol([op[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                            for sym in Gop:
                                s=sym.index('{')
                                e=sym.index('|')
                                if sop[0] == sym[s+1:e]:
                                    gamma1[sym]=np.dot(gamma1[q],gamma1[h])
                                    gamma2[sym]=np.dot(gamma2[q],gamma2[h])
                                    gamma3[sym]=np.dot(gamma3[q],gamma3[h])

                    qinv=coset[2]
                    qmat=self._Seitz2Matrix(qinv,self.crystalsystem[0],self.crystalsystem[1])
                    gamma1[qinv]=np.linalg.inv(gamma1[q])
                    gamma2[qinv]=np.linalg.inv(gamma2[q])
                    gamma3[qinv]=np.linalg.inv(gamma3[q])
                    #calculate irreps of group
                    #2.- use q^-1 as coset
                    for h in hop:
                        if '{1|(0.0,0.0,0.0)}' in h:
                            pass
                        else:
                            hmat=self._Seitz2Matrix(h,self.crystalsystem[0],self.crystalsystem[1])
                            op=np.dot(qmat,hmat)
                            sop=self._Mat2Symbol([op[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                            for sym in Gop:
                                if sop[0] in sym:
                                    gamma1[sym]=np.dot(gamma1[qinv],gamma1[h])
                                    gamma2[sym]=np.dot(gamma2[qinv],gamma2[h])
                                    gamma3[sym]=np.dot(gamma3[qinv],gamma3[h])

                    Dirr['G'+str(indxrep)]=gamma1
                    indxrep=indxrep+1
                    Dirr['G'+str(indxrep)]=gamma2
                    indxrep=indxrep+1
                    Dirr['G'+str(indxrep)]=gamma3
            else:
                #use induction theorem
                #select allowed (small) irrep from orbit
                #N.B.: all members of an orbit have conjugate little groups -> conjugate irreps
                #choose one irrep from orbit
                irr=str(key[2:-1])
                inducedIrrep=self.InducedRepresentations(Ds[irr],Gop,Nop,coset,k0)
                for ind in range(0,len(inducedIrrep)):
                    Dirr['G'+str(indxrep)]=inducedIrrep[ind]
                    indxrep=indxrep+1
                indxrep=indxrep-1
            indxrep=indxrep+1
        return Dirr

    def Orbit(self,Irreps,cosetrepresentatives,k0,brillouinzb):
        """
        return the orbit (i.e. maximal set of mutually inequivalent representations conjugated with respect to G)
        by conjugation of the elements of H
        with the coset representatives of G relative to H
        (Aroyo, Representation of Crystallographic Groups, p.26)

        N.B.: It may be shown that members of a same orbit induce equivalent irreps
        as a list of dictionaries with key = symmetry operation, value = irreps
        e.g. 'manual' input for group '4' would be

        Irreps=[]
        d={'1':np.array([[1]]),'4^+_00z':np.array([[1]]), '2_00z':np.array([[1]]),'4^-_00z':np.array([[1]])}

        Irreps.append(d)
        d={'1':np.array([[1]]),'4^+_00z':np.array([[-1]]),'2_00z':np.array([[1]]),'4^-_00z':np.array([[-1]])}

        Irreps.append(d)
        d={'1':np.array([[1]]),'4^+_00z':np.array([[1J]]),'2_00z':np.array([[-1]]),'4^-_00z':np.array([[-1J]])}

        Irreps.append(d)
        d={'1':np.array([[1]]),'4^+_00z':np.array([[-1J]]),'2_00z':np.array([[-1]]),'4^-_00z':np.array([1J]])}
        Irreps.append(d)

        @param Irreps: irreducible representations
        @type Irreps: dictionary
        @param cosetrepresentatives: coset representatives of G relative to subgroup
        @type cosetrepresentatives: set
        @param k0: k-vector
        @type k0: numpy array
        @param brillouinzb: True if k-vector on Brillouin zone surface
        @type brillouinzb: boolean
        @rtype:dictionary
        @return: orbit relative to point group G
        """
        Ds=dict()
        #symmetry operations
        Nop=Irreps['G0'].keys()
        for Gi, irrep in Irreps.iteritems():
            dcoset=dict()
            for coset in cosetrepresentatives:
                d=dict()
                try:
                    R=self.PointGroupSymbol2Matrix(coset,self.crystalsystem[0],self.crystalsystem[1])
                    for h in Nop:
                        M=self.PointGroupSymbol2Matrix(h,self.crystalsystem)
                        O=self._Mat2Symbol([np.dot(np.linalg.inv(R),np.dot(M,R))],self.crystalsystem[0],self.crystalsystem[1])
                        d[h]=irrep[O[0]]
                except KeyError:   #if coset has translations
                    R=self._Seitz2Matrix(coset,self.crystalsystem[0],self.crystalsystem[1])
                    for h in Nop:
                        M=self._Seitz2Matrix(h,self.crystalsystem[0],self.crystalsystem[1])
                        #conjugation
                        mat=np.dot(np.linalg.inv(R),np.dot(M,R))
                        O=self._Mat2Symbol([mat[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                        try:
                            for sym in irrep:
                                s=sym.index('{')+1
                                e=sym.index('|')
                                if O[0] ==  sym[s:e]:
                                    key=sym
                                    raise Found
                        except Found:
                            pass
                        if brillouinzb == True:
                            T=self._Seitz2Matrix(key,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                            trans=mat[0:3,3]-T
                            factor=np.exp(-2*np.pi*1j*np.dot(k0,trans))
                            #round to 2 decimals
                            realpart="{0:.2f}".format(factor.real)
                            imagpart="{0:.2f}".format(factor.imag)
                            factor=float(realpart)+1j*float(imagpart)
                        else:
                            factor=1.0
                        d[h]=factor*irrep[key]
                #conjugate representation
                dcoset[coset]=d
            Ds[Gi]=dcoset
        #orbits
        Orbit=dict()
        cosetlist=Ds['G0'].keys()
        for Gi, irrep in Irreps.iteritems():
            orbit=set()
            for coset in cosetlist:
                for label in Ds.iterkeys(): #loop over conjugate irreps
                    inorbit=True
                    for symop_Gi in irrep.iterkeys():
                        if np.shape(Ds[label][coset][symop_Gi]) != np.shape(irrep[symop_Gi]):
                            inorbit=False
                            break
                        #if np.array_equal(np.around(Ds[label][coset][symop_Gi],3),np.around(irrep[symop_Gi],3))==False:
                        if np.allclose(np.around(Ds[label][coset][symop_Gi],3),np.around(irrep[symop_Gi],3))==False:
                            inorbit=False
                            break
                    if inorbit == True:
                        #orbit.add((key,coset))
                        orbit.add(label)
            Orbit['O('+Gi+')']=orbit
        #remove duplicates
        dic=dict()
        for k,v in Orbit.iteritems():
            if v not in dic.values():
                dic[k]=v
        return dic

    def PointGroupConjugacyClasses(self,multiplicationtable):
        """
        calculate conjugacy classes of a point group
        uses theorem:
        distinct alements a and b in G are conjugate (and belong to the
        same class) if and only if there are elements u and v in G such
        that a=uv and b=vu
        (S.W. Golomb, Mathematics Magazine, 69, 373 (1996))

        @param multiplicationtable:  multiplication table  of group
        @type multiplicationtable: dictionary
        @rtype:  list
        @return: conjugacy classes
        """
        self.conjugacyclasses.clear()
        for key in self.pointgroupsymmetryoperations:
            self.conjugacyclasses[key]=[key]
        for key0 in self.pointgroupsymmetryoperations:
            for key1 in self.pointgroupsymmetryoperations:
                u=key0
                v=key1
                el1=multiplicationtable[(u,v)]
                el2=multiplicationtable[(v,u)]
                if el1 != el2:
                    try:
                        if el2 not in self.conjugacyclasses[el1]:
                            self.conjugacyclasses[el1].append(el2)
                            del self.conjugacyclasses[el2]
                        if el1 not in self.conjugacyclasses[el2]:
                            self.conjugacyclasses[el2].append(el1)
                            del self.conjugacyclasses[el1]
                    except:
                        pass
        return

    def CalcSubGroupIrreps(self,groupcompositionseries,k0=np.array([0.,0.,0]),brillouinzb=True):
        """
        calculate irreducible representation of group through induction starting
        from first group in the composition serie and going up the chain step by step

        @param groupcompositionseries: composition serie of group
        @type groupcompositionseries:  dictionary
        @param k0: k-vector
        @type k0:  numpy array
        @param brillouinzb: True if k-vector on Brilluoin zone boundary
        @type brillouinzb:  boolean
        """
        D=dict()
        D['G0']={'{1|(0.0,0.0,0.0)}':np.array([[1]])}
        if brillouinzb == True:
            tmp=(groupcompositionseries.keys())[::-1]
            subgroupsymmorphic=[]
            #check if normal subgroup is symmorphic
            for G,val in groupcompositionseries.iteritems():
                subgroupsymmorphic.append(True)
                for k in val:
                    mat=self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1])
                    trans=mat[0:3,3]
                    if np.allclose(trans,np.array([0,0,0]))==False:
                        subgroupsymmorphic[-1]=False
                        break
            indx=subgroupsymmorphic.index(True)
            tmp=(groupcompositionseries.keys()[indx:])[::-1]
        else:
            tmp=(groupcompositionseries.keys())[::-1]
        for k in range(0,len(tmp)):
            H=groupcompositionseries[tmp[k]]
            try:
                G=groupcompositionseries[tmp[k+1]]
            except IndexError:
                break
            #pointgroups
            g=[]
            for op in G:
                end=op.find('|')
                g.append(op[1:end])
            h=[]
            for op in H:
                end=op.find('|')
                h.append(op[1:end])
            cosetpg=self.CosetRepresentatives(g,h)
            cosetrepresentatives=[]
            for k in cosetpg:
                for op in G:
                    s=op.index('{')+1
                    e=op.index('|')
                    if k == op[s:e]:
                        cosetrepresentatives.append(op)
            O=self.Orbit(D,cosetrepresentatives,k0,brillouinzb)
            D=self.Irreps(D,O,G,H,cosetrepresentatives,k0,brillouinzb)
        return D

    def PointGroupMultiplicationTable(self,symoplist):
        """
        return multiplication table of point group

        @param symoplist:  symbols of symmetry operations
        @type symoplist:   list
        @rtype:  dictionary
        @return: multiplication table of group
        """
        multiplicationtable=dict()
        g=[]
        for g1 in symoplist:
            gmat1=self.PointGroupSymbol2Matrix(g1,self.crystalsystem[0],self.crystalsystem[1])
            for g2 in symoplist:
                gmat2=self.PointGroupSymbol2Matrix(g2,self.crystalsystem[0],self.crystalsystem[1])
                g.append(np.dot(gmat1,gmat2))
        glist=self._Mat2Symbol(g,self.crystalsystem[0],self.crystalsystem[1])
        l=0
        for g1 in symoplist:
            for g2 in symoplist:
                multiplicationtable[(g1,g2)]=glist[l]
                l+=1
        return multiplicationtable

    def PrintIrreps(self,d):
        print '\nIrreducible representations:'
        print'--------------------------------------------'
        for g,irrep in d.iteritems():
            print g
            for sym,mat in irrep.iteritems():
                print  '*:',sym
                print('\n'.join([''.join(['{:14}'.format(item) for item in row]) for row in np.around(mat,2)]))
                #print np.around(mat,3)
                print
            print'--------------------------------------------'
            print
        #print'--------------------------------------------'
