#!/usr/bin/env python
"""
Irreducible representations and co-representations of a space/paramagnetic group

Method:
1.- calculate allowed irreducible representations of the little group 
    by induction (effectively done in class _PointGroupIrreps);
2.- calculate the irreps of the space group are by induction from the allowed irreps of the little group (from 1)
2.- calculate irreducible co-representations
"""

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


from collections import OrderedDict
import cmath
import copy
import itertools
from itertools import combinations
import numpy as np
import os
import sys
import warnings

from Kvector import Kvector
from PointGroupIrreps import _PointGroupIrreps

try:
    from prettytable import *
except ImportError:
    sys.exit('Module Pretty Table could not be imported')


class Found(Exception): pass

class BetaMatrices():
    def __init__(self):
        #auxiliary matrices used to reduce the corepresentations to
        #an irreducble form. Taken from Kovalev's book
        self.BetaMat=dict()
        self.BetaMat['B0']=np.array([[1]])
        self.BetaMat['B1']=np.array([[0,1],[1,0]])
        self.BetaMat['B2']=np.array([[0,1],[-1,0]])
        self.BetaMat['B3']=np.array([[1,0],[0,-1]])
        self.BetaMat['B4']=np.array([[1,0],[0, 1]])
        self.BetaMat['B5']=np.array([[1,0],[0,1j]])
        self.BetaMat['B6']=np.array([[1,0],[0,-1j]])
        self.BetaMat['B7']=np.array([[1, 1j],[ 1j,1]])
        self.BetaMat['B8']=np.array([[1,-1j],[-1j,1]])
        self.BetaMat['B9']=np.array([[1, 1j],[-1j,-1]])
        self.BetaMat['B10']=np.array([[1,-1],[-1,-1]])
        self.BetaMat['B11']=np.array([[-1,1],[-1,-1]])
        self.BetaMat['B12']=np.array([[-1,1j],[-1j,1]])
        self.BetaMat['B13']=np.array([[ 1,1j],[-1j,-1]])
        self.BetaMat['B14']=np.array([[ 1,1],[-1,-1]])
        self.BetaMat['B15']=np.array([[-1,1],[-1,-1]])
        self.BetaMat['B16']=np.array([[1,0,0],[0,1,0],[0,0,1]])
        self.BetaMat['B17']=np.array([[0,1,0],[1,0,0],[0,0,1]])

        NMat=np.zeros((2,2))
        B1=np.hstack((NMat,self.BetaMat['B2']))
        B2 =np.hstack((self.BetaMat['B2'],NMat))
        self.BetaMat['B21']=np.vstack((B1,B2))
        self.BetaMat['B22']=np.vstack((B1,-B2))

        B1=np.hstack((NMat,self.BetaMat['B4']))
        B2=np.hstack((self.BetaMat['B4'],NMat))
        self.BetaMat['B23']=np.vstack((B1,B2))
        self.BetaMat['B24']=np.vstack((B1,-B2))

        B1=np.hstack((NMat,self.BetaMat['B3']))
        B2=np.hstack((self.BetaMat['B3'],NMat))
        self.BetaMat['B25']=np.vstack((B1,B2))
        self.BetaMat['B26']=np.vstack((B1,-B2))

        B1=np.hstack((self.BetaMat['B2'],NMat))
        B2=np.hstack((NMat,self.BetaMat['B2']))
        self.BetaMat['B27']=np.vstack((B1,B2))
        self.BetaMat['B28']=np.vstack((B1,-B2))

        B1=np.hstack((self.BetaMat['B1'],NMat))
        B2=np.hstack((NMat,self.BetaMat['B1']))
        self.BetaMat['B29']=np.vstack((B1,B2))
        self.BetaMat['B30']=np.vstack((B1,-B2))

        NMat=np.zeros((3,3))
        B1=np.hstack((self.BetaMat['B16'],NMat))
        B2=np.hstack((NMat,self.BetaMat['B16']))
        self.BetaMat['B31']=np.vstack((B1,B2))
        self.BetaMat['B32']=np.vstack((B1,-B2))

class SpaceGroupIrreps(Kvector,_PointGroupIrreps,BetaMatrices):
    def __init__(self,sglabel,k0):

        np.set_printoptions(precision=3)

        Kvector.__init__(self,sglabel,k0)
        _PointGroupIrreps.__init__(self,self.crystalsystem)
        BetaMatrices.__init__(self)

        self.pointgroupcompositionseries=OrderedDict()
        self.spacegroup=sglabel
        pgop,sgop=self.SymmetryOperations()

    def CoReps(self,Irreps):
        """
        return the (possibly reducible) co-representation matrices of the
        magnetic little group. Antisymmetric element contains the time-reversal T

        @param Irreps: irreducible representaitions of the space group
        @type Irreps: dictionary
        @rtype: dictionay
        @return: co-representations as defined by Wigner
        """
        Smat=self._AntiSym2Mat()
        if Smat == None:
            sys.exit('\nThere is no reversing element in the space group, coreps are not calculated.\nNormal end.\n')

        correps_R=dict()
        correps_B=dict()
        for G, irreps in Irreps.iteritems():
            tmp_R=dict()
            tmp_B=dict()
            for symop, irrepmat in irreps.iteritems():
                #D(R) R in G
                D=irrepmat
                #D^*(A^-1*R*A), A=T*S
                Rmat=self._Seitz2Matrix(symop,self.crystalsystem[0],self.crystalsystem[1])
                op=np.conjugate(np.dot(np.linalg.inv(Smat),np.dot(Rmat,Smat)))
                opsym=self._Mat2Symbol([op[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                try:
                    for k in self.magneticgroupoperations:
                        s=k.index('{')+1
                        e=k.index('|')
                        if opsym[0] == k[s:e]:
                            tau=(op-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                            factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                            Dstar=np.conjugate(np.dot(factor,Irreps[G][k]))
                            raise Found
                except Found:
                    pass
                dim=np.shape(irrepmat)
                Dcorrep_R=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
                Dcorrep_R[0:dim[0],0:dim[1]]=D
                Dcorrep_R[dim[0]:,dim[1]:]=Dstar
                Dcorrep_B=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
                #Bop = label of B=A*R with A antisymmetric operator and R in group
                Bop=np.dot(Smat,Rmat)
                Bsym=self._Matrix2SeitzSymbol(Bop)
                Bsym='T*'+Bsym
                #D(B) B in A*G
                op=np.dot(Smat,np.dot(Rmat,Smat))
                opsym=self._Mat2Symbol([op[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                try:
                    for k in self.magneticgroupoperations:
                        s=k.index('{')+1
                        e=k.index('|')
                        if opsym[0] == k[s:e]:
                            tau=(op-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                            factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                            D_BA=np.dot(factor,Irreps[G][k])
                            raise Found
                except Found:
                    pass
                Dcorrep_B[dim[0]:,0:dim[1]]=D_BA
                Dcorrep_B[0:dim[0],dim[1]:]=np.conjugate(irrepmat)
                tmp_R[symop]=Dcorrep_R
                tmp_B[Bsym]=Dcorrep_B
            correps_R[G]=tmp_R
            correps_B[G]=tmp_B
        return correps_R,correps_B

    def _NMatrix(self,irrep):
        """
        return N, the unitary matrix so that G(R)=N*G^*(A^-1*R*A)*N^-1
        Bradley and Cracknell, (7.3.45), (7.3.46) and p. 618

        @param irrep: irreducible representation
        @type irrep: dictionary
        @rtype: numpy array
        @return: unitary matrix that transforms a co-representation to an equivalent one
        """
        dim=np.shape(irrep['{1|(0.0,0.0,0.0)}'])
        for k, val in self.BetaMat.iteritems():
            if np.shape(val) == dim:
                N=self._ComputeN(val,irrep)
                #det not 0
                detN=np.linalg.det(N)
                if detN != 0:
                    N=np.dot(1./detN,N)
                    #N must be unitary
                    if np.allclose(np.dot(N,(N.conjugate()).T),np.eye(dim[0])):
                        return N
        return

    def _ComputeN(self,X,irrep):
        """
        compute auxiliary 'beta'-matrix
        using formula given in Cracknel's book.
        The trial matrix X is one of the auxiliary matrix
        from Kovalev's book

        @param X: trial matrix
        @type X: numpy array
        @param irrep : irredicible representations
        @type  irrep: dictionary
        @rtype: numpy array
        @return: unitary matrix N
        """
        Smat=self._AntiSym2Mat()
        dim=np.shape(irrep['{1|(0.0,0.0,0.0)}'])
        N=np.zeros(dim,dtype='complex')
        for R, irrepmat in irrep.iteritems():
            D_R=irrepmat
            Sinv=np.linalg.inv(Smat)
            Rinv=np.linalg.inv(self._Seitz2Matrix(R,self.crystalsystem[0],self.crystalsystem[1]))
            op=np.conjugate(np.dot(Sinv,np.dot(Rinv,Smat)))
            opsym=self._Mat2Symbol([op[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
            try:
                for k in self.magneticgroupoperations:
                    s=k.index('{')+1
                    e=k.index('|')
                    if opsym[0] == k[s:e]:
                        tau=(op-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                        factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                        D_Rstar=np.dot(factor,np.conjugate(irrep[k]))
                        raise Found
            except Found:
                pass
            dim=np.shape(D_R)
            N+=np.dot(D_R,np.dot(X,D_Rstar))
        N/=len(irrep)
        return N

    def ReduceCoReps(self,irreps,case):
        """
        reduce corepresentations according to result of Herring's test a,b,c

        @param irreps: irreducible representations of space-group
        @type irreps: dictionary
        @param case: result from Herring's  test
        @type case: string
        @rtype: dictionary
        @return: irreducible corepresentations
        """
        Smat=self._AntiSym2Mat()
        ircoreps=dict()
        for G, irrep in irreps.iteritems():
            if case[G] == 'c':
                pass
            else:
                if  case[G] == 'a':
                    A2=np.dot(Smat,Smat)
                elif case[G] == 'b':
                    A2=np.dot(Smat,Smat)
                A2op=self._Mat2Symbol([A2[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                A2op=A2op[0]
                try:
                    for op, mat in irrep.iteritems():
                        s=op.index('{')+1
                        e=op.index('|')
                        if A2op==op[s:e]:
                            DA2op=mat
                            raise Found
                except Found:
                    pass
                NG=self._NMatrix(irrep)
                if NG is None:
                    sys.exit('Auxiliary matrix could not be computed. Stop\n')
            #reduce coreps
            if case[G]=='a':
                if not np.allclose(np.dot(NG,NG.conjugate())-DA2op,np.zeros(np.shape(NG)[0])):
                    sys.exit('N*N^* not equal to D(A^2)')
                else:
                    tmp_p=dict()
                    tmp_m=dict()
                    for op, mat in irrep.iteritems():
                        D_R=mat
                        Rmat=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
                        #Bop = label of B=A*R with A antisymmetric operator and R in group
                        Bop=np.dot(Smat,Rmat)
                        Bsym=self._Matrix2SeitzSymbol(Bop)
                        Bsym='T*'+Bsym
                        #BAinv = B*A^-1
                        BAinv=np.dot(Smat,np.dot(Rmat,np.linalg.inv(Smat)))
                        hsym=self._Mat2Symbol([BAinv[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                        try:
                            for k in self.magneticgroupoperations:
                                s=k.index('{')+1
                                e=k.index('|')
                                if k[s:e] == hsym[0]:
                                    tau=(BAinv-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                                    factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                                    DBAinv=np.dot(factor,irrep[k])
                                    raise Found
                        except Found:
                            pass
                        D_B=np.dot(DBAinv,NG)
                        tmp_p[op]=D_R
                        tmp_p[Bsym]=D_B
                        tmp_m[op]=D_R
                        tmp_m[Bsym]=-D_B
                    ircoreps[G+'p']=tmp_p
                    ircoreps[G+'m']=tmp_m

            elif case[G]=='b':
                print 'NG', NG
                if not np.allclose(np.dot(NG,NG.conjugate())+DA2op,np.zeros(np.shape(NG)[0])):
                    sys.exit('N*N^* not equal to -D(A^2)')
                else:
                    tmp=dict()
                    for op, mat in irrep.iteritems():
                        dim=np.shape(mat)
                        D_R=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
                        D_B=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
                        D_R[0:dim[0],0:dim[1]]=mat
                        D_R[dim[0]:,dim[1]:]=mat
                        Rmat=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
                        #Bop = label of B=A*R with A antisymmetric operator and R in group
                        Bop=np.dot(Smat,Rmat)
                        Bsym=self._Matrix2SeitzSymbol(Bop)
                        Bsym='T*'+Bsym
                        BAinv=np.dot(Smat,np.dot(Rmat,np.linalg.inv(Smat)))
                        hsym=self._Mat2Symbol([BAinv[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                        try:
                            for k in self.magneticgroupoperations:
                                s=k.index('{')+1
                                e=k.index('|')
                                if k[s:e] == hsym[0]:
                                    tau=(BAinv-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                                    factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                                    DBAinv=np.dot(factor,irrep[k])
                                    raise Found
                        except Found:
                            pass
                        D_B[0:dim[0],dim[1]:]=-np.dot(DBAinv,NG)
                        D_B[dim[0]:,0:dim[1]]=np.dot(DBAinv,NG)
                        tmp[op]=D_R
                        tmp[Bsym]=D_B
                    ircoreps[G]=tmp

            elif case[G]=='c':
                tmp=dict()
                for op, mat in irrep.iteritems():
                    dim=np.shape(mat)
                    D_R=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
                    D_B=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
                    Rmat=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
                    #Bop = label of B=A*R with A antisymmetric operator and R in group
                    Bop=np.dot(Smat,Rmat)
                    Bsym=self._Matrix2SeitzSymbol(Bop)
                    Bsym='T*'+Bsym
                    #D(R) R \in H
                    D_R[0:dim[0],0:dim[1]]=mat
                    BAinv=np.dot(np.linalg.inv(Smat),np.dot(Rmat,Smat))
                    hsym=self._Mat2Symbol([BAinv[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                    try:
                        for k in self.magneticgroupoperations:
                            s=k.index('{')+1
                            e=k.index('|')
                            if hsym[0] == k[s:e]:
                                tau=(BAinv-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                                factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                                D_R[dim[0]:,dim[1]:]=np.conjugate(np.dot(factor,irrep[k]))
                                raise Found
                    except Found:
                        pass
                    #D(B) B \in AH, A antisymmetric operator
                    #BA
                    BA=np.dot(Smat,np.dot(Rmat,Smat))
                    basym=self._Mat2Symbol([BA[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                    try:
                        for k in self.magneticgroupoperations:
                            s=k.index('{')+1
                            e=k.index('|')
                            if basym[0] == k[s:e]:
                                tau=(BA-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                                factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                                DBA=np.dot(factor,irrep[k])
                                raise Found
                    except Found:
                        pass
                    #A^-1*B
                    AinvB=np.dot(np.linalg.inv(Smat),np.dot(Rmat,Smat))
                    ainvbsym=self._Mat2Symbol([AinvB[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])
                    try:
                        for k in self.magneticgroupoperations:
                            s=k.index('{')+1
                            e=k.index('|')
                            if k[s:e] == ainvbsym[0]:
                                tau=(BAinv-self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1]))[0:3,3]
                                factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                                DAinvB=np.dot(factor,irrep[k])
                                raise Found
                    except Found:
                        pass
                    D_B[0:dim[0],dim[1]:]=DBA
                    D_B[dim[0]:,0:dim[1]]=np.conjugate(DAinvB)
                    tmp[op]=D_R
                    tmp[Bsym]=D_B
                ircoreps[G]=tmp
            else:
                sys.exit('coreps case undetermined')
        return ircoreps


    def _Matrix2SeitzSymbol(self,mat):
        """
        return the Seitz symbol of a symmetry operation
        @param mat: matrix of symmetry operation
        @type  mat: array
        """
        rotpart=self._Mat2Symbol([mat[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])[0]
        seitzsymbol='{'+rotpart+'|(' +','.join([str(mat[k,3]) for k in range(0,3)])+')}'
        return seitzsymbol


    def KVectorGroup(self):
        """
        determine coset representatives of space group relative to little group,
        symmetry operations in little group and position of k-vector

        @rtype: list
        @return: cosets, symmetry operations
        """
        # k-vector information
        self.LittleCoGroup()
        print 'Little co-group Gk of k0=(',','.join([str(k) for k in self.k0]),'):', self.cogroupsymmetryoperations
        starofk=self.StarOfKvector()
        #coset decomposition of space group relative to little group
        #                       -----                   ------
        cosetsofg=self.CosetOfSpaceGroup(starofk)
        print 'Coset representative of G relative to the little group:', cosetsofg
        konBZB=self.KVectorOnBZB()
        print 'k-vector is a special point in BZ and little group is non-symmorphic:', konBZB
        #little group
        littlegroupoperations=self.LittleGroupOfk()
        self.MagneticLittleGroup(littlegroupoperations,self.Mcoset)
        self.magneticgroupoperations=[]
        for s in self.magneticlittlegroup:
            tmp=s.replace('*T','')
            self.magneticgroupoperations.append(tmp)
        return cosetsofg, littlegroupoperations, konBZB


    def CalcSpaceGroupIrreps(self,cosetsofg,littlegroupoperations,onBZB):
        """
        calculate irreducible representation of a space-group

        @type cosetsofg: list
        @param cosetsofg: coset representatives of group relative to littlegroup
        @type  littlegroupoperations: list
        @param littlegroupoperations: symmetry operations in little group
        @param onBZB: True if k-vector in Brillouin zone boundary
        @type onBZB: booelan
        @rtype: dictionary
        @return: irreducible representations
        """
        spacegroupissymmorphic=self.SpaceGroupIsSymmorphic(self.spacegroupsymmetryoperations)

        #A-
        #symmorphic space-groups or K inside BZ
        #--------------------------------------
        if spacegroupissymmorphic==True or onBZB == False:
            #composition series of little group
            self.CoGroupCompositionSeries()
            littlegroupcompositionseries=OrderedDict()
            for k,val in self.pointgroupcompositionseries.iteritems():
                tmp=[]
                for rot in val:
                    for symop in self.spacegroupsymmetryoperations:
                        s=symop.index('{')+1
                        e=symop.index('|')
                        if rot == symop[s:e]:
                            tmp.append(symop)
                littlegroupcompositionseries[k]=tmp
            print 'Composition series of little group:'
            for k, val in littlegroupcompositionseries.iteritems():
                print k, val
            #construct allowed irreps of symmorphic normal subgroup
            #1.- irreps of (normal) subgroup
            Ds=self.CalcSubGroupIrreps(littlegroupcompositionseries,self.k0,False)
            #multiply by exp(-ikt) from {E|t}
            AllowedIrrepsOfLittleGroup=dict()
            for G,irrep in Ds.iteritems():
                tmpdict=dict()
                for k,val in irrep.iteritems():
                    mat=self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1])
                    tau=mat[0:3,3]
                    factor=np.exp(-2*np.pi*1j*np.dot(self.k0,tau))
                    tmpdict[k]=np.dot(factor,val)
                AllowedIrrepsOfLittleGroup[G]=tmpdict

            #save allowed irreps of littlegroup
            self.allowedirrepsofgk=copy.copy(AllowedIrrepsOfLittleGroup)
            if len(cosetsofg) == 1:
                #irreducible represenations of the little group
                IrrepsOfSpaceGroup = AllowedIrrepsOfLittleGroup
            else: 
                #induced irreps from the allowed irreps of the little group to obtain those of the space group
                M_Ww=self.InductionMatrix(self.spacegroupsymmetryoperations,littlegroupoperations,cosetsofg)
                IrrepsOfSpaceGroup=self.InducedIrrepsOfSpaceGroup(M_Ww,AllowedIrrepsOfLittleGroup,cosetsofg)
        #B-
        #non-symmorphic space group and K on BZ
        #--------------------------------------
        else:
            #composition series of little group
            self.CoGroupCompositionSeries()
            littlegroupcompositionseries=OrderedDict()
            for k,val in self.pointgroupcompositionseries.iteritems():
                tmp=[]
                for rot in val:
                    for symop in self.spacegroupsymmetryoperations:
                        s=symop.find('{')
                        e=symop.find('|')
                        if rot == symop[s+1:e]:
                            tmp.append(symop)
                littlegroupcompositionseries[k]=tmp
            print 'Composition series of little group:'
            for k, val in littlegroupcompositionseries.iteritems():
                print k, val
            #construct allowed irreps from irreps of translation group
            labels=self.pointgroupcompositionseries.keys()
            AllowedIrrepsOfLittleGroup=dict()
            tmp=dict()
            tmp['{1|(0.0,0.0,0.0)}']=np.array([[1]])
            AllowedIrrepsOfLittleGroup['G0']=tmp
            #induction
            print 'Coset representative:'
            for j in reversed(range(1,len(labels))):
                Gop=self.pointgroupcompositionseries[labels[j-1]]
                Hop=self.pointgroupcompositionseries[labels[j]]
                cosets=self.CosetRepresentatives(Gop,Hop)
                cosetsrepr=[]
                for k in cosets:
                    for symop in self.spacegroupsymmetryoperations:
                        s=symop.index('{')+1
                        e=symop.index('|')
                        if k == symop[s:e]:
                            cosetsrepr.append(symop)
                #3.- conjugation of Ds with coset representative --> orbit
                orbit=self.Orbit(AllowedIrrepsOfLittleGroup,cosetsrepr,self.k0,brillouinzb=True)
                Gop=littlegroupcompositionseries[labels[j-1]]
                Hop=littlegroupcompositionseries[labels[j]]
                #4.- Allowed Irreps of little group
                AllowedIrrepsOfLittleGroup=self.Irreps(AllowedIrrepsOfLittleGroup,orbit,Gop,Hop,cosetsrepr,self.k0,brillouinzb=True)
                j=j-1
            #save allowed irreps of littlegroup 
            self.allowedirrepsofgk=copy.copy(AllowedIrrepsOfLittleGroup)
            #5- induced irrep of space group from the allowed irreps of the little group
            if len(cosetsofg) == 1:  
                #the coset representative contains only the unit element if we want the irreps of the little group
                #in which case we do not want to induce once more
                IrrepsOfSpaceGroup = AllowedIrrepsOfLittleGroup
            else:
                #the irreps of the space group are obtained by induction from the allowed irreps of the little group                                         
                M_Ww=self.InductionMatrix(self.spacegroupsymmetryoperations,littlegroupoperations,cosetsofg)
                IrrepsOfSpaceGroup=self.InducedIrrepsOfSpaceGroup(M_Ww,AllowedIrrepsOfLittleGroup,cosetsofg)
        return IrrepsOfSpaceGroup


    def CoGroupCompositionSeries(self):
        """
        return composition serie for little co-group

        @rtype: list
        @return: group composition series
        """
        compositionseries=[]
        groupoperations=self.cogroupsymmetryoperations

        if len(groupoperations) == 1:
            self.pointgroupcompositionseries['Group',str(0)]='1'
            return
        compositionseries.append(groupoperations)
        while True:
            index = 2
            H=self.NormalSubGroups(groupoperations,index)
            if not H:
                index = 3
                H=self.NormalSubGroups(groupoperations,index)
            groupoperations=H[0]
            compositionseries.append(H[0])
            if len(H[0])==1:
                break
        #set normalsubroups in self.pointgroupgenerators
        h=0
        for op in compositionseries:
            self.pointgroupcompositionseries['Group',str(h)]=op
            h+=1
        return


    def InducedIrrepsOfSpaceGroup(self,M_Ww,Irreps,cosets):
        """
        calculate the induced irreducible representation of a space group
        by induction from the allowed irreducible representations of the
        litte group for both symmorphic and non-symmorphic subgroups

        tensor   product of M_Ww x Irrep(h):
        [M]=mxn; [I]=pxq
        D_alfa,beta=m_i,j*I_k,l
        alfa=p*(i-1)+k; beta=q*(j-1)+l

        @param M_Ww: induction matrix
        @type M_Ww: dictionary
        @param Irreps: allowed irreps of little group
        @type Irreps:   dictionary
        @param  cosets: coset representatives
        @type cosets:   list
        @rtype:  dictionary
        @return: irreducible representations of space group
        """
        Girreps=dict()
        (dimMx,dimMy)=(len(cosets),len(cosets))
        for G, irrep in Irreps.iteritems():
            (p,q)=np.shape(irrep['{1|(0.0,0.0,0.0)}'])
            tmp=dict()
            for symop, val in M_Ww.iteritems():
                mat=np.zeros((dimMx*p,dimMy*q),dtype='complex')
                for indx, h in val.iteritems():
                    (i,j)=indx
                    factor=np.exp(-2*np.pi*1j*np.dot(self.k0,h[1]))
                    mat[p*(i-1):p*(i-1)+p,q*(j-1):q*(j-1)+q]=factor*irrep[h[0]]
                    tmp[symop]=mat
            Girreps[G]=tmp
        return Girreps


    def InductionMatrix(self,spacegroupoperations,littlegroupoperations,cosets):
        """
        return induction matrix M({a|t})_{i,j}=1 if q_i^-1{a|t}q_j in little group
        {a|t} are elements of the little group and q_j's the coset representatives
        """
        M_Ww=dict()
        Gmat=dict()
        dim=len(cosets)
        for op in spacegroupoperations:
            Gmat[op]=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
        Gkmat=dict()
        for op in littlegroupoperations:
            Gkmat[op]=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])            
        M_Ww=dict()
        for op in spacegroupoperations:
            i=1
            tmp=dict()
            for qi in cosets:
                qi_inv_mat=np.linalg.inv(self._Seitz2Matrix(qi,self.crystalsystem[0],self.crystalsystem[1]))
                j=1
                for qj in cosets:
                    qj_mat=self._Seitz2Matrix(qj,self.crystalsystem[0],self.crystalsystem[1])
                    retval=np.dot(qi_inv_mat,np.dot(Gmat[op],qj_mat))
                    for h,v in Gkmat.iteritems():
                        #check that the rotational part is in the co-group
                        if np.allclose(v[0:3,0:3],retval[0:3,0:3]):
                            #translation part
                            T=retval[0:3,3]-v[0:3,3]
                            tmp[(i,j)]=(h,T)
                            break
                    j=j+1
                i=i+1
            M_Ww[op]=tmp
        return M_Ww


    def NormalSubGroups(self,groupoperations,index):
        """
        find a normal subgroup (union of conjugacy classes) with prime
        (2,3) index
        Theorem:
        Let G be a group.
        Let H <| G.
        Then H is normal in G iff H is a union of conjugacy classes of G.

        @param groupoperations: symmetry opreations i ngroup
        @type groupoperations: list
        @param index: index of subgroup
        @type index: integer
        @rtype: list
        @return: list of symmetry operations of subgroup (empty if none)
        """
        multiplicationtable=self.PointGroupMultiplicationTable(groupoperations)
        self.pointgroupsymmetryoperations=groupoperations
        self.PointGroupConjugacyClasses(multiplicationtable)
        normalsubgroups = []
        conjugacyclasses=[]
        for key, value in self.conjugacyclasses.iteritems():
            conjugacyclasses.append(value)
        for x in range(1,len(conjugacyclasses)):
            for combo in combinations(conjugacyclasses,x):
                retval=[]
                for classes in list(combo):
                    retval.append(classes)
                merged=list(itertools.chain.from_iterable(retval))
                if len(merged)==int(len(groupoperations)/index):
                    #is it a group?
                    isingroup=False
                    ghasinverse=[]
                    for g in merged:
                        hasinverse=False
                        for h in merged:
                            gh=(g,h)
                            try:
                                indx=merged.index(multiplicationtable[gh])
                                isingroup=True
                                if merged[indx] == '1':
                                    hasinverse=True
                                    ghasinverse.append(hasinverse)
                            except ValueError:
                            #except:
                                isingroup=False
                                ghasinverse.append(hasinverse)
                                break
                        if isingroup == False: break
                        if hasinverse == False: break
                    if (isingroup==True) and (False not in ghasinverse):
                        normalsubgroups.append(sorted(merged))
        return normalsubgroups

    def PrintCoreps(self,d):
        symop=d[d.keys()[0]].keys()
        l=sorted(symop,reverse=True)
        symk=sorted(l[0:len(l)/2])+l[len(l)/2:]
        G=sorted(d.keys())
        print '\nIrreducible Co-Representations:'
        print '-------------------------------'
        for g in G:
            print g
            print '----'
            for k in range(0,len(symk)):
                print '*:',symk[k]
                matrix=np.around(d[g][symk[k]],2)
                print('\n'.join([' '.join(['{:14}'.format(item) for item in row])
                      for row in matrix]))
                #print matrix
                print
            print'--------------------------------------------'
            print

    def RealityTest(self, chartable):
        """
        Apply Herring test to characters (of the represenations)
        to determine type of representation, i.e.
        \sum\chi(P^2) = n --> G equivalent to real matrices
        \sum\chi(P^2) = 0 --> G equivalent to G^*
        \sum\chi(P^2) = -n --> G not equivalent to G^*
        where the sum extends over the antisymmetric elements in AG or
        over those elements of the little co-group that reverse k.
        n is the order of the group.
        If there is no such element the sum is zero.
        (Bradley and Cracknell p.641)

        @param chartable: character table of irrep of the little group G^k
        @type chartable: dictionary
        @rtype: string
        @return: co-representation case (a,b or c)
        """
        #antisymmetric element: G=H+(Theta*A)*H
        corepcase=dict()
        realitycase = dict()

        print '\nHerring test:'
        print '-------------'

        antisymmetricelements=set(self.magneticlittlegroup)-self.spacegroupsymmetryoperations
        if len(antisymmetricelements) == 0: #i.e k=0 case
            antisymmetricelements=self.magneticlittlegroup
        #    return None
        antisymmetricoperators=[]
        for s in antisymmetricelements:
            tmp=s.replace('*T','')
            antisymmetricoperators.append(tmp)
        for G, val in chartable.iteritems():
            sumchi=0.0
            gseitz=val.keys()
            for R in antisymmetricoperators:
                g=self._Seitz2Matrix(R,self.crystalsystem[0],self.crystalsystem[1])
                g2=np.dot(g,g)
                g2symbol=self._Mat2Symbol([g2[0:3,0:3]],self.crystalsystem[0],self.crystalsystem[1])[0]
                for hseitz in gseitz:
                    s=hseitz.index('{')
                    e=hseitz.index('|')
                    if g2symbol == hseitz[s+1:e]:
                        g2trans=self._Seitz2Matrix(hseitz,self.crystalsystem[0],self.crystalsystem[1])[0:3,3]
                        T=g2[0:3,3]-g2trans
                        factor=np.exp(-2j*np.pi*np.dot(self.k0,T))
                        sumchi+=np.round(factor*val[hseitz],3)

            #test value
            if sumchi.imag != 0 :
                sys.exit('value of reality test is not a real number!')
            if sumchi != 0:
                if (sumchi.real % len(val)) != 0:
                    sys.exit('value of reality test is not equal to the order of the group!')

            if self.isGreyGroup:
                print '*--> The sum over the characters is'

                if sumchi > 0:
                    print sumchi,':-->',G,'the representations are equivalent to a group of real matrices by a similarity transformation\n'
                    realitycase[G] = 'a'

                if sumchi < 0:
                    print  sumchi,':-->',G, 'is a complex representation equivalent to its conjugate G*'
                    print '--> Basis vectors should be combined with their complex conjugates to obtain real quantities\n'
                    realitycase[G] = 'b'

                if sumchi == 0:
                    print  sumchi,':-->',G, 'is not equivalent to its complex conjugate'
                    print '--> The direct sum with the conjugate irreducible representation must be considered to obtain real physical quantities\n'
                    realitycase[G] = 'c'

            if self.isColorGroup:
                if sumchi > 0:
                    print sumchi,':-->',G,'coreps corresponds to case a)'
                    print '       :--> each coreps is reducible and splits into 2 irreducible coreps G+, G-\n'
                    corepcase[G]='a'

                if sumchi < 0:
                    print sumchi,':-->',G,'coreps corresponds to case b)\n'
                    corepcase[G]='b'

                if sumchi == 0:
                    print sumchi,':-->',G,'coreps corresponds to case c)\n'
                    corepcase[G]='c'
        if self.isGreyGroup:
            return realitycase
        elif self.isColorGroup:
            return corepcase
        else:
            return None

    def _AntiSym2Mat(self):
        """
        transform antisymmetric operation to matrix form
        """
        try:
            S=[s for s in self.Mcoset if '*T' in s][0]
            S=S[:-2]
            return self._Seitz2Matrix(S,self.crystalsystem[0],self.crystalsystem[1])
        except:
            #sys.exit('There is no symmetry element that reverses the k-vector --> Normal End')
            return None

if __name__ == "__main__":
    #SpaceGroupLabel='P n m a'
    #kvector=np.array([0.,0.,0.5])
    #kvector=np.array([0.,0.,0.45])

    #SpaceGroupLabel='P 3'
    #kvector=np.array([0.,0.,0.5])
    #kvector=np.array([1./3,1./3,0.])

    #SpaceGroupLabel='P 6/m m m'
    #kvector=np.array([0.,0.,0.1])

    #ErFe4Ge4
    #SpaceGroupLabel='P 42/mnm'
    #SpaceGroupLabel='P n n m'
    #SpaceGroupLabel='P -4 n 2'
    #kvector=[0.45,0.45,0.]
    #kvector=[0.5,0.5,0.]

    #SpaceGroupLabel='P m m 2'
    #kvector=[0.,0.,0.]

    #SpaceGroupLabel='-P 2c'
    #kvector=np.array([0.5,0.,0.5])

    #SpaceGroupLabel = 'P -3'
    #kvector=np.array([1./3,1./3,0])

    ## examples below were implemented in the tests
    ## -------------------------------------------
    #SpaceGroupLabel='P 4/m'
    #kvector=[0.,0,0.25]

    #CuO
    #Toledano
    #SpaceGroupLabel='C 1 2/c 1'
    #kvector=[0.5,0.,-0.5]

    #CuB2O4
    #SpaceGroupLabel = 'I-42d'
    #kvector=[0,0,0]

    #SpaceGroupLabel='I 1 2/m 1'
    #kvector=[0.5,0,0.5]

    #SpaceGroupLabel='P 4/m b m'
    #kvector=[0.1,0,0]

    #SpaceGroupLabel='I 4/m'
    #kvector=[1.,0,0]

    #SpaceGroupLabel='P n m a'
    #kvector = [0,0,0.25]

    #SpaceGroupLabel='R 3 2:H'
    #kvector=[0.,0.,1.5]


    #R3c, k = 0, Rhombohedral setting
    #SpaceGroupLabel = 'R 3 c:R'
    #kvector = [0,0,0]

    #R3m, k = 0, Rhombohedral setting
    #SpaceGroupLabel = 'R 3 m:R'
    #kvector = [0,0,0]

    #P4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    #SpaceGroupLabel = 'P 4'
    #kvector = [0,0,0.15]

    #-P4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    #SpaceGroupLabel = '-P 4'
    #kvector = [0,0,0.15]

    #Mn2GeO4
    #SpaceGroupLabel='P n m a'
    #kvector=np.array([0.15,0.15,0.])

    #LiHoF4
    #SpaceGroupLabel='I 41/a:2'
    #kvector=[0.,0.,0.]

    #REMnO3
    #SpaceGroupLabel = 'P b a m'
    #kvector= [0.5,0.,0.25]

    #SpaceGroupLabel = 'P -4 21 m'
    #kvector= [0.5,0.5,0.]

    #SpaceGroupLabel = 'R -3 c:R'
    #kvector= [0.5,0.5,0.]

    #SpaceGroupLabel = 'F -4 3 m'
    #kvector= [0.,0.,0.]

    #MnSi
    #SpaceGroupLabel = 'P 21 3'
    #kvector = [-0.1,0.0,0.0]
    
    #P-4
    SpaceGroupLabel = 'P -4'
    kvector = [0,0,0.15]
    
    #..................................................................
    #..................................................................
    #calculate irreducible representations of a space group 
    gt=SpaceGroupIrreps(SpaceGroupLabel,kvector)
    #calculate allowed irreps of the group
    cosetsofg,littlegroupoperations,onBZ=gt.KVectorGroup()
    Irreps=gt.CalcSpaceGroupIrreps(cosetsofg,littlegroupoperations,onBZ)
    gt.PrintIrreps(Irreps)
    #characters
    """
    for G, symel in Irreps.iteritems():
        for seitz, mat in symel.iteritems():
            #print seitz, np.trace(np.conjugate(mat))
            print seitz, np.trace(mat)
            raw_input()
    """
    sys.exit(0)
    #-------------------------------------------------------------------
    #-------------------------------------------------------------------
    
    
    print 'The following output is for testing purposes'     
    #-------------------------------------------------------------------
    #output for test_Irreps
    #(small/allowed) irreps of the little group
    print 'allowed irreducible representations of Gk'
    print gt.allowedirrepsofgk
    raw_input('copy-paste into test_Irreps.py')
    charactertable,charT=gt.CharacterTable(gt.allowedirrepsofgk)
    print 'character table of little group:'
    #copy-paste into test_Irreps
    print charactertable
    print charT
    raw_input('copy-paste into test_Irreps.py')
    """
    #Coreps
    coreps_R,coreps_B=gt.CoReps(gt.allowedirrepsofgk)
    for G, irrep in coreps_R.iteritems():
        print G,':'
        for op, mat in irrep.iteritems():
            print
            print op
            print '-'*len(op)
            print np.around(mat,3)
        print

    for G, irrep in coreps_B.iteritems():
        print G,':'
        for op, mat in irrep.iteritems():
             print
             print op
             print '-'*len(op)
             print np.around(mat,3)
        print
    """
    corepcase=gt.RealityTest(chartable=charT)
    #irreducible co-reps of the paramagnetic group
    #gt.Mcoset=set()
    #grey group
    #$gt.Mcoset.add('{1|(0.0,0.0,0.0)}*T')
    #rcoreps=gt.ReduceCoReps(gt.allowedirrepsofgk,corepcase)
    if corepcase == None:
        pass
    else:
        rcoreps=gt.ReduceCoReps(gt.allowedirrepsofgk,corepcase)
        gt.PrintCoreps(rcoreps)
        charactertable,charT=gt.CharacterTable(rcoreps)
        print 'character table:'
        print charactertable
        print charT
    sys.exit(0)
