#!/usr/bin/env python
"""
Test the magnetic representation and its decomposition
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "beta"    

from VectorRepresentation import VectorRep
import unittest

import numpy as np
class KnownMagneticRepresentations(unittest.TestCase):
    """
    test decomposition of the magnetic representation 
    and the permutation representation by comparing the 
    characters.
    keys:
    'Gm': decomposition of the magnetic representation
    'orbits': crystallogrphic orbits
    """

    #ErMn2O5, sapce group P b a m, k = [0.5,0.,0.25], Mn at (0,0.5,0.25)
    ErMn2O5_Mn4_MagneticRepDict = dict()
    ErMn2O5_Mn4_MagneticRepDict['Gm'] = 'G=(6+0j)*G0m+(6+0j)*G0p'
    ErMn2O5_Mn4_MagneticRepDict['orbits'] = dict()
    ErMn2O5_Mn4_MagneticRepDict['orbits']['[ 0.    0.5   0.25]'] = [np.array([ 0.  ,  0.5 ,  0.25]), 
                                                                     np.array([ 0.5 ,  0.  ,  0.75]), 
                                                                     np.array([ 0.  ,  0.5 ,  0.75]), 
                                                                     np.array([ 0.5 ,  0.  ,  0.25])]

    #FeGe, space group P 21 3, k = [0.001,0.0,0.0], Fe at (0.137,0.137,0.137)
    FeGe_MagneticRepDict = dict()
    FeGe_MagneticRepDict['Gm'] = 'G=(6+0j)*G0m+(6+0j)*G0p+(6+0j)*G1m+(6+0j)*G1p'
    FeGe_MagneticRepDict['orbits'] = dict()
    FeGe_MagneticRepDict['orbits']['[ 0.137  0.137  0.137]'] = [np.array([ 0.137,  0.137,  0.137]),
                                                                np.array([ 0.363,  0.863,  0.637]), 
                                                                np.array([ 0.863,  0.637,  0.363]),
                                                                np.array([ 0.637,  0.363,  0.863])]
    FeGe_PermutationRepDict=dict()
    FeGe_PermutationRepDict['T*{2_0y0|(0.0,-0.5,0.5)}'] = 0j
    FeGe_PermutationRepDict['{2_x00|(0.5,0.5,0.0)}'] = 0j
    FeGe_PermutationRepDict['T*{2_00z|(0.5,0.0,0.5)}'] = 0j
    FeGe_PermutationRepDict['{1|(0.0,0.0,0.0)}'] = (4+0j)
    

    #MnSi, sapce group P 21 3, k = [0.05,0.05,0.05], Mn at (0.137,0.137,0.137)
    MnSi_MagneticRepDict = dict()
    MnSi_MagneticRepDict['Gm'] = 'G=(3+0j)*G0+(3+0j)*G1+(3+0j)*G2'
    MnSi_MagneticRepDict['orbits'] = dict()
    MnSi_MagneticRepDict['orbits']['[ 0.137  0.137  0.137]'] = [np.array([ 0.137,  0.137,  0.137])]
    MnSi_MagneticRepDict['orbits']['[ 0.363  0.863  0.637]'] = [np.array([ 0.363,  0.863,  0.637]), 
                                                                np.array([ 0.637,  0.363,  0.863]), 
                                                                np.array([ 0.863,  0.637,  0.363])]
    MnSi_PermutationRepDict= dict()
    MnSi_PermutationRepDict['{3^+_xxx|(0.0,0.0,0.0)}']= 0j
    MnSi_PermutationRepDict['{3^-_xxx|(0.0,0.0,0.0)}']= 0j
    MnSi_PermutationRepDict['{1|(0.0,0.0,0.0)}']= (3+0j)


    #MnP, P2/b11, k=[0.14,0.0,0.0], Mn at [0.15,0.15,0.15]
    MnP_MagneticRepDict = dict()
    MnP_MagneticRepDict['Gm'] = 'G=(6+0j)*G0m+(6+0j)*G0p+(6+0j)*G1m+(6+0j)*G1p'
    MnP_MagneticRepDict['orbits'] = dict()
    MnP_MagneticRepDict['orbits']['[ 0.15  0.15  0.15]'] = [np.array([ 0.15,  0.15,  0.15]),
                                                            np.array([ 0.85,  0.85,  0.85]), 
                                                            np.array([ 0.85,  0.65,  0.15]), 
                                                            np.array([ 0.15,  0.35,  0.85])]
                                                            
    #CeAl2, C1 2/c 1, k = [0.45,0.55,0.5], Ce at (1/8,1/8,1/8)
    CeAl2_MagneticRepDict = dict()
    CeAl2_MagneticRepDict['Gm'] = 'G=(3+0j)*G0m+(3+0j)*G0p+(3+0j)*G1m+(3+0j)*G1p'
    CeAl2_MagneticRepDict['orbits'] = dict()
    CeAl2_MagneticRepDict['orbits']['[ 0.125  0.125  0.125]'] = [np.array([ 0.125,  0.125,  0.125]), np.array([ 0.875,  0.875,  0.875])]
    
    #Jarosite
    Jarosite_MagneticRepDict = dict()    
    Jarosite_MagneticRepDict['Gm'] = 'G=(2+0j)*G1+(3+0j)*G3+(1+0j)*G5'
    Jarosite_MagneticRepDict['orbits'] = dict()
    Jarosite_MagneticRepDict['orbits']['[ 0.5  0.5  0.5]'] = [np.array([ 0.5,  0.5,  0.5]), 
                                                              np.array([ 0. ,  0.5,  0.5]), 
                                                              np.array([ 0.5,  0. ,  0.5])]
    Jarosite_PermutationRepDict= dict()
    Jarosite_PermutationRepDict['{2_xx0|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_PermutationRepDict['{2_0y0|(0.0,0.0,0.0)}'] = -1+0j  
    Jarosite_PermutationRepDict['{3^-_00z|(0.0,0.0,0.0)}']= 0j
    Jarosite_PermutationRepDict['{-3^+_00z|(0.0,0.0,0.0)}']= 0j
    Jarosite_PermutationRepDict['{m_x-xz|(0.0,0.0,0.0)}']= 1+0j
    Jarosite_PermutationRepDict['{m_x2xz|(0.0,0.0,0.0)}']= 1+0j
    Jarosite_PermutationRepDict['{2_x00|(0.0,0.0,0.0)}']= -1+0j  
    Jarosite_PermutationRepDict['{1|(0.0,0.0,0.0)}']= 3+0j
    Jarosite_PermutationRepDict['{3^+_00z|(0.0,0.0,0.0)}']= 0j     
    Jarosite_PermutationRepDict['{-3^-_00z|(0.0,0.0,0.0)}']= 0j    
    Jarosite_PermutationRepDict['{m_2xxz|(0.0,0.0,0.0)}']= 1+0j  
    Jarosite_PermutationRepDict['{-1|(0.0,0.0,0.0)}']= -3+0j    

    #HoAuGe, P 63 m c 
    HoAuGe_MagneticRepDict = dict()    
    HoAuGe_MagneticRepDict['Gm'] = 'G=(1+0j)*G0+(2+0j)*G1+(2+0j)*G2+(1+0j)*G3'
    HoAuGe_MagneticRepDict['orbits'] = dict()
    HoAuGe_MagneticRepDict['orbits']['[ 0.    0.    0.05]'] = [np.array([ 0.,  0.,  0.05]), 
                                                              np.array([ 0. ,  0.,  0.55])] 

    HoAuGe_PermutationRepDict=dict()
    HoAuGe_PermutationRepDict['{2_00z|(0.0,0.0,0.5)}'] = 0j
    HoAuGe_PermutationRepDict['{1|(0.0,0.0,0.0)}'] = 2+0j
    HoAuGe_PermutationRepDict['{m_2xxz|(0.0,0.0,0.0)}'] = 2+0j
    HoAuGe_PermutationRepDict['{m_0yz|(0.0,0.0,0.5)}'] = 0j


    #PrPtAl, Pnma, k = [0,0,kz], kz = 0.25
    PrPtAl_MagneticRepDict = dict()    
    PrPtAl_MagneticRepDict['Gm'] ='G=(2+0j)*G0m+(2+0j)*G0p+(4+0j)*G1m+(4+0j)*G1p+(2+0j)*G2m+(2+0j)*G2p+(4+0j)*G3m+(4+0j)*G3p'
    PrPtAl_MagneticRepDict['orbits'] = dict()
    PrPtAl_MagneticRepDict['orbits']['[ 0.    0.25  0.  ]'] = [np.array([ 0.  ,  0.25,  0.  ]), 
                                                               np.array([ 0.5 ,  0.25,  0.5 ]), 
                                                               np.array([ 0.  ,  0.75,  0.  ]), 
                                                               np.array([ 0.5 ,  0.75,  0.5 ])]

    PrPtAl_PermutationRepDict = dict()    
    PrPtAl_PermutationRepDict['T*{2_x00|(-0.5,-0.5,-0.5)}']= 0j
    PrPtAl_PermutationRepDict['T*{-1|(0.0,0.0,0.0)}']= 0j
    PrPtAl_PermutationRepDict['{1|(0.0,0.0,0.0)}']= 4+0j
    PrPtAl_PermutationRepDict['{m_0yz|(0.5,0.5,0.5)}']= 0j
    PrPtAl_PermutationRepDict['{2_00z|(0.5,0.0,0.5)}']= 0j
    PrPtAl_PermutationRepDict['T*{2_0y0|(0.0,-0.5,0.0)}']= 0j
    PrPtAl_PermutationRepDict['T*{m_xy0|(-0.5,0.0,-0.5)}']= 0j
    PrPtAl_PermutationRepDict['{m_x0z|(0.0,0.5,0.0)}']= 4+0j


    #Ho2BaNiO5, Ni site at (0,0,0)
    Ho2BaNiO5_MagneticRepDict_Ni = dict()
    Ho2BaNiO5_MagneticRepDict_Ni['Gm']='G=(1+0j)*G0+(2+0j)*G1'
    Ho2BaNiO5_MagneticRepDict_Ni['orbits'] = dict()
    Ho2BaNiO5_MagneticRepDict_Ni['orbits']['[ 0.  0.  0.]'] = [np.array([0, 0, 0])]    

    #Ho2BaNiO5, Ho site at (0,0,z)
    Ho2BaNiO5_MagneticRepDict_Ho = dict()
    Ho2BaNiO5_MagneticRepDict_Ho['Gm']='G=(1+0j)*G0+(2+0j)*G1+(2+0j)*G2+(1+0j)*G3'
    Ho2BaNiO5_MagneticRepDict_Ho['orbits'] = dict()
    Ho2BaNiO5_MagneticRepDict_Ho['orbits']['[ 0.5  0.   0.2]'] = [np.array([ 0.5,  0. ,  0.2]), np.array([ 0.5,  0. ,  0.8])]
    
    #U2T2X, U site at 4h position (0.1,0.6,0.5), k = [0.5,0,0.5]
    U2T2X_MagneticRepDict = dict()
    U2T2X_MagneticRepDict['Gm']='G=(1+0j)*G0+(2+0j)*G1+(1+0j)*G2+(1+0j)*G3+(1+0j)*G4+(1+0j)*G6+(1+0j)*G7+(1+0j)*G8'
    U2T2X_MagneticRepDict['orbits'] = dict()
    U2T2X_MagneticRepDict['orbits']['[ 0.1  0.6  0.5]'] = [np.array([ 0.1,  0.6,  0.5]), 
                                           np.array([ 0.4,  0.1,  0.5]), 
                                           np.array([ 0.9,  0.4,  0.5]), 
                                           np.array([ 0.6,  0.9,  0.5])]

    U2T2X_PermutationRepDict= dict()
    U2T2X_PermutationRepDict['{m_x-xz|(0.5,0.5,0.0)}'] = 2+0j
    U2T2X_PermutationRepDict['{-4^+_00z|(0.0,0.0,0.0)}']= 0j
    U2T2X_PermutationRepDict['{m_xy0|(0.0,0.0,0.0)}']= -4+0j
    U2T2X_PermutationRepDict['{-4^-_00z|(0.0,0.0,0.0)}'] = 0j  
    U2T2X_PermutationRepDict['{2_0y0|(0.5,0.5,0.0)}'] =0j
    U2T2X_PermutationRepDict['{2_x-x0|(0.5,0.5,0.0)}']= -2+0j
    U2T2X_PermutationRepDict['{m_0yz|(0.5,0.5,0.0)}']= 0j     
    U2T2X_PermutationRepDict['{1|(0.0,0.0,0.0)}']= 4+0j  
    U2T2X_PermutationRepDict['{4^+_00z|(0.0,0.0,0.0)}']= 0j
    U2T2X_PermutationRepDict['{2_x00|(0.5,0.5,0.0)}']= 0j
    U2T2X_PermutationRepDict['{2_xx0|(0.5,0.5,0.0)}']= -2+0j
    U2T2X_PermutationRepDict['{4^-_00z|(0.0,0.0,0.0)}']= 0j
    U2T2X_PermutationRepDict['{2_00z|(0.0,0.0,0.0)}']= 0j
    U2T2X_PermutationRepDict['{m_xxz|(0.5,0.5,0.0)}']= 2+0j
    U2T2X_PermutationRepDict['{m_x0z|(0.5,0.5,0.0)}']= 0j
    U2T2X_PermutationRepDict['{-1|(0.0,0.0,0.0)}']= 0j

    #CeAl2, F d -3 m:2, k = [0.45,0.55,0.5]), Ce at [0.125,0.125,0.125]
    CeAl2_MagneticRepDict = dict()
    CeAl2_MagneticRepDict['Gm']='G=(3+0j)*G0m+(3+0j)*G0p+(3+0j)*G1m+(3+0j)*G1p'
    CeAl2_MagneticRepDict['orbits'] = dict()
    CeAl2_MagneticRepDict['orbits']['[ 0.125  0.125  0.125]'] =  [np.array([ 0.125,  0.125,  0.125]), np.array([ 0.875,  0.875,  0.875])]

    #CuB2O4, I -4 2 d, k = [0,0,0], Cu at (0,0,0.5)        
    CuB2O4_MagneticRepDict_ASite = dict()
    CuB2O4_MagneticRepDict_ASite['Gm']='G=(1+0j)*G0+(1+0j)*G2+(2+0j)*G4'
    CuB2O4_MagneticRepDict_ASite['orbits'] = dict()
    CuB2O4_MagneticRepDict_ASite['orbits']['[ 0.   0.   0.5]'] =  [np.array([ 0. ,  0. ,  0.5]), np.array([ 0.  ,  0.5 ,  0.75])]
    #CuB2O4, I -4 2 d, k = [0,0,0], Cu at (0.08,0.25,0.125)        
    CuB2O4_MagneticRepDict_BSite = dict()
    CuB2O4_MagneticRepDict_BSite['Gm']='G=(2+0j)*G0+(1+0j)*G1+(1+0j)*G2+(2+0j)*G3+(3+0j)*G4'
    CuB2O4_MagneticRepDict_BSite['orbits'] = dict()
    CuB2O4_MagneticRepDict_BSite['orbits']['[ 0.08   0.25   0.125]'] = [np.array([ 0.08 ,  0.25 ,  0.125]), 
                                                                       np.array([ 0.25 ,  0.92 ,  0.875]), 
                                                                       np.array([ 0.75 ,  0.08 ,  0.875]), 
                                                                       np.array([ 0.92 ,  0.75 ,  0.125])]
    #NdFeBO, k = [0,0,1.5], Nd position at (0,0,0)
    NdFeBO_MagneticRepDict_A = dict()
    NdFeBO_MagneticRepDict_A['Gm']='G=(1+0j)*G0+(1+0j)*G2'
    NdFeBO_MagneticRepDict_A['orbits'] = dict()
    NdFeBO_MagneticRepDict_A['orbits']['[ 0.  0.  0.]'] = [np.array([ 0.,  0.,  0.])]
    #NdFeBO, k = [0,0,1.5], Fe position at (0.55,0,0)
    NdFeBO_MagneticRepDict_B = dict()
    NdFeBO_MagneticRepDict_B['Gm']='G=(3+0j)*G0+(1+0j)*G1+(2+0j)*G2'
    NdFeBO_MagneticRepDict_B['orbits'] = dict()
    NdFeBO_MagneticRepDict_B['orbits']['[ 0.55  0.    0.  ]'] = [np.array([ 0.55,  0.  ,  0.  ]), np.array([ 0.45,  0.45,  0.  ]), np.array([ 0.  ,  0.55,  0.  ])]

    #Ce2Pd2Sn, P 4/m b m, k = [0.15,0.,0.]), Ce at [0.1,0.6,0.5]
    Ce2Pd2Sn_MagneticRepDict = dict()
    Ce2Pd2Sn_MagneticRepDict['Gm']='G=(2+0j)*G0m+(2+0j)*G0p+(4+0j)*G1m+(4+0j)*G1p+(4+0j)*G2m+(4+0j)*G2p+(2+0j)*G3m+(2+0j)*G3p'
    Ce2Pd2Sn_MagneticRepDict['orbits'] = dict()
    Ce2Pd2Sn_MagneticRepDict['orbits']['[ 0.1  0.6  0.5]'] = [np.array([ 0.1,  0.6,  0.5]), np.array([ 0.4,  0.1,  0.5]), np.array([ 0.9,  0.4,  0.5]), np.array([ 0.6,  0.9,  0.5])]

    #CuO, C 1 2/c 1, k = [0.5,0.,-0.5]), Cu at [0,0,0]
    CuO_MagneticRepDict = dict()
    CuO_MagneticRepDict['Gm']='G=(3+0j)*G0m+(3+0j)*G0p+(3+0j)*G1m+(3+0j)*G1p'
    CuO_MagneticRepDict['orbits'] = dict()
    CuO_MagneticRepDict['orbits']['[ 0.   0.   0.5]'] = [np.array([ 0. ,  0. ,  0.5]), np.array([ 0.,  0.,  0.])]

    CuO_PermutationRepDict = dict()
    CuO_PermutationRepDict['{m_x0z|(0.0,0.0,0.5)}']= 0j
    CuO_PermutationRepDict['T*{-1|(0.0,0.0,0.0)}']= 0j
    CuO_PermutationRepDict['T*{2_0y0|(0.0,0.0,-0.5)}']= 0j
    CuO_PermutationRepDict['{1|(0.0,0.0,0.0)}']= 2+0j

    #P-4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    mP4_MagneticRepDict = dict()
    mP4_MagneticRepDict['Gm']='G=(1+0j)*G0m+(1+0j)*G0p+(2+0j)*G2+(2+0j)*G3'
    mP4_MagneticRepDict['orbits'] = dict()
    #python uses [0 0 0] as key and not [0. 0. 0.] here ...
    mP4_MagneticRepDict['orbits']['[0 0 0]'] = [np.array([ 0.,  0.,  0.])]


    #Mn2GeO4, P n m a, k = [0.15,0.15,0], Mn at (0,0,0)
    Mn2GeO4A_MagneticRepDict = dict()
    Mn2GeO4A_MagneticRepDict['Gm']='G=(3+0j)*G0m+(3+0j)*G0p+(3+0j)*G1m+(3+0j)*G1p'

    Mn2GeO4A_MagneticRepDict['orbits'] = dict()
    Mn2GeO4A_MagneticRepDict['orbits']['[ 0.  0.  0.]'] =  [np.array([ 0.,  0.,  0.]), np.array([ 0.5,  0. ,  0.5])]
    Mn2GeO4A_MagneticRepDict['orbits']['[ 0.5  0.5  0.5]'] = [np.array([ 0.5,  0.5,  0.5]), np.array([ 0. ,  0.5,  0. ])]

    #Mn2GeO4, P n m a, k = [0.15,0.15,0], Mn at (0.28,0.25,0.99)
    Mn2GeO4B_MagneticRepDict = dict()
    Mn2GeO4B_MagneticRepDict['Gm']='G=(6+0j)*G0m+(6+0j)*G0p+(6+0j)*G1m+(6+0j)*G1p'

    Mn2GeO4B_MagneticRepDict['orbits'] = dict()
    Mn2GeO4B_MagneticRepDict['orbits']['[ 0.28  0.25  0.99]'] = [np.array([ 0.28,  0.25,  0.99]), 
                                                                 np.array([ 0.72,  0.75,  0.01]), 
                                                                 np.array([ 0.22,  0.75,  0.49]), 
                                                                 np.array([ 0.78,  0.25,  0.51])]
                                                                                                                          
    #LiHoF4, I 4/1 a:2, k = [0.,0.,0], Ho at (0.,0.25,0.125)
    LiHoF4_MagneticRepDict = dict()
    LiHoF4_MagneticRepDict['Gm']='G=(1+0j)*G0+(1+0j)*G1+(1+0j)*G2+(1+0j)*G4+(1+0j)*G5+(1+0j)*G6'
    LiHoF4_MagneticRepDict['orbits'] = dict()
    LiHoF4_MagneticRepDict['orbits']['[ 0.     0.25   0.125]'] = [np.array([ 0.   ,  0.25 ,  0.125]), np.array([ 0.5  ,  0.25 ,  0.375])]

    #LaMnO3, Pbnm, k = 0, Mn at [0.5,0,0]
    LaMnO3_MagneticRepDict = dict()
    LaMnO3_MagneticRepDict['Gm']='G=(3+0j)*G0+(3+0j)*G2+(3+0j)*G4+(3+0j)*G6'
    LaMnO3_MagneticRepDict['orbits'] = dict()
    LaMnO3_MagneticRepDict['orbits']['[ 0.5  0.   0. ]']= [np.array([ 0.5,  0. ,  0. ]), np.array([ 0.5,  0. ,  0.5]), 
                                                           np.array([ 0. ,  0.5,  0. ]), np.array([ 0. ,  0.5,  0.5])]
    
    #test characters permutation representation
    knownpermutation = (('P 4/m b m', np.array([0.,0.,0.5]),       np.array([0.1,0.6,0.5]),       U2T2X_PermutationRepDict),
                        ('R -3 m:H',  np.array([0.,0.,1.5]),       np.array([0.5,0.,0.5]),        Jarosite_PermutationRepDict),
                        ('P 21 3',    np.array([0.05,0.05,0.05]),  np.array([0.137,0.137,0.137]), MnSi_PermutationRepDict),
                        ('P n m a',   np.array([0.,0.,0.45]),      np.array([0.,0.25,0.]),        PrPtAl_PermutationRepDict),
                        ('C 1 2/c 1', np.array([0.5,0.,-0.5]),     np.array([0.0,0.0,0.5]),       CuO_PermutationRepDict),                          
                        ('P 63 m c',  np.array([0.5,0.,0.]),       np.array([0.,0.,0.05]),        HoAuGe_PermutationRepDict),
                        ('P 21 3',    np.array([0.001,0,0]),       np.array([0.137,0.137,0.137]), FeGe_PermutationRepDict),
                        )
            
    #test of magnetic representation
    #space group, k-vector, atomic position
    knowndecomposition = (('P213',      np.array([0.05,0.05,0.05]), np.array([0.137,0.137,0.137]), MnSi_MagneticRepDict),
                          ('P2/b11',    np.array([0.14,0.0,0.0]),   np.array([0.15,0.15,0.15]), MnP_MagneticRepDict),
                          ('F d -3 m:2', np.array([0.45,0.55,0.5]), np.array([1./8.,1./8.,1./8.]), CeAl2_MagneticRepDict),
                          ('R -3 m:H',  np.array([0.,0.,1.5]),      np.array([0.5,0.5,0.5]), Jarosite_MagneticRepDict),
                          ('P 63 m c',  np.array([0.5,0.,0.]),      np.array([0.,0.,0.05]), HoAuGe_MagneticRepDict),
                          ('P m n a',   np.array([0.,0.,0.45]),     np.array([0.,0.25,0.]), PrPtAl_MagneticRepDict),
                          ('I m m m',   np.array([0.5,0.,0.5]),     np.array([0.,0.,0.]), Ho2BaNiO5_MagneticRepDict_Ni),                          
                          ('I m m m',   np.array([0.5,0.,0.5]),     np.array([0.5,0.,0.20]), Ho2BaNiO5_MagneticRepDict_Ho),                          
                          ('I m m m',   np.array([0.5,0.,0.5]),     np.array([0.5,0.,0.20]), Ho2BaNiO5_MagneticRepDict_Ho),                          
                          ('P 4/m b m', np.array([0.,0.,0.5]),      np.array([0.1,0.6,0.5]), U2T2X_MagneticRepDict),                          
                          ('I -4 2 d',  np.array([0.,0.,0.]),       np.array([0.,0.,0.5]), CuB2O4_MagneticRepDict_ASite),                          
                          ('I -4 2 d',  np.array([0.,0.,0.]),       np.array([0.08,0.25,0.125]), CuB2O4_MagneticRepDict_BSite),                          
                          ('R 32:H',    np.array([0.,0.,1.5]),      np.array([0.0,0.0,0.0]),  NdFeBO_MagneticRepDict_A),                          
                          ('R 32:H',    np.array([0.,0.,1.5]),      np.array([0.55,0.0,0.0]), NdFeBO_MagneticRepDict_B),                          
                          ('C 1 2/c 1', np.array([0.5,0.,-0.5]),    np.array([0.0,0.0,0.5]), CuO_MagneticRepDict),                          
                          ('P n m a',   np.array([0.15,0.15,0.]),   np.array([0.0,0.0,0.]), Mn2GeO4A_MagneticRepDict),                          
                          ('P n m a',   np.array([0.15,0.15,0.]),   np.array([0.28,0.25,0.99]), Mn2GeO4B_MagneticRepDict),                          
                          ('-P 4'   ,   np.array([0,0,0.15]),       np.array([0,0,0]), mP4_MagneticRepDict),
                          ('P b a m',   np.array([0.5,0,0.25]),     np.array([0,0.5,0.25]), ErMn2O5_Mn4_MagneticRepDict),
                          ('P b n m',   np.array([0,0,0]),          np.array([0.5,0,0]), LaMnO3_MagneticRepDict),
                          ('I 41/a:2',  np.array([0,0,0]),          np.array([0.,0.25,0.125]), LiHoF4_MagneticRepDict),
                          ('P 21 3',    np.array([0.001,0,0]),      np.array([0.137,0.137,0.137]), FeGe_MagneticRepDict),
                          )


    def testcharacterpermutation(self):
        """*-->test characters of permutation representation"""
        for spacegroup, kvector, atomicposition, result in self.knownpermutation:
            M=VectorRep(spacegroup,kvector,np.array(atomicposition),axialvector=True)
            orbits=M.SplitOrbits()
            M.CalcMagneticRep(orbits)
            char=M.characterpermutation
            for key, val in char.iteritems():
                self.assertEqual(result[key],val,
                '\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(spacegroup,kvector,result[key],val))
    
    
    def testdecomposition(self):
        """*-->test decomposition of magnetic representations"""
        for spacegroup, kvector, atomicposition, result in self.knowndecomposition:
            M=VectorRep(spacegroup,kvector,np.array(atomicposition),axialvector=True)
            orbits=M.SplitOrbits()
            M.CalcMagneticRep(orbits)
            self.assertEqual(result['Gm'], M.Gm[:-1],'\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(spacegroup,kvector,result['Gm'],M.Gm[:-1]))

    def testorbits(self):
        """*-->test crystallographic orbits"""
        for spacegroup, kvector, atomicposition, result in self.knowndecomposition:
            M=VectorRep(spacegroup,kvector,np.array(atomicposition),axialvector=True)
            orbits=M.SplitOrbits()
            #check number orbits is ok
            self.assertEqual(len(set(orbits.keys()) ^ set(result['orbits'].keys())),0,'\n\n for space group {0}:\n number of orbits is not correct'.format(spacegroup))
            #check atomic positions
            for key,val in orbits.iteritems():
                for i,j in enumerate(val):
                    self.assertEqual(np.allclose(j,result['orbits'][key][i]),True,
                    '\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3} in orbit with key {4} and atomic position {5}'.format(spacegroup,kvector,j,result['orbits'][key][i],key,i+1))
                    
                
if __name__ == "__main__":
    import sys
    import tempfile

    #turn off output
    print 'Test magnetic representation'
    sys.stdout = tempfile.TemporaryFile()   
    unittest.main(verbosity=2)
    sys.stdout.close()
    #now the temp file is gone
    sys.stdout = sys.__stdout__ 
                                  