#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

#import wxversion
#wxversion.ensureMinimal('3.0')

import collections
import copy
import math
import numpy as np
import wx

if wx.VERSION < ('2.8.11.0'):
    from wx.lib.pubsub import Publisher
else:
    from wx.lib.pubsub import setuparg1
    from wx.lib.pubsub import pub as Publisher

import vtk
from vtk.wx.wxVTKRenderWindowInteractor import wxVTKRenderWindowInteractor

from vpydraw_chemical_structure import StructureParametersBook
from vpydraw_magnetic_structure import MagneticParametersBook
from vpydraw_powderdifpattern import PowderDiffractionBook
from vpydraw_globals import  GlobalParameters

import os
import sys
try:
    from Lattice import Lattice
except ImportError:
    print sys.path[-1]
    sys.exit('Module Lattice was not found in system path')
try:
    from symmetry.PermutationRep import WyckoffPositions
except ImportError:
    print sys.path[-1]
    sys.exit('Module PermutationRep was not found in system path')

ID_FILE_CLOSE = wx.NewId()
ID_FILE_EXIT = wx.NewId()
ID_FILE_OPEN = wx.NewId()
ID_FILE_NEWSTRUCTURE = wx.NewId()
ID_FILE_NEWSMAGNETICTRUCTURE = wx.NewId()
ID_FILE_NEWMAGNETICPOWDERPATTERN = wx.NewId()
ID_FILE_EXPORT = wx.NewId()
ID_HELP_ABOUT = wx.NewId()
ID_OPTIONS_AXESON = wx.NewId()
ID_OPTIONS_AXESOFF = wx.NewId()
ID_OPTIONS_LABELSON = wx.NewId()
ID_OPTIONS_LABELSOFF = wx.NewId()
ID_OPTIONS_TOOLTIPSON = wx.NewId()
ID_OPTIONS_TOOLTIPSOFF = wx.NewId()
ID_OPTIONS_SETBACKGROUNDCOLOR = wx.NewId()


class MouseInteractorCellPick(vtk.vtkInteractorStyleTrackballCamera):
    """
    class return atomic coordinates of clicked atom. The job is done in 
    method leftButtonPressEvent.
    """ 
    def __init__(self):
        self.AddObserver("LeftButtonPressEvent",self.leftButtonPressEvent)
        self.transform=vtk.vtkMatrix4x4()
        self.transform.Identity()

    def SetInverseTranslateTransform(self,itf):
        """
        inverse of translation to center of mass 
        itf is vtkTransform() object
        """
        self.inversetranslate = itf
        return

    def SetMatLat2Cart(self,mat):
        """
        transformation matrix to cartesian coordinates
        """
        self.lat2cart=mat
        return

    def SetAtomsCoordinates(self, atoms):
        """
        list of all atomic coordinates on the plot window
        """
        self.atomiccoordinates=atoms
        return

    def GetDisplayToWorldCoordinates(self,x,y):
        """
        Transformation from world to screen coordinates
        """
        coordinates=np.array([x,y,0.])
        self.GetDefaultRenderer().SetDisplayPoint(coordinates)
        self.GetDefaultRenderer().DisplayToView()
        self.GetDefaultRenderer().GetViewPoint()
        self.GetDefaultRenderer().ViewToWorld()
        return self.GetDefaultRenderer().GetWorldPoint()

    def GetWorldToDisplayCoordinates(self,x,y,z):
        """
        Transformation from display to world coordinates
        """
        worldPoints=np.array([x,y,z])
        self.GetDefaultRenderer().SetWorldPoint(worldPoints)
        self.GetDefaultRenderer().WorldToView()
        self.GetDefaultRenderer().ViewToDisplay()
        return self.GetDefaultRenderer().GetDisplayPoint()

    def SetTransformMatrix(self,mat):
        """
        vtk transformation matrix that is applied after scale and rotations are done
        """   
        self.transform.DeepCopy(mat)
        return

    def leftButtonPressEvent(self,obj,event):
        """"
        return atomic position when mouse is left-click on an atom
        """
        clickPos = self.GetInteractor().GetEventPosition()
        picker = vtk.vtkCellPicker()
        picker.Pick(clickPos[0], clickPos[1], 0, self.GetDefaultRenderer())
        pid = picker.GetPointId()
        
        assemblypath = picker.GetPath()
        if (assemblypath):
            assemblypath.InitTraversal()
            assemblynode=vtk.vtkAssemblyNode()
            for i in range(0, assemblypath.GetNumberOfItems()):
                assemblynode = assemblypath.GetNextNode()
                actor = vtk.vtkLODActor().SafeDownCast(assemblynode.GetViewProp())
                if actor:
                    dlist=[]
                    algorithm = actor.GetMapper().GetInputConnection(0, 0).GetProducer()
                    glyphReference = vtk.vtkGlyph3D().SafeDownCast(algorithm)
                    if glyphReference is None:
                        return
                    p=glyphReference.GetOutput().GetPoint(pid)
                    p=self.inversetranslate.TransformFloatPoint(p[0],p[1],p[2])
                    #calculate distances between atomic positions and picked points
                    for xyz in self.atomiccoordinates:
                        x=xyz[0]
                        y=xyz[1]
                        z=xyz[2]
                        px=p[0]
                        py=p[1]
                        pz=p[2]
                        #distance between clicked coordinates and atomic position in self.atomiccoordinates
                        d=np.sqrt((x-px)*(x-px)+(y-py)*(y-py)+(z-pz)*(z-pz))
                        dlist.append(d)
                    #find minimum distance
                    index_min = np.argmin(dlist)
                    #atomic coordinates in (rlu)
                    xyz=np.around(np.dot(np.linalg.inv(self.lat2cart),self.atomiccoordinates[index_min]),3)
                    #add 0.0 to avoid negativ zero
                    x=str(xyz[0] +0.0)
                    y=str(xyz[1] +0.0)
                    z=str(xyz[2] +0.0)
                    msg='('+x +', '+ y +', '+ z +')'
                    dlg = wx.MessageDialog(None, msg, "Coordinates", wx.OK | wx.ICON_INFORMATION)
                    dlg.ShowModal()
                    dlg.Destroy()
        return

class vpydraw_main(wx.Frame, Lattice, GlobalParameters):
    def __init__(self, *args, **kwargs):
        displaySize= wx.DisplaySize()
        wx.Frame.__init__(self, parent = None, title='vpydraw', pos=(150,150), size=(displaySize[0]/2., displaySize[1]/2.))
        Lattice.__init__(self,[0.,0.,0.,0.,0.,0.])  
        GlobalParameters.__init__(self)
        self.Layout()
        self.Centre()
        self.Show(True)

        menubar = wx.MenuBar()
        fileMenu = wx.Menu()
        menubar.Append(fileMenu, '&File')
        fitem = fileMenu.Append(ID_FILE_OPEN, 'Open', 'Open')
        fitem = fileMenu.Append(ID_FILE_EXPORT, 'Export', 'Save image')
        fitem = fileMenu.Append(ID_FILE_EXIT, 'Quit', 'Quit application')

        structMenu = wx.Menu()
        menubar.Append(structMenu,'&Structure')
        sitem = structMenu.Append(ID_FILE_NEWSTRUCTURE, 'Chemical', 'Chemical Structure')
        sitem = structMenu.Append(ID_FILE_NEWSMAGNETICTRUCTURE, 'Magnetic', 'Magnetic Structure')

        simMenu = wx.Menu()
        menubar.Append(simMenu,'&Simulation')
        sitem = simMenu.Append(ID_FILE_NEWMAGNETICPOWDERPATTERN, 'Powder', 'Powder Diffraction')

        wx.ToolTip.Enable(False)
        self.optionsMenu = wx.Menu()
        #radio group
        optionsMenu=self.optionsMenu
        optionsMenu.AppendRadioItem(ID_OPTIONS_TOOLTIPSOFF, "ToolTips off")
        optionsMenu.AppendRadioItem(ID_OPTIONS_TOOLTIPSON, "ToolTips on")
        optionsMenu.AppendSeparator()
        optionsMenu.Append(ID_OPTIONS_SETBACKGROUNDCOLOR, "Renderer Color")
        optionsMenu.AppendSeparator()
        #labels group
        optionsMenu.AppendRadioItem(ID_OPTIONS_LABELSOFF, "Labels off")
        optionsMenu.AppendRadioItem(ID_OPTIONS_LABELSON, "Labels on")
        optionsMenu.AppendSeparator()
        #axes group
        optionsMenu.AppendRadioItem(ID_OPTIONS_AXESOFF, "Axes off")
        optionsMenu.AppendRadioItem(ID_OPTIONS_AXESON, "Axes on")

        #print optionsMenu.IsChecked(ID_OPTIONS_LABELSON)
        menubar.Append(optionsMenu,'&Options')

        self.SetMenuBar(menubar)

        wx.EVT_MENU(self, ID_FILE_NEWSTRUCTURE, self.OnNewStructure)
        wx.EVT_MENU(self, ID_FILE_NEWSMAGNETICTRUCTURE, self.OnNewMagneticStructure)
        wx.EVT_MENU(self, ID_FILE_NEWMAGNETICPOWDERPATTERN, self.OnNewMagneticPowderPattern)
        wx.EVT_MENU(self, ID_FILE_EXIT, self.OnQuit)
        wx.EVT_MENU(self, ID_OPTIONS_TOOLTIPSON, self.ToolTipsOn)
        wx.EVT_MENU(self, ID_OPTIONS_TOOLTIPSOFF, self.ToolTipsOff)
        wx.EVT_MENU(self, ID_OPTIONS_LABELSON, self.LabelsOn)
        wx.EVT_MENU(self, ID_OPTIONS_LABELSOFF, self.LabelsOff)
        wx.EVT_MENU(self, ID_OPTIONS_AXESON, self.AxesOn)
        wx.EVT_MENU(self, ID_OPTIONS_AXESOFF, self.AxesOff)

        wx.EVT_MENU(self, ID_OPTIONS_SETBACKGROUNDCOLOR, self.ChangeRendererBackgroundColor)
        wx.EVT_MENU(self, ID_FILE_EXPORT, self.OnExport)

        wx.EVT_CLOSE(self, self.OnQuit)
 
        self.render_window = wxVTKRenderWindowInteractor(self, wx.ID_ANY)
        #disable mouse
        #self.render_window.Disable()
        self.render_window.AddObserver("ExitEvent", lambda o,e,f=self: f.Close())
        # open a window and create a renderer
        self.renderer = vtk.vtkRenderer()
        
        self.renderer.SetBackground(0.3, 0.6, 0.3)
        self.renderer.SetViewport(0.2,0,1.,1.)  #renderer size in window
        self.render_window.GetRenderWindow().AddRenderer(self.renderer)
        #self.render_window.SetSize((6.,6.))
        self.renderer.GetActiveCamera().SetClippingRange(0.0001,1000000.)
        self.renderer.GetActiveCamera().ParallelProjectionOn()
        self.renderer.GetActiveCamera().SetParallelScale(1.)
        #to avoid clipping when parallelprojection is on
        self.renderer.GetActiveCamera().SetViewUp((0,1,0))
        self.renderer.GetActiveCamera().SetFocalPoint((0.,0.,0.))
        self.renderer.GetActiveCamera().SetPosition((0.,0.,2.))
        self.renderer.ResetCamera()
        # pick atomic position with the mouse 
        self.mousepick = MouseInteractorCellPick()
        self.mousepick.SetDefaultRenderer(self.renderer)
        self.render_window.SetInteractorStyle(self.mousepick)

        #add renderers for labels (1 renderer/atom and 1 renderer/label), max. 10 columns
        self.labelsrenderers = list()
        #max. number of different atoms set to 5
        for k in range(0,6):
            rsphere=vtk.vtkRenderer()
            rsphere.SetViewport(0.,1.-(k+1.)/10.,0.1,1.-k/10.)
            rlabel=vtk.vtkRenderer()
            rlabel.SetViewport(0.1,1.-(k+1.)/10.,0.2,1.-k/10.)
            rsphere.SetBackground(0.3, 0.6, 0.3)
            self.render_window.GetRenderWindow().AddRenderer(rsphere)
            rlabel.SetBackground(0.3, 0.6, 0.3)
            self.render_window.GetRenderWindow().AddRenderer(rlabel)
            self.labelsrenderers.append((rsphere,rlabel))
        self.axesrenderer=vtk.vtkRenderer()
        self.axesrenderer.SetBackground(0.3, 0.6, 0.3)
        self.axesrenderer.SetViewport(0.,0.,0.2,0.5)
        self.render_window.GetRenderWindow().AddRenderer(self.axesrenderer)
        self.renderer.Render()
        self.render_window.Start()

        #create the actor
        self.overallscalefactor = 1.0

        #scale and rotations
        self.plotparameters=dict()
        self.plotparameters['scale'] = self.defaultvalueswidgets['scale']
        self.plotparameters['rotx'] = self.defaultvalueswidgets['rotx']
        self.plotparameters['roty'] = self.defaultvalueswidgets['roty']
        self.plotparameters['rotz'] = self.defaultvalueswidgets['rotz']
        self.plotparameters['rotx_old'] = '0' 
        self.plotparameters['roty_old'] = '0'
        self.plotparameters['rotz_old'] = '0'
        self.plotparameters['scale_old'] = self.plotparameters['scale']
        self.transform = vtk.vtkTransform() 

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.render_window, 1, wx.EXPAND)
        self.SetSizer(sizer)
        self.SetTitle('PyModes')
        self.isploted = False

        #space group
        self.spacegroup_label=None
        #lattice parameters
        self.latticeparameters=[0.,0.,0.,0.,0.,0.]
        #atoms
        self.atoms = dict()
        self.atomglyphActordict= dict()
        #bonds
        self.chemicalbondsdict=dict()
        self.chemicalbondsactorsdict=dict()
        #spins
        self.spinsglyphActordict = dict()
        #cell parameters
        self.latticecolour=(0,0,0)
        #spins
        self.magneticmomentsdict=dict()
        #kvector
        self.kvector=None
        #list of ALL atomic coordinates in unit cell (used for mouse picking)
        self.atomiccoordinateslist=[]
        #plot labels
        self.atomlabelsdict=dict()
        #axes
        self.axes=vtk.vtkAxesActor()
        
        self.axes.GetXAxisCaptionActor2D().GetCaptionTextProperty().SetColor(0,0,0) 
        #self.axes.GetXAxisCaptionActor2D ().GetTextActor().SetTextScaleModeToNone() 
        #self.axes.GetXAxisCaptionActor2D().GetTextActor().GetScaledTextProperty() 

        self.axes.GetYAxisCaptionActor2D().GetCaptionTextProperty().SetColor(0,0,0)
        self.axes.GetZAxisCaptionActor2D().GetCaptionTextProperty().SetColor(0,0,0)
        self.axes.GetXAxisShaftProperty().SetColor(0,0,0)
        self.axes.GetYAxisShaftProperty().SetColor(0,0,0)
        self.axes.GetZAxisShaftProperty().SetColor(0,0,0)
        self.axes.GetXAxisTipProperty().SetColor(0,0,0)
        self.axes.GetYAxisTipProperty().SetColor(0,0,0)
        self.axes.GetZAxisTipProperty().SetColor(0,0,0)
        self.axes.SetXAxisLabelText("a")
        self.axes.SetYAxisLabelText("b")
        self.axes.SetZAxisLabelText("c")

        if wx.VERSION <=  ('2.8.11.0'):
            #space group
            Publisher().subscribe(self.GetSpaceGroupLabel, ("spacegroup.HM"))
            Publisher().subscribe(self.GetLatticeParameters, ("cell.parameters"))
            Publisher().subscribe(self.GetLatticeColour, ("cell.colour"))
            #atoms
            Publisher().subscribe(self.GetAtomNameAndCoordinates, ("atom.name_coordinates"))
            Publisher().subscribe(self.ChangeAtomColour, ("atom.color"))
            Publisher().subscribe(self.ChangeAtomRadius, ("atom.radius"))
            Publisher().subscribe(self.DeleteAtomFromList, ("atom.delete"))
            #chemical bonds
            Publisher().subscribe(self.GetChemicalBonds, ("atom.bonds"))
            Publisher().subscribe(self.ChangeBondColour, ("atom.bondsnew_color"))
            Publisher().subscribe(self.ChangeBondRadius, ("atom.bondsnew_radius"))
            #clear all atoms
            Publisher().subscribe(self.DeleteAllAtoms, ("atom.delete_all_atoms"))
            #scale factor and rotations
            Publisher().subscribe(self.OnPlotOptionsScroll, ("cell.scale.rotations"))
            #arrows
            Publisher().subscribe(self.GetMagneticMoments, ("spins"))
            #symmetry elements not in the little group
            Publisher().subscribe(self.GetCosetRepresentatives, ("cosetrepresentatives"))
            #propagation vector
            Publisher().subscribe(self.GetKVector, ("kvector"))
            Publisher().subscribe(self.GetSelectedMagneticIons,("selectedmagneticions"))
        else:
            #space group
            Publisher.subscribe(self.GetSpaceGroupLabel, ("spacegroup.HM"))
            Publisher.subscribe(self.GetLatticeParameters, ("cell.parameters"))
            Publisher.subscribe(self.GetLatticeColour, ("cell.colour"))
            #atoms
            Publisher.subscribe(self.GetAtomNameAndCoordinates, ("atom.name_coordinates"))
            Publisher.subscribe(self.ChangeAtomColour, ("atom.color"))
            Publisher.subscribe(self.ChangeAtomRadius, ("atom.radius"))
            Publisher.subscribe(self.DeleteAtomFromList, ("atom.delete"))
            #chemical bonds
            Publisher.subscribe(self.GetChemicalBonds, ("atom.bonds"))
            Publisher.subscribe(self.ChangeBondColour, ("atom.bondsnew_color"))
            Publisher.subscribe(self.ChangeBondRadius, ("atom.bondsnew_radius"))
            #clear all atoms
            Publisher.subscribe(self.DeleteAllAtoms, ("atom.delete_all_atoms"))
            #scale factor and rotations
            Publisher.subscribe(self.OnPlotOptionsScroll, ("cell.scale.rotations"))
            #arrows
            Publisher.subscribe(self.GetMagneticMoments, ("spins"))
            #symmetry elements not in the little group
            Publisher.subscribe(self.GetCosetRepresentatives, ("cosetrepresentatives"))
            #propagation vector
            Publisher.subscribe(self.GetKVector, ("kvector"))
            Publisher.subscribe(self.GetSelectedMagneticIons,("selectedmagneticions"))

    def AtomicPositions(self):
        """
        Calculate atomic positions in unit cell 
        and draw atoms
        """
        self.RemoveActors()
        self.DrawUnitCell()
        self.ballsdict = dict()
        try:
            wf = WyckoffPositions(spacegrouplabel=self.spacegroup_label,atomicposition=(0,0,0),centering=True)
        except AttributeError:
            dlg = wx.MessageDialog(None, 'Space group not defined!', "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return

        for atomlabel, val in self.atoms.iteritems():
            for key, coor in val.iteritems():
                if key == 'x': x=float(coor)
                if key == 'y': y=float(coor)
                if key == 'z': z=float(coor)
            wf.SetAtomicPositions(np.array([x,y,z]))
            wf.AtomicPositions()
            wf.InequivalentPositions()
            tmpdict = dict()
            tmpdict['atom'] = val['atom']
            tmpdict['color'] = val['color']
            tmpdict['radius'] = val['radius']
            tmpdict['magnetic'] = val['magnetic']
            pos = []
            for coord in wf.atomsinconventionalcell:
                pos.append(coord)
                #add atom on the border of the cell
                for x in range(-1,2):
                    for y in range(-1,2):
                        for z in range(-1,2):
                            if (x==0) and (y==0) and (z==0):
                                pass
                            else:
                                xyz=np.array([coord[0]+x,coord[1]+y,coord[2]+z])
                            if ((xyz>1).sum() == 0) and ((xyz<0).sum() == 0): 
                                pos.append(xyz)
            tmpdict['xyz']=pos
            self.ballsdict[atomlabel] = tmpdict
        #create glyphs for each atomic sort
        idx=0
        for atomlabel, atom in self.ballsdict.iteritems():
            Atomslist = []
            tmp = dict()
            for xyz in atom['xyz']:
                coord = np.dot(self.Mcryst2cart,xyz)
                Atomslist.append(coord)
            self.atomglyphActordict[atomlabel]= self.MakeSphereSourceGlyph3D(idx,Atomslist,atom['color'],atom['radius'])
            idx+=1
        self.DrawBalls()
        if self.optionsMenu.IsChecked(ID_OPTIONS_LABELSON):
            self.RemoveLabels()    #clear renderers first
            self.AddLabels() 
        else:
            pass    
        self.SendAtomsToBondsPage()
        return

    def AxesOn(self,event):
        """
        Add axes to plot
        """
        self.axesrenderer.AddActor(self.axes)
        self.axesrenderer.ResetCamera()
        self.render_window.Render()      
        return

    def AxesOff(self,event):
        """
        Remove axes
        """
        self.axesrenderer.RemoveAllViewProps()
        self.render_window.Render()
        return

    def SendAtomsToBondsPage(self):
        """
        send atomic coordinates and transformation matrix to cartesian coordinates to AtomicBonds (page3)
        """
        info=dict()
        info['atoms']=self.ballsdict
        info['crys2cart']=self.Mcryst2cart
        Publisher.sendMessage(("atomstobondspage"), info)
        return

    def CenterOfCellFilter(self):
        """
        vector to center of cell
        """
        a_cart_cell = self.a_cart_cell
        b_cart_cell = self.b_cart_cell
        c_cart_cell = self.c_cart_cell    

        #center of mass
        cmx = a_cart_cell[0]+b_cart_cell[0]+c_cart_cell[0]+a_cart_cell[0]+b_cart_cell[0]+\
                 b_cart_cell[0]+c_cart_cell[0]+a_cart_cell[0]+c_cart_cell[0]+\
                 a_cart_cell[0]+b_cart_cell[0]+c_cart_cell[0]
        cmx/=8.
        cmy = a_cart_cell[1]+b_cart_cell[1]+c_cart_cell[1]+a_cart_cell[1]+b_cart_cell[1]+\
                 b_cart_cell[1]+c_cart_cell[1]+a_cart_cell[1]+c_cart_cell[1]+\
                 a_cart_cell[1]+b_cart_cell[1]+c_cart_cell[1]
        cmy/=8.
        cmz = a_cart_cell[2]+b_cart_cell[2]+c_cart_cell[2]+a_cart_cell[2]+b_cart_cell[2]+\
                 b_cart_cell[2]+c_cart_cell[2]+a_cart_cell[2]+c_cart_cell[2]+\
                 a_cart_cell[2]+b_cart_cell[2]+c_cart_cell[2]
        cmz/=8.

        #apply translation so that center of mass is in the center of the screen
        translateFilter = vtk.vtkTransformPolyDataFilter()
        translate = vtk.vtkTransform()
        #translate.PostMultiply()
        translate.Translate(-cmx,-cmy,-cmz)
        translateFilter.SetTransform(translate)
        #save inverse for use with point picking
        inversetranslate=vtk.vtkTransform()
        inversetranslate=translate.GetInverse()
        return translateFilter, inversetranslate

    def ChangeAtomRadius(self,msg):
        """
        Get atomic radius
        """
        self.atoms = msg.data
        #change radius
        for label, actor in self.atomglyphActordict.iteritems():
           algorithm = actor.GetMapper().GetInputConnection(0, 0).GetProducer()
           glyphReference = vtk.vtkGlyph3D().SafeDownCast(algorithm)
           #glyphReference.GetInputConnection(0,0).GetProducer()
           SphereSource = vtk.vtkSphereSource()
           radius = self.atoms[label]['radius']
           SphereSource.SetRadius(radius)
           SphereSource.SetPhiResolution(20)
           SphereSource.SetThetaResolution(20)
           glyphReference.SetSource(SphereSource.GetOutput())
           self.ballsdict[label]['radius']=radius

        transform = vtk.vtkTransform()
        scale=radius/self.atoms[label]['radius_default']
        transform.Scale(scale,scale,scale)
        transformFilter=vtk.vtkTransformFilter()
        for label, actor in self.spinsglyphActordict.iteritems():
            algorithm = actor.GetMapper().GetInputConnection(0, 0).GetProducer()
            glyphReference = vtk.vtkGlyph3D().SafeDownCast(algorithm)
            ArrowSource = vtk.vtkArrowSource()
            transformFilter.SetInputConnection(ArrowSource.GetOutputPort());
            transformFilter.SetTransform(transform)
            glyphReference.SetSource(transformFilter.GetOutput())
        #self.renderer.AddActor(self.UnitCellActorAssembly)
        self.UpdateRenderer()

        if self.optionsMenu.IsChecked(ID_OPTIONS_LABELSON):
            self.RemoveLabels()
            self.AddLabels()
        return

    def ChangeAtomColour(self,msg):
        """
        Get atom color from color picker and update plot
        """
        self.atoms = msg.data
        #change colour
        for label, actor in self.atomglyphActordict.iteritems():
           color = list(msg.data[label]['color'])
           actor.GetProperty().SetColor(color[0]/255.,color[1]/255.,color[2]/255.)
           self.ballsdict[label]['color']=(color[0],color[1],color[2])

        for label, actor in self.spinsglyphActordict.iteritems():
           idx = label.find('_')
           label_at=label[:idx]
           color = list(msg.data[label_at]['color'])
           actor.GetProperty().SetColor(color[0]/255.,color[1]/255.,color[2]/255.)

        self.UpdateRenderer()

        if self.optionsMenu.IsChecked(ID_OPTIONS_LABELSON):
            self.RemoveLabels()
            self.AddLabels()
        return

    def ChangeBondColour(self,msg):
        """
        Get bond color from color picker and update plot
        """
        for bondlalel, actor in self.chemicalbondsactorsdict.iteritems():
            self.UnitCellActorAssembly.RemovePart(actor)
        self.UpdateRenderer()
        self.chemicalbondsdict=msg.data
        self.MakeChemicalBonds()
        self.DrawChemicalBonds()
        return

    def ChangeRendererBackgroundColor(self,event):
        """
        Choose and set background color
        """
        dialog = wx.ColourDialog(None)
        dialog.GetColourData().SetChooseFull(True)
        if dialog.ShowModal() == wx.ID_OK:
            data = dialog.GetColourData()
            rgb=list(data.GetColour().Get())
            rgb[0]/=255.
            rgb[1]/=255.
            rgb[2]/=255.
            self.renderer.SetBackground((rgb[0],rgb[1],rgb[2]))
            for k in range(0,len(self.labelsrenderers)):
                self.labelsrenderers[k][0].SetBackground((rgb[0],rgb[1],rgb[2]))
                self.labelsrenderers[k][1].SetBackground((rgb[0],rgb[1],rgb[2]))
            self.axesrenderer.SetBackground((rgb[0],rgb[1],rgb[2]))
        dialog.Destroy()
        return

    def ChangeBondRadius(self,msg):
        """
        Get bond radius from slider and update plot
        """
        for bondlalel, actor in self.chemicalbondsactorsdict.iteritems():
            self.UnitCellActorAssembly.RemovePart(actor)
        self.UpdateRenderer()
        self.chemicalbondsdict=msg.data
        self.MakeChemicalBonds()
        self.DrawChemicalBonds()
        return

    def DeleteAtomFromList(self, msg):
        """
        Delete one atom and redraw
        """
        self.atoms = msg.data
        #delete atom from ballsdict 
        tmpdict=dict()
        for label in self.ballsdict.keys():
            if label in self.atoms.keys():
                tmpdict[label]=self.ballsdict[label]
            else:
                pass
        self.ballsdict={}
        self.ballsdict=tmpdict.copy()
        #update labels in chemical bonds page
        self.SendAtomsToBondsPage()

        #delete atom from figure
        tmpdict=copy.copy(self.atomglyphActordict)
        for label, actor in tmpdict.iteritems():
            if label in self.atoms.keys():
                pass
            else:
                try:
                    self.UnitCellActorAssembly.RemovePart(self.atomglyphActordict[label])
                    del self.atomglyphActordict[label]
                    for key in self.spinsglyphActordict.keys():
                        if key.find(label) != -1:
                            self.UnitCellActorAssembly.RemovePart(self.spinsglyphActordict[key])
                            del self.spinsglyphActordict[key]
                except:
                    print 'could not delete atom'
        self.UpdateRenderer()
        del tmpdict
        #update labels
        if self.optionsMenu.IsChecked(ID_OPTIONS_LABELSON):
            self.RemoveLabels()
            self.AddLabels()
        return

    def DeleteAllAtoms(self, msg):
        """
        remove all atoms from plot when clear button is clicked
        """
        for k,v in self.atomglyphActordict.iteritems():
            self.UnitCellActorAssembly.RemovePart(v)
            #self.renderer.RemoveActor(v) 
        del self.atomglyphActordict
        self.atomglyphActordict = dict()
        for k,v in self.spinsglyphActordict.iteritems():
            self.UnitCellActorAssembly.RemovePart(v)
            #self.renderer.RemoveActor(v) 
        del self.spinsglyphActordict
        self.spinsglyphActordict = dict()
        self.UpdateRenderer()
        self.atommiccoordinateslist=[]
        #remove also labels
        if self.optionsMenu.IsChecked(ID_OPTIONS_LABELSON):
            self.RemoveLabels()
        return

    def DrawArrows(self):
        for label, actor in self.spinsglyphActordict.iteritems():
            scale = self.overallscalefactor
            actor.SetScale(scale,scale,scale)
            self.UnitCellActorAssembly.AddPart(actor)
        self.renderer.AddActor(self.UnitCellActorAssembly)
        self.UpdateRenderer()
        return

    def DrawBalls(self):
        for label, actor in self.atomglyphActordict.iteritems():
            scale = self.overallscalefactor
            actor.SetScale(scale,scale,scale)
            self.UnitCellActorAssembly.AddPart(actor)
        self.renderer.AddActor(self.UnitCellActorAssembly)
        self.UpdateRenderer()
        return

    def DrawChemicalBonds(self):
        """
        add bond actors to assembly and redraw
        """
        for bondlabel, bondactor in self.chemicalbondsactorsdict.iteritems():
            self.UnitCellActorAssembly.AddPart(bondactor)
        #self.renderer.AddActor(self.UnitCellActorAssembly)
        self.UpdateRenderer()
        return

    def DrawUnitCell(self):
        """
        Plot unit cell
        """
        if self.isploted == True:
            self.RemoveActors()
        try:
            a_cart_cell = self.a_cart_cell
            b_cart_cell = self.b_cart_cell
            c_cart_cell = self.c_cart_cell
        except AttributeError:
            dlg = wx.MessageDialog(None, "Lattice not defined!", "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return
        points = vtk.vtkPoints()
        points.SetNumberOfPoints(8)
       
        points.SetPoint(0,0.,0.,0.)
        tmp = a_cart_cell
        points.SetPoint(1,tmp[0],tmp[1],tmp[2])
        tmp = b_cart_cell
        points.SetPoint(2,tmp[0],tmp[1],tmp[2])       
        tmp = c_cart_cell
        points.SetPoint(3,tmp[0],tmp[1],tmp[2])       
        tmp = a_cart_cell+b_cart_cell
        points.SetPoint(4,tmp[0],tmp[1],tmp[2])
        tmp = b_cart_cell+c_cart_cell
        points.SetPoint(5,tmp[0],tmp[1],tmp[2])
        tmp = a_cart_cell+c_cart_cell
        points.SetPoint(6,tmp[0],tmp[1],tmp[2])
        tmp = a_cart_cell+b_cart_cell+c_cart_cell
        points.SetPoint(7,tmp[0],tmp[1],tmp[2])

        lines = vtk.vtkCellArray()
        lines.InsertNextCell(2)
        lines.InsertCellPoint(0)
        lines.InsertCellPoint(1)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(0)
        lines.InsertCellPoint(2) 

        lines.InsertNextCell(2)
        lines.InsertCellPoint(1)
        lines.InsertCellPoint(4)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(2)
        lines.InsertCellPoint(4)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(0)
        lines.InsertCellPoint(3)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(1)
        lines.InsertCellPoint(6)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(4)
        lines.InsertCellPoint(7)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(2)
        lines.InsertCellPoint(5)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(3)
        lines.InsertCellPoint(6) 

        lines.InsertNextCell(2)
        lines.InsertCellPoint(3) 
        lines.InsertCellPoint(5)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(6)
        lines.InsertCellPoint(7)

        lines.InsertNextCell(2)
        lines.InsertCellPoint(5)
        lines.InsertCellPoint(7)

        unitcell = vtk.vtkPolyData()
        unitcell.SetPoints(points)
        unitcell.SetLines(lines)
        
        #shift the cell to center of screen
        #get inverse of translation as vtk.vtktransform()
        translateFilter, inversetranslate = self.CenterOfCellFilter()
        if vtk.VTK_MAJOR_VERSION <= 5:
            translateFilter.SetInputConnection(unitcell.GetProducerPort())
        else:
            translateFilter.SetInputData(unitcell)
        translateFilter.Update()
        unitcell = translateFilter.GetOutput()
        unitcellMapper = vtk.vtkPolyDataMapper()
        if vtk.VTK_MAJOR_VERSION <= 5:
            unitcellMapper.SetInputConnection(unitcell.GetProducerPort())
        else:
            unitcellMapper.SetInputData(unitcell)
        unitcellMapper.ScalarVisibilityOff()
        unitcellMapper.SetColorModeToMapScalars()
        unitcellMapper.SetColorMode(2)
        unitcellMapper.SetScalarRange(0,1.0)
        unitcellMapper.Update()
        self.unitcellActor=vtk.vtkActor()
        self.unitcellActor.SetMapper(unitcellMapper)
        self.unitcellActor.GetProperty().SetLineWidth(2)
        #set color
        self.unitcellActor.GetProperty().SetColor(self.latticecolour)
        scale = self.overallscalefactor
        self.unitcellActor.SetScale(scale,scale,scale)
        self.UnitCellActorAssembly.AddPart(self.unitcellActor)
        self.renderer.AddActor(self.UnitCellActorAssembly)

        self.RotateAndScalePlot()
        self.UpdateRenderer()
        self.isploted = True
        return

    def GetAtomNameAndCoordinates(self,msg):
        """
        Get atom label and coordinates
        """
        self.atoms = msg.data
        self.AtomicPositions()
        self.RotateAndScalePlot()
        return

    def GetChemicalBonds(self,msg):
        """
        get bonds list and properties 
        """
        #delete old ones first as method called of deleting a bond
        for bondlabel, actor in self.chemicalbondsactorsdict.iteritems():
            self.UnitCellActorAssembly.RemovePart(actor)
        self.chemicalbondsactorsdict={}
        self.UpdateRenderer()
        self.chemicalbondsdict=msg.data
        self.MakeChemicalBonds()
        self.DrawChemicalBonds()
        return

    def GetCosetRepresentatives(self,msg):
        """
        Get symmetry elements in space group that are not 
        in the little group
        """
        self.cosetreps=msg.data        
        return

    def GetKVector(self,msg):
        self.kvector=msg.data
        return

    def GetLatticeColour(self,msg):
        """
        unit cell color
        """
        colour=msg.data
        r=list(colour)[0]/255.
        g=list(colour)[1]/255.
        b=list(colour)[2]/255.
        self.latticecolour=(r,g,b)
        #in case the colour of the lattice is changed before the lattice parameters are defined
        try:
            self.unitcellActor.GetProperty().SetColor(r,g,b)
        except AttributeError:
            pass 
        self.UpdateRenderer()
        return

    def GetLatticeParameters(self,msg):
        """
        Read lattice parameters from render_window and call drawing function
        """
        self.latticeparameters = msg.data
        self.overallscalefactor=1.5/max(self.latticeparameters[0:3])
        if 0 in self.latticeparameters:
            pass
        else:
            self.TransformLatticeToCartesianCoordinates()
            self.CenterOfCellFilter()
            self.DrawUnitCell()
        return
 
    def GetSelectedMagneticIons(self,msg):
        """
        Get selected magnetic atoms and set selected label of formfactor in dictionary 
        """
        for label, val in msg.data.iteritems():
            self.atoms[label]['FQ']=val['FQ']
        return

    def GetMagneticMoments(self,msg):
        """
        Get list of magnetic moments when button 'Draw' is clicked
        """
        for label in self.spinsglyphActordict.iterkeys():
            self.UnitCellActorAssembly.RemovePart(self.spinsglyphActordict[label])
        self.renderer.Modified()
        self.UpdateRenderer()
        k=self.spinsglyphActordict.keys()
        for label in k:
            del self.spinsglyphActordict[label]
        self.magneticmomentsdict=msg.data[1]
        for label,atoms in self.magneticmomentsdict.iteritems():
            spins=[]
            idx=label.find('_')
            atomlabel = label[:idx]
            color=self.ballsdict[atomlabel]['color']
            for xyz,spindir in atoms.iteritems():
                tmp=dict()
                tmp['xyz']=xyz
                tmp['spin']=spindir
                spins.append(tmp)
            self.spinsglyphActordict[label]=self.MakeArrowSourceGlyph3D(spins,color)
        self.DrawArrows()
        return

    def GetSpaceGroupLabel(self,msg):
        """
        Get space group label and coordinates
        """
        if self.isploted == True: self.RemoveActors()
        self.spacegroup_label = msg.data
        return

    def LabelsOn(self,event):
        self.AddLabels()

    def AddLabels(self, fontsize=1):
        """
        Add atomic labels
        """
        mappers = list()
        actors = list()
        textmappers = list()
        textactors = list()

        # Create a common text property.
        textProperty = vtk.vtkTextProperty()
        textProperty.SetFontSize(fontsize*20)
        textProperty.SetColor((0.,0.,0.))
        #textProperty.SetJustificationToCentered()
        try:
            self.ballsdict
        except AttributeError:
            return
        else:
            pass

        for key, val in self.ballsdict.iteritems():
            source = vtk.vtkSphereSource()
            source.SetCenter(0,0,0)
            source.SetRadius(val['radius']/6.)
            source.SetPhiResolution(20)
            source.SetThetaResolution(20)
            source.Update()
            mapper=vtk.vtkPolyDataMapper()

            if vtk.VTK_MAJOR_VERSION <= 5:
                mapper.SetInput(source.GetOutput())
            else:
                mapper.SetInputConnection(source.GetOutputPort())
            mappers.append(mapper)
            
            actor=vtk.vtkActor()
            rgb=(val['color'][0]/255.,val['color'][1]/255.,val['color'][2]/255.)
            actor.GetProperty().SetColor(rgb)
            actor.SetMapper(mapper)
            actors.append(actor)

            textmapper=vtk.vtkTextMapper()
            textmapper.SetInput(key)
            textmapper.SetTextProperty(textProperty)
            textmappers.append(textmapper)

            textactor=vtk.vtkActor2D() 
            textactor.SetMapper(textmapper)
            textactor.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
            textactor.GetPosition2Coordinate().SetCoordinateSystemToNormalizedViewport()
            textactor.SetPosition(0.05,0.4);
            textactor.SetPosition2(0.75,0.75);

            textactor.GetProperty().SetColor((1.,1.,1.))
            textactors.append(textactor)

        #Add everything to renderers
        for idx in range(0,len(mappers)):
                self.labelsrenderers[idx][0].AddActor(actors[idx])
                self.labelsrenderers[idx][1].AddActor(textactors[idx])
                if idx > 4:  #max. sub-renderers is 5
                    break 
        self.render_window.Render()                  
        return

    def LabelsOff(self,event):
        self.RemoveLabels()

    def RemoveLabels(self):
        for k in range(0,len(self.labelsrenderers)):
            self.labelsrenderers[k][0].RemoveAllViewProps()
            self.labelsrenderers[k][1].RemoveAllViewProps()
        self.render_window.Render()
        return

    def MakeArrowSourceGlyph3D(self,spinslist,color):
        """
        Add spins to atoms. The default color is the one used for the ball
        """
        points = vtk.vtkPoints()
        for spin in spinslist:
            coord = np.dot(self.Mcryst2cart,spin['xyz'])
            points.InsertNextPoint(coord)

        polydata = vtk.vtkPolyData()
        polydata.SetPoints(points)

        #shift the atoms with origin at to center of screen
        translateFilter, inversetransform = self.CenterOfCellFilter()
        if vtk.VTK_MAJOR_VERSION <= 5:
            translateFilter.SetInputConnection(polydata.GetProducerPort())
        else:
            translateFilter.SetInputData(polydata)
        translateFilter.Update()
        polydata = translateFilter.GetOutput()
        spinsArray = vtk.vtkDoubleArray()
        spinsArray.SetNumberOfComponents(3) 
        spinsArray.SetNumberOfTuples(polydata.GetNumberOfPoints())

        spindir=[]
        spinnorm=[]
        for spinvec in spinslist:
            s = np.dot(self.Mcryst2cart,np.array(spinvec['spin'],dtype='float'))
            spindir.append(s)
            spinnorm.append(np.linalg.norm(s))

        v=max(spinnorm)
        for k in range(0,len(spindir)):
            #spindir[k]/=v 
            spinsArray.SetTuple(k,tuple(spindir[k]))
        polydata.GetPointData().SetVectors(spinsArray)
        polydata.Update()

        glyph3D = vtk.vtkGlyph3D()
        Arrow = vtk.vtkArrowSource()
        glyph3D.SetSource(Arrow.GetOutput())
        glyph3D.SetInput(polydata)

        glyph3D.ScalingOn()
        glyph3D.SetScaleModeToScaleByVector()
        glyph3D.Update()

        mapper = vtk.vtkPolyDataMapper()
        mapper.ScalarVisibilityOff()
        mapper.SetScalarRange(0,1.0)
        mapper.SetInputConnection(glyph3D.GetOutputPort())
        mapper.Update()

        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        rgb=(color[0]/255.,color[1]/255.,color[2]/255.)
        actor.GetProperty().SetColor(rgb)
        return actor
 
    def MakeChemicalBonds(self):
        """
        add chemical bonds
        """
        for bondid, bondsdict in self.chemicalbondsdict.iteritems():
                points = vtk.vtkPoints()
                #colors = vtk.vtkUnsignedCharArray()
                #colors.SetNumberOfComponents(3)
                rgb=bondsdict['color']
                rgb=np.array(rgb)/255.
                idx=0
                for bonddict in bondsdict['bonds']:
                    points.InsertPoint(idx,bonddict['from'][0],bonddict['from'][1],bonddict['from'][2])
                    points.InsertPoint(idx+1,bonddict['to'][0],bonddict['to'][1],bonddict['to'][2])
                    #if vtk.VTK_MAJOR_VERSION < 7:
                    #    colors.InsertNextTupleValue(rgb)
                    #else:
                    #    colors.InsertNextTypedTupleValue(rgb)
                    idx=idx+2
                idx=0
                lines=vtk.vtkCellArray()
                for bonddict in bondsdict['bonds']:
                    line = vtk.vtkLine()
                    line.GetPointIds().SetId(0,idx)
                    line.GetPointIds().SetId(1,idx+1)
                    lines.InsertNextCell(line)
                    idx=idx+2
   
                #Create a polydata to store everything
                polydata = vtk.vtkPolyData()
                #Add the points to the dataset
                polydata.SetPoints(points)
                #Add the lines to the dataset
                polydata.SetLines(lines)
                #polydata.GetCellData().SetScalars(colors)
                #polydata.Update()

                translateFilter, inversetransform = self.CenterOfCellFilter()
                if vtk.VTK_MAJOR_VERSION <= 5:
                    translateFilter.SetInputConnection(polydata.GetProducerPort())
                else:
                    translateFilter.SetInputData(polydata)
                translateFilter.Update()
                polydata = translateFilter.GetOutput()

                tubes = vtk.vtkTubeFilter()
                tubes.SetInputConnection(polydata.GetProducerPort())
                radius=bondsdict['radius']
                tubes.SetRadius(radius)
                tubes.SetVaryRadius(0);
                tubes.SetRadiusFactor(10);
                tubes.SetNumberOfSides(6)
                tubes.Update()
    
                #Visualize
                mapper = vtk.vtkPolyDataMapper()
                #if vtk.VTK_MAJOR_VERSION <= 5:
                #   mapper.SetInput(polydata)
                #else:
                #    mapper.SetInputData(polydata)

                mapper.SetInputConnection(tubes.GetOutputPort())
                #mapper.ScalarVisibilityOff()

                bondsactor = vtk.vtkLODActor()
                bondsactor.SetMapper(mapper)
                #bondsactor.GetProperty().SetOpacity(0.2)
                #bondsactor.GetProperty().SetRepresentationToSurface()
                #bondsactor.GetProperty().SetInterpolationToGouraud()
                bondsactor.GetProperty().SetColor(rgb)
                #bondsactor.GetProperty().SetAmbient(0.15);
                #bondsactor.GetProperty().SetDiffuse(0.85);
                #bondsactor.GetProperty().SetSpecular(0.1);
                #bondsactor.GetProperty().SetSpecularPower(30);
                #bondsactor.GetProperty().SetSpecularColor(1, 1, 1);
                #bondsactor.GetProperty().SetDiffuseColor(1.0000, 0.8941, 0.70981);
                scale = self.overallscalefactor
                bondsactor.SetScale(scale,scale,scale)
                self.chemicalbondsactorsdict[bondid]=bondsactor
        return 

    def MakeSphereSourceGlyph3D(self,idxglyph,coordinates=None,rgb=(0,0,0),radius=0.0):
        """
        enter coordinates into a vtkPoints object and produces a vtkGlyph3D 
        using vtkSphere as a source
        returns a vtkActor
        input: list of coordinates e.g. [(0,0,0),(1,0,0),...], color, radius
        """
        rgb=np.array(rgb)/255. 
        points = vtk.vtkPoints()
        colors = vtk.vtkUnsignedCharArray()
        colors.SetName("Colors")
        colors.SetNumberOfComponents(3)
        idp=0
        for xyz in coordinates:
            points.InsertPoint(idp,xyz[0],xyz[1],xyz[2])
            if vtk.VTK_MAJOR_VERSION < 7:
                colors.InsertNextTupleValue(rgb)
            else:
                colors.InsertNextTypedTupleValue(rgb)
            idp=idp+1
        self.atomiccoordinateslist+=coordinates
        self.mousepick.SetAtomsCoordinates(self.atomiccoordinateslist)            
        polydata = vtk.vtkPolyData()
        polydata.SetPoints(points)
        polydata.GetPointData().SetScalars(colors)
        polydata.Update()
        #polydata.Modified()
        #polydata.Update()
        #shift the atoms with origin at to center of screen
        translateFilter, inversetransform = self.CenterOfCellFilter()
        #mouse picking
        self.mousepick.SetInverseTranslateTransform(inversetransform)

        if vtk.VTK_MAJOR_VERSION <= 5:
            translateFilter.SetInputConnection(polydata.GetProducerPort())
        else:
            translateFilter.SetInputData(polydata)
        translateFilter.Update()
        polydata = translateFilter.GetOutput()

        SphereSource = vtk.vtkSphereSource()
        SphereSource.SetRadius(radius)
        SphereSource.SetPhiResolution(20)
        SphereSource.SetThetaResolution(20)
        SphereSource.Update()

        glyph3D = vtk.vtkGlyph3D()
        #needed to identify clicked sphere!!
        glyph3D.GeneratePointIdsOn()
        glyph3D.SetColorModeToColorByScalar()

        if vtk.VTK_MAJOR_VERSION <= 5:
            glyph3D.SetSource(SphereSource.GetOutput())
            glyph3D.SetInput(polydata)
        else:
            glyph3D.SetSourceConnection(SphereSource.GetOutputPort())
            glyph3D.SetInputData(polydata)

        glyph3D.ScalingOn()
        glyph3D.SetVectorModeToUseNormal()   
        glyph3D.SetScaleModeToScaleByVector() 
        glyph3D.Update()                

        mapper = vtk.vtkPolyDataMapper()
        if vtk.VTK_MAJOR_VERSION <= 5:
            mapper.SetInputConnection(glyph3D.GetOutputPort())
        else:
            mapper.SetInputData(glyph3D)
        mapper.ImmediateModeRenderingOn()
        mapper.ScalarVisibilityOff()
        mapper.SetColorModeToMapScalars()
        mapper.SetColorMode(2)
        mapper.SetScalarRange(0,1.0)

        actor = vtk.vtkLODActor()
        actor.SetMapper(mapper)
        actor.GetProperty().SetRepresentationToSurface()
        actor.GetProperty().SetInterpolationToGouraud()
        actor.GetProperty().SetColor(rgb)
        actor.SetNumberOfCloudPoints(30000)
        return actor

    def OnNewMagneticStructure(self, event):
        """
        open magnetic structrure parameters widget
        """
        try: 
           self.mst.Destroy()
        except:
           pass
        finally:
           self.mst=MagneticParametersBook(mcrys2cart = self.Mcryst2cart, mrec2cart = self.Mrec2lat, spacegroup = self.spacegroup_label, atomicpositions = self.atoms)
        return

    def OnNewMagneticPowderPattern(self, event):
        """
        open powder diffraction pattern widget (magnetic atoms only) 
        """
        try: 
            self.simpow.Destroy()
        except:
            pass
        finally:
            if self.kvector == None or not self.magneticmomentsdict:
                pass
            else:
                self.simpow=PowderDiffractionBook(lattice = self.latticeparameters, atomicpositions = self.atoms,
                                                  kvector = self.kvector, magmomdict=self.magneticmomentsdict)
        return

    def OnNewStructure(self, event):
        """
        open chemical structure parameters widget
        """
        try: 
           self.st.Destroy()
        except:
           pass
        finally:
           self.RemoveActors()
           self.st=StructureParametersBook()
        return

    def OnPlotOptionsScroll(self,msg):
        """
        Get values for scale, rotations around x,y,z (world coordinates)
        and replot
        """
        self.plotparameters = msg.data
        self.RotateAndScalePlot()
        return

    def OnQuit(self, e):
        """
        Quit program
        """
        try:
           self.st.Destroy()
        except AttributeError:
           pass
        try:
           self.mst.Destroy()
        except AttributeError:
           pass
        try:
           self.simpow.Destroy()
        except AttributeError:
           pass
        self.Destroy()       
        return

    def OnExport(self, event):
        """
        save renderer to file
        """
        exportFileDialog = wx.FileDialog(self, "Export image", "", "",
                                   "png files (*.png)|*.png", wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)

        if exportFileDialog.ShowModal() == wx.ID_CANCEL:
            return     # the user changed idea...

        # screenshot code:
        w2if = vtk.vtkWindowToImageFilter()
        w2if.SetInput(self.render_window.GetRenderWindow())
        self.RemoveLabels()  #have to change font size of labels befor printing and then reset back after file is written
        magnification = 3
        self.AddLabels(magnification)
        w2if.SetMagnification(magnification)                   #set the resolution of the output image (3 times the current resolution of vtk render window)
        w2if.SetInputBufferTypeToRGBA() #also record the alpha (transparency) channel
        w2if.ReadFrontBufferOff()                #read from the back buffer
        w2if.Update();

        #writer=vtk.vtkPNGWriter()
        #file = exportFileDialog.GetPath()
        #try:
        #    file = file[0:file.index('.')]
        #except ValueError:
        #    pass
        #writer.SetFileName(file+'.png')
        #writer.SetInputConnection(renderLarge.GetOutputPort())
        #writer.SetInputConnection(w2if.GetOutputPort())
        #writer.Write()

        #w2i = vtk.vtkWindowToImageFilter()
        #w2i.SetMagnification(3)
        #w2i.SetInput(self.render_window.GetRenderWindow())
        #w2i.SetInput(self.renderer.GetRenderWindow())
        #w2i.FixBoundaryOn()
        #w2i.Update()
        #self.render_window.Render()
        writer = vtk.vtkPNGWriter()
        writer.SetInputConnection(w2if.GetOutputPort())
        file = exportFileDialog.GetPath()
        try:
            file = file[0:file.index('.')]
        except ValueError:
            pass
        writer.SetFileName(file+'.png')
        writer.Write()
        self.RemoveLabels()
        self.AddLabels(1)
        return

    def RemoveActors(self):
        """
        Remove all objects from plot
        """
        self.renderer.RemoveAllViewProps()
        self.UpdateRenderer()
        #delete old actors
        try:
            del self.UnitCellActorAssembly
        except AttributeError:
            pass
        #create a new one
        self.UnitCellActorAssembly = vtk.vtkAssembly()
        del self.atomglyphActordict
        self.atomglyphActordict = dict()
        self.spacegrouplabel=None
        self.isploted = False
        return

    def RotateAndScalePlot(self):
        """
        scale/rotate cell
        """
        plotpar = self.plotparameters
        transform = vtk.vtkTransform()
        s=float(plotpar['scale'])/100.
        transform.Scale(s,s,s)
        angle_x = float(plotpar['rotx'])
        transform.RotateWXYZ(angle_x,1,0,0)
        angle_y = float(plotpar['roty'])
        transform.RotateWXYZ(angle_y,0,1,0)
        angle_z = float(plotpar['rotz'])
        transform.RotateWXYZ(angle_z,0,0,1)

        self.mousepick.SetTransformMatrix(transform.GetMatrix())
        self.UnitCellActorAssembly.SetUserTransform(transform)
        self.UnitCellActorAssembly.Modified()

        transformaxes = vtk.vtkTransform()
        transformaxes.RotateWXYZ(angle_x,1,0,0)
        transformaxes.RotateWXYZ(angle_y,0,1,0)
        transformaxes.RotateWXYZ(angle_z,0,0,1)
        self.axes.SetUserTransform(transformaxes)    
           
        self.UpdateRenderer()
        return

    def ToolTipsOn(self, event):
        wx.ToolTip.Enable(True)
        return

    def ToolTipsOff(self, event):
        wx.ToolTip.Enable(False)
        return

    def TransformLatticeToCartesianCoordinates(self):
        """
        Transform lattice parameters to cartesian parameters
        """
        self.SetLattice(self.latticeparameters)
        self.Mcryst2cart=self.lat2cart()   #transformation matrix to cartesian coordinates (real space)
        self.Mrec2lat = self.recilat2cart()   #tranformation matrix to cartesian coordinates (reciprocal lattice)
        self.mousepick.SetMatLat2Cart(self.Mcryst2cart)
        self.a_cart_cell=np.dot(self.Mcryst2cart,np.array([1., 0., 0. ]));
        self.b_cart_cell=np.dot(self.Mcryst2cart,np.array([0., 1., 0. ])); 
        self.c_cart_cell=np.dot(self.Mcryst2cart,np.array([0., 0., 1. ])); 
        return

    def UpdateRenderer(self):
        """
        Update plot when change has occured
        """
        self.renderer.Modified()       
        self.render_window.Update()
        self.render_window.Render()
        return

def main(): 
    ex = wx.App()
    vpydraw_main()
    ex.MainLoop()    


if __name__ == '__main__':
    main()


